#include "pch.h"
#include "BaseAntTweakBar.h"
#include "AntTweakBar.h"
#include "GSky.h"
#include "GothicAPI.h"
#include "Engine.h"
#include "zCMaterial.h"
#include "BaseGraphicsEngine.h"
#include <algorithm>

#pragma comment(lib, "AntTweakBar.lib")

BaseAntTweakBar::BaseAntTweakBar(void)
{
	IsActive = false;
	Bar_Sky = NULL;
	TS_Active = false;
	TS_TextureName[0] = 0;
	ActiveMaterialInfo = NULL;
	LastFrameActiveMaterialInfo = NULL;
}


BaseAntTweakBar::~BaseAntTweakBar(void)
{
	//TwTerminate();
}

/** Creates the resources */
XRESULT BaseAntTweakBar::Init()
{
	// Sky
	Bar_Sky = TwNewBar("Sky");
	TwDefine(" Sky position='400 0'");
	TwAddVarRW(Bar_Sky, "G", TW_TYPE_FLOAT, &Engine::GAPI->GetSky()->GetAtmoshpereSettings().G, NULL);
	TwDefine(" Sky/G  step=0.01 ");
	TwDefine(" Sky/G  help='Size of the Sun' ");

	TwAddVarRW(Bar_Sky, "RayleightScaleDepth", TW_TYPE_FLOAT, &Engine::GAPI->GetSky()->GetAtmoshpereSettings().RayleightScaleDepth, NULL);
	TwDefine(" Sky/RayleightScaleDepth  step=0.01 ");
	TwDefine(" Sky/RayleightScaleDepth  help='Controls scattering' ");

	TwAddVarRW(Bar_Sky, "ESun", TW_TYPE_FLOAT, &Engine::GAPI->GetSky()->GetAtmoshpereSettings().ESun, NULL);
	TwDefine(" Sky/ESun  step=0.1 ");
	TwDefine(" Sky/ESun  help='Brightness of the sun' ");

	TwAddVarRW(Bar_Sky, "InnerRadius", TW_TYPE_FLOAT, &Engine::GAPI->GetSky()->GetAtmoshpereSettings().InnerRadius, NULL);
	TwAddVarRW(Bar_Sky, "OuterRadius", TW_TYPE_FLOAT, &Engine::GAPI->GetSky()->GetAtmoshpereSettings().OuterRadius, NULL);
	TwDefine(" Sky/InnerRadius  help='Inner Radius of the fake-planet. This must be greater than SphereOffset.y.' ");
	TwDefine(" Sky/OuterRadius  help='Outer Radius of the fake-planet' ");

	TwAddVarRW(Bar_Sky, "Km", TW_TYPE_FLOAT, &Engine::GAPI->GetSky()->GetAtmoshpereSettings().Km, NULL);
	TwDefine(" Sky/Km  step=0.0001 ");

	TwAddVarRW(Bar_Sky, "Kr", TW_TYPE_FLOAT, &Engine::GAPI->GetSky()->GetAtmoshpereSettings().Kr, NULL);
	TwDefine(" Sky/Kr  step=0.0001 ");

	TwAddVarRW(Bar_Sky, "Samples", TW_TYPE_INT32, &Engine::GAPI->GetSky()->GetAtmoshpereSettings().Samples, NULL);
	TwAddVarRW(Bar_Sky, "WaveLengths", TW_TYPE_DIR3F, &Engine::GAPI->GetSky()->GetAtmoshpereSettings().WaveLengths, NULL);
	TwAddVarRW(Bar_Sky, "SphereOffset.y", TW_TYPE_FLOAT, &Engine::GAPI->GetSky()->GetAtmoshpereSettings().SphereOffsetY, NULL);

	TwAddVarRW(Bar_Sky, "ReplaceSunDirection", TW_TYPE_BOOLCPP, &Engine::GAPI->GetRendererState()->RendererSettings.ReplaceSunDirection, NULL);
	TwDefine(" Sky/ReplaceSunDirection  help='Activates the LightDirection-Vector so you can modify the Direction of the sun' ");

	TwAddVarRW(Bar_Sky, "LightDirection", TW_TYPE_DIR3F, &Engine::GAPI->GetSky()->GetAtmoshpereSettings().LightDirection, NULL);
	TwDefine(	" Sky/ReplaceSunDirection  help='The direction the sun should come from. Only active when ReplaceSunDirection is active.\n"
				"Also useful to fix the sun in one position'");

	TwAddVarRW(Bar_Sky, "SunLightColor", TW_TYPE_COLOR3F, &Engine::GAPI->GetRendererState()->RendererSettings.SunLightColor, NULL);
	TwDefine(" Sky/SunLightColor  help='Color of the sunlight' ");

	TwAddVarRW(Bar_Sky, "SkyTimeScale", TW_TYPE_FLOAT, &Engine::GAPI->GetSky()->GetAtmoshpereSettings().SkyTimeScale, NULL);
	TwDefine(" Sky/SkyTimeScale  help='This makes the skys time pass slower or faster' ");
	
	Bar_General = TwNewBar("General");
	TwDefine(" General position='600 0'");

	TwAddVarRW(Bar_General, "Draw VOBs", TW_TYPE_BOOLCPP, &Engine::GAPI->GetRendererState()->RendererSettings.DrawVOBs, NULL);

	TwAddVarRW(Bar_General, "Draw WorldMesh", TW_TYPE_BOOLCPP, &Engine::GAPI->GetRendererState()->RendererSettings.DrawWorldMesh, NULL);
	TwAddVarRW(Bar_General, "Draw Skeletal Meshes", TW_TYPE_BOOLCPP, &Engine::GAPI->GetRendererState()->RendererSettings.DrawSkeletalMeshes, NULL);
	TwAddVarRW(Bar_General, "Draw ParticleEffects", TW_TYPE_BOOLCPP, &Engine::GAPI->GetRendererState()->RendererSettings.DrawParticleEffects, NULL);
	//TwAddVarRW(Bar_General, "Draw Sky", TW_TYPE_BOOLCPP, &Engine::GAPI->GetRendererState()->RendererSettings.DrawSky, NULL);
	TwAddVarRW(Bar_General, "Draw Fog", TW_TYPE_BOOLCPP, &Engine::GAPI->GetRendererState()->RendererSettings.DrawFog, NULL);	
	TwAddVarRW(Bar_General, "HDR", TW_TYPE_BOOLCPP, &Engine::GAPI->GetRendererState()->RendererSettings.EnableHDR, NULL);	
	TwAddVarRW(Bar_General, "DynamicLighting", TW_TYPE_BOOLCPP, &Engine::GAPI->GetRendererState()->RendererSettings.EnableDynamicLighting, NULL);	

	
	//TwAddVarRW(Bar_General, "FastShadows", TW_TYPE_BOOLCPP, &Engine::GAPI->GetRendererState()->RendererSettings.FastShadows, NULL);	
	TwAddVarRW(Bar_General, "DrawShadowGeometry", TW_TYPE_BOOLCPP, &Engine::GAPI->GetRendererState()->RendererSettings.DrawShadowGeometry, NULL);	
	TwAddVarRW(Bar_General, "WireframeWorld", TW_TYPE_BOOLCPP, &Engine::GAPI->GetRendererState()->RendererSettings.WireframeWorld, NULL);	
	TwAddVarRW(Bar_General, "WireframeVobs", TW_TYPE_BOOLCPP, &Engine::GAPI->GetRendererState()->RendererSettings.WireframeVobs, NULL);	
	
	TwAddVarRW(Bar_General, "Grass AlphaToCoverage", TW_TYPE_BOOLCPP, &Engine::GAPI->GetRendererState()->RendererSettings.VegetationAlphaToCoverage, NULL);	
	

	TwAddVarRW(Bar_General, "SectionDrawRadius", TW_TYPE_INT32, &Engine::GAPI->GetRendererState()->RendererSettings.SectionDrawRadius, NULL);
	TwDefine(" General/SectionDrawRadius  help='Draw distance for the sections' ");

	TwAddVarRW(Bar_General, "OutdoorVobDrawRadius", TW_TYPE_FLOAT, &Engine::GAPI->GetRendererState()->RendererSettings.OutdoorVobDrawRadius, NULL);
	TwDefine(" General/OutdoorVobDrawRadius  help='Draw distance for the static outdoor vobs' ");

	TwAddVarRW(Bar_General, "IndoorVobDrawRadius", TW_TYPE_FLOAT, &Engine::GAPI->GetRendererState()->RendererSettings.IndoorVobDrawRadius, NULL);
	TwDefine(" General/OutdoorVobDrawRadius  help='Draw distance for the static indoor vobs' ");

	TwAddVarRW(Bar_General, "OutdoorSmallVobRadius", TW_TYPE_FLOAT, &Engine::GAPI->GetRendererState()->RendererSettings.OutdoorSmallVobDrawRadius, NULL);
	TwDefine(" General/OutdoorVobDrawRadius  help='Draw distance for the vobs defined as small' ");

	//TwAddVarRW(Bar_General, "SmallVobSize", TW_TYPE_FLOAT, &Engine::GAPI->GetRendererState()->RendererSettings.SmallVobSize, NULL);
	//TwAddVarRW(Bar_General, "AtmosphericScattering", TW_TYPE_BOOLCPP, &Engine::GAPI->GetRendererState()->RendererSettings.AtmosphericScattering, NULL);

	TwAddVarRW(Bar_General, "WorldShadowRangeScale", TW_TYPE_FLOAT, &Engine::GAPI->GetRendererState()->RendererSettings.WorldShadowRangeScale, NULL);
	TwDefine(" General/WorldShadowRangeScale  step=0.01 min=0");
	TwDefine(" General/WorldShadowRangeScale  help='Makes shadows reach further at the cost of a lesser resolution of shadows' ");

	TwAddVarRW(Bar_General, "FogGlobalDensity", TW_TYPE_FLOAT, &Engine::GAPI->GetRendererState()->RendererSettings.FogGlobalDensity, NULL);
	TwDefine(" General/FogGlobalDensity  step=0.00001 min=0");
	TwDefine(" General/FogGlobalDensity  help='Controls thickness of the fog' ");

	TwAddVarRW(Bar_General, "FogHeightFalloff", TW_TYPE_FLOAT, &Engine::GAPI->GetRendererState()->RendererSettings.FogHeightFalloff, NULL);
	TwDefine(" General/FogHeightFalloff  step=0.00001 min=0");
	TwDefine(" General/FogHeightFalloff  help='Controls how sharp the falloff of the fog should be' ");

	TwAddVarRW(Bar_General, "FogHeight", TW_TYPE_FLOAT, &Engine::GAPI->GetRendererState()->RendererSettings.FogHeight, NULL);
	TwDefine(" General/FogHeight  help='Height at where the Fog should start' ");

	TwAddVarRW(Bar_General, "FogColorMod", TW_TYPE_COLOR3F, &Engine::GAPI->GetRendererState()->RendererSettings.FogColorMod, NULL);
	TwDefine(" General/FogColorMod  help='Color of the fog' ");

	TwAddVarRW(Bar_General, "HDRLumWhite", TW_TYPE_FLOAT, &Engine::GAPI->GetRendererState()->RendererSettings.HDRLumWhite, NULL);
	TwDefine(" General/HDRLumWhite  step=0.01");
	TwDefine(" General/HDRLumWhite  help='Luminance taken as *white* for the HDR renderer' ");

	TwAddVarRW(Bar_General, "HDRMiddleGray", TW_TYPE_FLOAT, &Engine::GAPI->GetRendererState()->RendererSettings.HDRMiddleGray, NULL);
	TwDefine(" General/HDRMiddleGray  step=0.01");
	TwDefine(" General/HDRMiddleGray  help='Controls the overall brightness of the HDR renderer' ");

	TwAddVarRW(Bar_General, "BloomThreshold", TW_TYPE_FLOAT, &Engine::GAPI->GetRendererState()->RendererSettings.BloomThreshold, NULL);
	TwDefine(" General/BloomThreshold  step=0.01");
	TwDefine(" General/BloomThreshold  help='Controls the brightness at when the blooming should start' ");

	TwAddVarRW(Bar_General, "BloomStrength", TW_TYPE_FLOAT, &Engine::GAPI->GetRendererState()->RendererSettings.BloomStrength, NULL);
	TwDefine(" General/BloomStrength  step=0.01");
	TwDefine(" General/BloomStrength  help='Controls the strength of the bloom-effect' ");
	
	TwAddVarRW(Bar_General, "WindStrength", TW_TYPE_FLOAT, &Engine::GAPI->GetRendererState()->RendererSettings.GlobalWindStrength, NULL);
	TwDefine(" General/WindStrength  step=0.01");
	
	
	//TwAddVarRW(Bar_General, "Max Num Indices", TW_TYPE_INT32, &Engine::GAPI->GetRendererState()->RendererSettings.MaxNumFaces, NULL);

	
	Bar_Info = TwNewBar("FrameStats");
	TwDefine(" FrameStats refresh=0.3");
	TwDefine(" FrameStats position='800 0'");
	TwAddVarRO(Bar_Info, "Version", TW_TYPE_CDSTRING, &VERSION_STRING, NULL);
	TwAddVarRO(Bar_Info, "FPS", TW_TYPE_INT32, &Engine::GAPI->GetRendererState()->RendererInfo.FPS, NULL);
	TwAddVarRO(Bar_Info, "DrawnVobs", TW_TYPE_INT32, &Engine::GAPI->GetRendererState()->RendererInfo.FrameDrawnVobs, NULL);
	TwAddVarRO(Bar_Info, "DrawnTriangles", TW_TYPE_INT32, &Engine::GAPI->GetRendererState()->RendererInfo.FrameDrawnTriangles, NULL);
	TwAddVarRO(Bar_Info, "VobUpdates", TW_TYPE_INT32, &Engine::GAPI->GetRendererState()->RendererInfo.FrameVobUpdates, NULL);
	TwAddVarRO(Bar_Info, "SectionsDrawn", TW_TYPE_INT32, &Engine::GAPI->GetRendererState()->RendererInfo.FrameNumSectionsDrawn, NULL);
	TwAddVarRO(Bar_Info, "FarPlane", TW_TYPE_FLOAT, &Engine::GAPI->GetRendererState()->RendererInfo.FarPlane, NULL);
	TwAddVarRO(Bar_Info, "NearPlane", TW_TYPE_FLOAT, &Engine::GAPI->GetRendererState()->RendererInfo.NearPlane, NULL);
	TwAddVarRO(Bar_Info, "VOBVerticesDataSize", TW_TYPE_UINT32, &Engine::GAPI->GetRendererState()->RendererInfo.VOBVerticesDataSize, NULL);
	TwAddVarRO(Bar_Info, "SkeletalVerticesDataSize", TW_TYPE_UINT32, &Engine::GAPI->GetRendererState()->RendererInfo.SkeletalVerticesDataSize, NULL);

	Bar_TextureSettings = TwNewBar("TextureSettings");
	TwAddVarRW(Bar_TextureSettings, "Texture", TW_TYPE_CSSTRING(256), TS_TextureName, "");
	TwDefine(" TextureSettings size='455 120' valueswidth=fit position='400 320'");

	return XR_SUCCESS;
}

/** On window message */
LRESULT BaseAntTweakBar::OnWindowMessage(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	if(IsActive)
		return TwEventWin(hWnd, msg, wParam, lParam);

	return true;
}

/** Sets the bars active and visible */
void BaseAntTweakBar::SetActive(bool value)
{
	IsActive = value;

	if(IsActive)
	{
		InitTextureSettingsBar();
	}
}

bool BaseAntTweakBar::GetActive()
{
	return IsActive;
}

/** Draws the bars */
void BaseAntTweakBar::Draw()
{
	if(IsActive)
	{
		TwDraw();

		// Check texture name
		if(LastFrameActiveMaterialInfo != ActiveMaterialInfo)
		{
			if(ActiveMaterialInfo)
			{
				// Update the bar
				UpdateTextureSettingsBar();
			}

			LastFrameActiveMaterialInfo = ActiveMaterialInfo;
		}

		// Check for change
		if(TS_Active)
		{
			if(ActiveMaterialInfo)
			{
				// Check for a change
				if(memcmp(&TS_OldMaterialInfo, TS_FrameTexturesInfos[ActiveMaterialInfo].Info, sizeof(MaterialInfo)) != 0)
				{
					TS_FrameTexturesInfos[ActiveMaterialInfo].Info->WriteToFile(TS_FrameTexturesInfos[ActiveMaterialInfo].Name);
					TS_OldMaterialInfo = *TS_FrameTexturesInfos[ActiveMaterialInfo].Info;

					LogInfo() << "Saved MaterialInfo: " << TS_TextureName;
				}
			}

			if(ActiveMaterialInfo && TS_FrameTexturesInfos[ActiveMaterialInfo].Texture)
			{
				const int previewSize = 512;
				const int previewDownscale = 2;
				INT2 pPosition;
				pPosition.x = Engine::GraphicsEngine->GetResolution().x - (previewSize / previewDownscale);
				pPosition.y = 0;

				INT2 pSize = INT2(previewSize / previewDownscale,previewSize / previewDownscale);

				TS_FrameTexturesInfos[ActiveMaterialInfo].Texture->Bind(0);
				Engine::GraphicsEngine->DrawQuad(pPosition, pSize);
			}
		}
	}
}
 
/** Updates the TS_Bar */
void BaseAntTweakBar::UpdateTextureSettingsBar()
{
		TwRemoveVar(Bar_TextureSettings, "NormalmapDepth");
		TwRemoveVar(Bar_TextureSettings, "SpecularIntensity");
		TwRemoveVar(Bar_TextureSettings, "SpecularPower");

		TwAddVarRW(Bar_TextureSettings, "NormalmapDepth", TW_TYPE_FLOAT, &TS_FrameTexturesInfos[ActiveMaterialInfo].Info->NormalmapStrength, NULL);
		TwDefine(" TextureSettings/NormalmapDepth  step=0.01");

		TwAddVarRW(Bar_TextureSettings, "SpecularIntensity", TW_TYPE_FLOAT, &TS_FrameTexturesInfos[ActiveMaterialInfo].Info->SpecularIntensity, NULL);
		TwDefine(" TextureSettings/SpecularIntensity  step=0.01");

		TwAddVarRW(Bar_TextureSettings, "SpecularPower", TW_TYPE_FLOAT, &TS_FrameTexturesInfos[ActiveMaterialInfo].Info->SpecularPower, NULL);
		TwDefine(" TextureSettings/SpecularPower  step=0.1");

		//TwAddVarRW(Bar_TextureSettings, "DisplacementFactor", TW_TYPE_FLOAT, &TS_FrameTexturesInfos[ActiveMaterialInfo].Info->DisplacementFactor, NULL);
		//TwDefine(" TextureSettings/DisplacementFactor  step=0.1");
}

static bool FrameTexEnumCompare(TS_TextureInfo& a, TS_TextureInfo& b)
{
	return std::string(a.Name) < std::string(b.Name);
}

/** Initializes the TextureSettingsBar */
void BaseAntTweakBar::InitTextureSettingsBar()
{
	TwRemoveAllVars(Bar_TextureSettings);
	//TwRemoveVar(Bar_TextureSettings, "FrameTextures");
	TS_FrameTexturesInfos.clear();

	std::vector<TS_TextureInfo> e;

	std::string enumDef = " enum='";

	const std::set<zCTexture *>& frameTextures = Engine::GraphicsEngine->GetFrameTextures();
	if(!frameTextures.size())
		return;

	for(std::set<zCTexture *>::const_iterator it = frameTextures.begin(); it != frameTextures.end(); it++)
	{
		e.push_back(TS_TextureInfo(Engine::GAPI->GetMaterialInfoFrom((*it)), (*it)->GetNameWithoutExt(), (*it)));
	}

	// Sort alphabetically
	std::sort(e.begin(), e.end(), FrameTexEnumCompare);

	for(unsigned int i=0;i<e.size();i++)
	{
		// Pointer to int. Since Gothic is 32-Bit this can't run in 64-bit mode anyways.
		enumDef += std::to_string(i) + " {" + e[i].Name + "}";

		TS_FrameTexturesInfos.push_back(TS_TextureInfo(e[i].Info, e[i].Name, e[i].Texture));

		if(i != e.size() - 1)
			enumDef += ", ";
	}

	
	enumDef += "'";

	// Add the list to the bar
	TwType t = TwDefineEnumFromString("frameTextureList", enumDef.c_str());
	TwAddVarRW(Bar_TextureSettings, "FrameTextures", t, &ActiveMaterialInfo, "0");

	TS_Active = true;
}
