#include "pch.h"
#include "zCMaterial.h"
#include "D2DEditorView.h"
#include "D2DView.h"
#include "SV_Border.h"
#include "SV_Panel.h"
#include "SV_Button.h"
#include "SV_Checkbox.h"
#include "Engine.h"
#include "GothicAPI.h"
#include "BaseGraphicsEngine.h"
#include "BaseLineRenderer.h"
#include "SV_TabControl.h"
#include "GVegetationBox.h"


D2DEditorView::D2DEditorView(D2DView* view, D2DSubView* parent) : D2DSubView(view, parent)
{
	InitControls();
	
	IsEnabled = false;
	Mode = EM_IDLE;
	DraggedBoxMinLocal = D3DXVECTOR3(-700,-500,-700);
	DraggedBoxMaxLocal = D3DXVECTOR3(700,500,700);
	DraggedBoxCenter = D3DXVECTOR3(0,0,0);

	memset(SelectedTriangle, 0, sizeof(SelectedTriangle));

	SelectedMesh = NULL;
	TracedMesh = NULL;
	TracedMaterial = NULL;
	SelectedMaterial = NULL;

	SelectedSomething = false;
}


D2DEditorView::~D2DEditorView(void)
{

}

/** Initializes the controls of this view */
XRESULT D2DEditorView::InitControls()
{
	SV_Panel* subPanel = new SV_Panel(MainView, this);
	subPanel->SetRect(D2D1::RectF(0, 0, 290, MainView->GetRenderTarget()->GetSize().height));
	subPanel->SetPanelShadow(true, 50.0f);
	MainPanel = subPanel;

	//SV_Panel* vegPanel = new SV_Panel(MainView, subPanel);
	//vegPanel->SetRect(D2D1::RectF(20, 20, 270, 180));
	//vegPanel->SetPanelShadow(true, 20.0f);

	//SV_Button* subButton = new SV_Button(MainView, subPanel);
	//subButton->SetPositionAndsize(D2D1::Point2F(20, 30), D2D1::SizeF(70, 40));

	


	// Test Tab-Control
	MainTabControl = new SV_TabControl(MainView, subPanel);
	MainTabControl->SetSize(D2D1::SizeF(200, 300));
	MainTabControl->SetRect(D2D1::RectF(20, 20, 270, 180));

	MainTabControl->SetTabSwitchedCallback(MainTabSwitched, this);

	/** Vegetation placement */
	SV_Button* addVegButton = new SV_Button(MainView, MainTabControl->GetTabPanel());
	addVegButton->SetPositionAndsize(D2D1::Point2F(15, 10), D2D1::SizeF(100, 25));
	addVegButton->SetPressedCallback(AddVegButtonPressed, this);
	addVegButton->SetCaption("Add Vegetation");
	MainTabControl->AddControlToTab(addVegButton, "Vegetation");

	SV_Button* fillVegButton = new SV_Button(MainView, MainTabControl->GetTabPanel());
	fillVegButton->SetPositionAndsize(D2D1::Point2F(10, 10), D2D1::SizeF(100, 25));
	fillVegButton->SetPressedCallback(FillVegButtonPressed, this);
	fillVegButton->AlignRightTo(addVegButton, 10.0f);
	fillVegButton->SetCaption("Fill selection");
	MainTabControl->AddControlToTab(fillVegButton, "Vegetation");

	VegRestrictByTextureCheckBox = new SV_Checkbox(MainView, MainTabControl->GetTabPanel());
	VegRestrictByTextureCheckBox->SetSize(D2D1::SizeF(160, 20));
	VegRestrictByTextureCheckBox->AlignUnder(addVegButton, 5.0f);
	VegRestrictByTextureCheckBox->SetCaption("Texture aware");
	MainTabControl->AddControlToTab(VegRestrictByTextureCheckBox, "Vegetation");

	VegCircularShapeCheckBox = new SV_Checkbox(MainView, MainTabControl->GetTabPanel());
	VegCircularShapeCheckBox->SetSize(D2D1::SizeF(160, 20));
	VegCircularShapeCheckBox->AlignUnder(VegRestrictByTextureCheckBox, 5.0f);
	VegCircularShapeCheckBox->SetCaption("Circular shape");
	MainTabControl->AddControlToTab(VegCircularShapeCheckBox, "Vegetation");

	

	/** Selection */
	SelectTrianglesOnlyCheckBox = new SV_Checkbox(MainView, MainTabControl->GetTabPanel());
	SelectTrianglesOnlyCheckBox->SetSize(D2D1::SizeF(160, 20));
	SelectTrianglesOnlyCheckBox->SetPosition(D2D1::Point2F(10, 10));
	SelectTrianglesOnlyCheckBox->SetCaption("Select Triangles Only CheckBox");
	MainTabControl->AddControlToTab(SelectTrianglesOnlyCheckBox, "Selection");

	SV_Button* testbutton1 = new SV_Button(MainView, MainTabControl->GetTabPanel());
	testbutton1->SetPositionAndsize(D2D1::Point2F(10, 10), D2D1::SizeF(120, 25));
	testbutton1->SetCaption("Vegetation");
	MainTabControl->AddControlToTab(testbutton1, "Test 1");

	SV_Button* testbutton2 = new SV_Button(MainView, MainTabControl->GetTabPanel());
	testbutton2->SetPositionAndsize(D2D1::Point2F(10, 40), D2D1::SizeF(120, 25));
	testbutton2->SetCaption("Test 2");
	MainTabControl->AddControlToTab(testbutton2, "Test 2");

	return XR_SUCCESS;
}

/** Draws this sub-view */
void D2DEditorView::Draw(const D2D1_RECT_F& clientRectAbs, float deltaTime)
{
	// If the editor is not open, dont draw it. Slide it in otherwise.

	

	D2D1_POINT_2F p;
	p.y = 0;
	if(!IsEnabled)
	{
#ifdef PUBLIC_RELEASE
		if(deltaTime == 0.0f) // Don't show it in menus
			p.x = -(MainPanel->GetSize().width + 50.0f);
		else
#endif
			p.x = Toolbox::lerp(MainPanel->GetPosition().x, -(MainPanel->GetSize().width + 50.0f), std::min(deltaTime * 5.0f, 1.0f));
	}else
	{
		p.x = Toolbox::lerp(MainPanel->GetPosition().x, 0, std::min(deltaTime * 8.0f, 1.0f));
	}
	MainPanel->SetPosition(p);

	if(IsEnabled || MainPanel->GetPosition().x + 40.0f > -MainPanel->GetSize().width)
	{
		D2DSubView::Draw(clientRectAbs, deltaTime);
	}
}

/** Updates the editor */
void D2DEditorView::Update(float deltaTime)
{
	if(!IsEnabled)
		return;

	if(Mode == EM_PLACE_VEGETATION)
	{
		DoVegetationPlacement();
	}else if(Mode == EM_SELECT_POLY || Mode == EM_IDLE)
	{
		DoSelection();
	}

	if(SelectedMesh)
	{
		VisualizeMeshInfo(SelectedMesh, D3DXVECTOR4(1,0,0,1));
	}
}

/** Handles vegetationbox placement */
void D2DEditorView::DoVegetationPlacement()
{
	D3DXVECTOR3 wDir = Engine::GAPI->UnprojectCursor();
	D3DXVECTOR3 hit;
	D3DXVECTOR3 hitTri[3];
	
	TracedTexture = "";

	// Check for restricted by texture
	std::string* rtp = NULL;
	if(VegRestrictByTextureCheckBox->GetChecked())
		rtp = &TracedTexture;

	// Trace the worldmesh from the cursor
	if(Engine::GAPI->TraceWorldMesh(Engine::GAPI->GetCameraPosition(), wDir, hit, rtp, hitTri))
	{
		// Update the position if successful
		DraggedBoxCenter = hit;

		// Visualize box
		Engine::GraphicsEngine->GetLineRenderer()->AddAABBMinMax(DraggedBoxCenter + DraggedBoxMinLocal, DraggedBoxCenter + DraggedBoxMaxLocal);

		// Visualize triangle
		D3DXVECTOR3 nrm = Toolbox::ComputeNormal(hitTri[0], hitTri[1], hitTri[2]);
		Engine::GraphicsEngine->GetLineRenderer()->AddTriangle(hitTri[0] + nrm, hitTri[1] + nrm, hitTri[2] + nrm);
	}
}

/** Handles selection */
void D2DEditorView::DoSelection()
{
	D3DXVECTOR3 wDir = Engine::GAPI->UnprojectCursor();
	D3DXVECTOR3 hit;
	D3DXVECTOR3 hitTri[3];
	MeshInfo* hitMesh;
	zCMaterial* hitMaterial;
	TracedTexture = "";

	// Trace the worldmesh from the cursor
	if(Engine::GAPI->TraceWorldMesh(Engine::GAPI->GetCameraPosition(), wDir, hit, &TracedTexture, hitTri, &hitMesh, &hitMaterial))
	{
		if(SelectTrianglesOnlyCheckBox->GetChecked())
		{
			memcpy(SelectedTriangle, hitTri, sizeof(SelectedTriangle));

			// Visualize triangle
			D3DXVECTOR3 nrm = Toolbox::ComputeNormal(hitTri[0], hitTri[1], hitTri[2]);
			Engine::GraphicsEngine->GetLineRenderer()->AddTriangle(hitTri[0] + nrm, hitTri[1] + nrm, hitTri[2] + nrm);

			TracedMaterial = hitMaterial;
		}else
		{
			TracedMesh = hitMesh;
			TracedMaterial = hitMaterial;
			
			VisualizeMeshInfo(hitMesh);
		}
	}
}

/** Visualizes a mesh info */
void D2DEditorView::VisualizeMeshInfo(MeshInfo* m, const D3DXVECTOR4& color)
{
	for(unsigned int i=0;i<m->Indices.size();i+=3)
	{
		D3DXVECTOR3 tri[3];

		tri[0] = *m->Vertices[m->Indices[i]].Position.toD3DXVECTOR3();
		tri[1] = *m->Vertices[m->Indices[i+1]].Position.toD3DXVECTOR3();
		tri[2] = *m->Vertices[m->Indices[i+2]].Position.toD3DXVECTOR3();

		// Visualize triangle
		D3DXVECTOR3 nrm = Toolbox::ComputeNormal(tri[0], tri[1], tri[2]);
		Engine::GraphicsEngine->GetLineRenderer()->AddTriangle(tri[0] + nrm, tri[1] + nrm, tri[2] + nrm, color);
	}
}

/** Called on LBUTTONDOWN */
void D2DEditorView::OnLButtonDown()
{
	POINT p;
	GetCursorPos(&p);
	ScreenToClient(Engine::GAPI->GetOutputWindow(), &p);

	// Catch clicks inside the editor window
	if(D2DSubView::PointInsideRect(D2D1::Point2F((float)p.x, (float)p.y), MainPanel->GetRect()))
	{
		return;
	}

	if(Mode == EM_PLACE_VEGETATION)
	{
		PlaceDraggedVegetationBox();
	}else if(Mode == EM_SELECT_POLY || Mode == EM_IDLE)
	{
		SelectedMesh = TracedMesh;
		SelectedMaterial = TracedMaterial;
	}
}


/** Places the currently dragged vegetation box */
GVegetationBox* D2DEditorView::PlaceDraggedVegetationBox()
{
	SetEditorMode(EM_IDLE);

	GVegetationBox::EShape shape = GVegetationBox::S_Box;
	if(VegCircularShapeCheckBox->GetChecked())
		shape = GVegetationBox::S_Circle;

	GVegetationBox* box = new GVegetationBox;
	if(XR_SUCCESS == box->InitVegetationBox(DraggedBoxCenter + DraggedBoxMinLocal, DraggedBoxCenter + DraggedBoxMaxLocal,
		"", 1.0f, 1.0f, TracedTexture, shape))
	{
		Engine::GAPI->AddVegetationBox(box);
	}else
	{
		delete box;
	}

	return box;
	//return Engine::GAPI->SpawnVegetationBoxAt(DraggedBoxCenter, DraggedBoxMinLocal, DraggedBoxMaxLocal, 1.0f, TracedTexture);
}

/** Processes a window-message. Return false to stop the message from going to children */
bool D2DEditorView::OnWindowMessage(HWND hWnd, unsigned int msg, WPARAM wParam, LPARAM lParam, const D2D1_RECT_F& clientRectAbs)
{
	switch(msg)
	{
	case WM_KEYDOWN:
		switch(wParam)
		{
		case VK_F1:
			IsEnabled = !IsEnabled;

			if(IsEnabled)
				Engine::GAPI->SetEnableGothicInput(false);

			return false;
			break;
		}
		break;

	case WM_LBUTTONDOWN:
		OnLButtonDown();
		break;
	}

	if(IsEnabled)
		return D2DSubView::OnWindowMessage(hWnd, msg, wParam, lParam, clientRectAbs);

	return true;
}

/** Sets the editor-mode */
void D2DEditorView::SetEditorMode(EditorMode mode)
{
	Mode = mode;
}
 

/** Button to add a vegetation-volume was pressed */
void D2DEditorView::AddVegButtonPressed(SV_Button* sender, void* userdata)
{
	D2DEditorView* v = (D2DEditorView *)userdata;

	if(v->Mode != EM_IDLE)
	{
		LogWarn() << "Editor must be in idle-state before you can add a vegetation-volume!";
		return;
	}

	v->SetEditorMode(EM_PLACE_VEGETATION);
}

/** Button to add a vegetation-volume was pressed */
void D2DEditorView::FillVegButtonPressed(SV_Button* sender, void* userdata)
{
	D2DEditorView* v = (D2DEditorView *)userdata;

	if(v->SelectedMesh)
	{
		LogInfo() << "Filling selected mesh with vegetation";

		GVegetationBox* box = new GVegetationBox;
		if(XR_SUCCESS == box->InitVegetationBox(v->SelectedMesh, "", 1.0f, 1.0f, v->SelectedMaterial->GetTexture()))
		{
			Engine::GAPI->AddVegetationBox(box);
		}else
		{
			delete box;
		}
	}
}

/** Tab in main tab-control was switched */
void D2DEditorView::MainTabSwitched(SV_TabControl* sender, void* userdata)
{
	D2DEditorView* v = (D2DEditorView *)userdata;

	//v->SetEditorMode(D2DEditorView::EM_SELECT_POLY);

	/*if(sender->GetActiveTab() == "Selection")
	{
		v->SetEditorMode(D2DEditorView::EM_SELECT_POLY);
	}else if(sender->GetActiveTab() == "Vegetation")
	{
		v->SetEditorMode(D2DEditorView::EM_PLACE_VEGETATION);
	}else
	{
		v->SetEditorMode(D2DEditorView::EM_IDLE);
	}*/
}