#pragma once
#include "D2DSubView.h"
#include "SV_Button.h"
#include "SV_TabControl.h"

class SV_Panel;
class SV_Checkbox;
class GVegetationBox;
struct MeshInfo;
class zCMaterial;
class D2DEditorView :
	public D2DSubView
{
public:
	D2DEditorView(D2DView* view, D2DSubView* parent);
	virtual ~D2DEditorView(void);

	enum EditorMode
	{
		EM_IDLE,
		EM_PLACE_VEGETATION,
		EM_SELECT_POLY,
	};

	/** Initializes the controls of this view */
	virtual XRESULT InitControls();

	/** Draws this sub-view */
	virtual void Draw(const D2D1_RECT_F& clientRectAbs, float deltaTime);

	/** Updates the subview */
	void Update(float deltaTime);

	/** Processes a window-message. Return false to stop the message from going to children */
	virtual bool OnWindowMessage(HWND hWnd, unsigned int msg, WPARAM wParam, LPARAM lParam, const D2D1_RECT_F& clientRectAbs);

protected:
	/** Visualizes a mesh info */
	void VisualizeMeshInfo(MeshInfo* m, const D3DXVECTOR4& color = D3DXVECTOR4(1,1,1,1));

	/** Sets the editor-mode */
	void SetEditorMode(EditorMode mode);

	/** Handles vegetationbox placement */
	void DoVegetationPlacement();

	/** Handles selection */
	void DoSelection();

	/** Called on LBUTTONDOWN */
	void OnLButtonDown();

	/** Places the currently dragged vegetation box */
	GVegetationBox* PlaceDraggedVegetationBox();

	/** Callbacks */
	/** Button to add a vegetation-volume was pressed */
	static void AddVegButtonPressed(SV_Button* sender, void* userdata);

	/** Button to add a vegetation-volume was pressed */
	static void FillVegButtonPressed(SV_Button* sender, void* userdata);

	/** Tab in main tab-control was switched */
	static void MainTabSwitched(SV_TabControl* sender, void* userdata);


	/** Editor enabled? */
	bool IsEnabled;

	/** Main editor panel */
	SV_Panel* MainPanel;
	SV_TabControl* MainTabControl;

	/** Current mode */
	EditorMode Mode;

	std::string TracedTexture;

	/** Vegetation-box specific values, valid in EM_PLACE_VEGETATION*/
	D3DXVECTOR3 DraggedBoxMinLocal;
	D3DXVECTOR3 DraggedBoxMaxLocal;
	D3DXVECTOR3 DraggedBoxCenter;
	
	SV_Checkbox* VegRestrictByTextureCheckBox;
	SV_Checkbox* VegCircularShapeCheckBox;

	/** Selection specific values */
	bool SelectedSomething;
	D3DXVECTOR3 SelectedTriangle[3];
	MeshInfo* TracedMesh;
	zCMaterial* TracedMaterial;
	MeshInfo* SelectedMesh;
	zCMaterial* SelectedMaterial;
	SV_Checkbox* SelectTrianglesOnlyCheckBox;
};

