#pragma once
#include "pch.h"
#include <d2d1_1.h>
#include <dwrite_1.h>

const D2D1_COLOR_F SV_DEF_INNER_LINE_COLOR = D2D1::ColorF(0.3f,0.3f,0.6f,1.0f);

class D2DSubView;
class D2DView
{
public:
	D2DView(void);
	~D2DView(void);

	/** Inits this d2d-view */
	XRESULT Init(const INT2& initialResolution, ID3D11Texture2D* rendertarget);

	/** Releases all resources needed to resize this view */
	XRESULT PrepareResize();

	/** Resizes this d2d-view */
	XRESULT Resize(const INT2& initialResolution, ID3D11Texture2D* rendertarget);

	/** Draws the view */
	void Render(float deltaTime);

	/** Updates the view */
	void Update(float deltaTime);

	/** Drass a smooth shadow around the given rectangle */
	void DrawSmoothShadow(const D2D1_RECT_F* Rectangle, 
		float Size, 
		float Opacity, 
		bool bDoNotShrink = false, 
		float RectShrink = 0.8f,
		bool bLeft = true,
		bool bTop = true,
		bool bRight = true,
		bool bBottom = true);

	/** Processes a window-message. Return false to stop the message from going to children */
	bool OnWindowMessage(HWND hWnd, unsigned int msg, WPARAM wParam, LPARAM lParam);

	/** Shrinks a rectangle */
	static void D2DView::ShrinkRect(D2D1_RECT_F* Rect,float Offset);

	/** Returns the current rendertarget */
	ID2D1RenderTarget* GetRenderTarget(){return RenderTarget;}

	/** Returns the main brush */
	ID2D1SolidColorBrush* GetBrush(){return Brush;}

	/** Returns the main brush */
	ID2D1RadialGradientBrush* GetRadialBrush(){return RadialBrush;}

	/** Returns the main brush */
	ID2D1LinearGradientBrush* GetLinearBrush(){return LinearBrush;}

	/** Returns the main brush */
	ID2D1LinearGradientBrush* GetGUIStyleLinearBrush(){return GUIStyleLinearBrush;}

	/** Returns the main brush */
	ID2D1LinearGradientBrush* GetLinearReflectBrushHigh(){return LinearReflectBrushHigh;}

	/** Returns the main brush */
	ID2D1LinearGradientBrush* GetLinearReflectBrush(){return LinearReflectBrush;}

	/** Returns the main brush */
	ID2D1LinearGradientBrush* GetBackgroundBrush(){return BackgroundBrush;}

	/** Returns the WriteFactory */
	IDWriteFactory1* GetWriteFactory(){return WriteFactory;}

	/** Returns the default text format */
	IDWriteTextFormat* GetDefaultTextFormat(){return DefaultTextFormat;};

	/** Returns the width of the specified text */
	static float GetLabelTextWidth(IDWriteTextLayout* layout, const std::string& text);

	/** Returns the main sub view */
	D2DSubView* GetMainSubView(){return MainSubView;}
protected:
	ID2D1Factory* Factory;
	ID2D1RenderTarget* RenderTarget;
	IDWriteFactory1* WriteFactory;

	IDWriteTextFormat* DefaultTextFormat;

	ID2D1SolidColorBrush* Brush;
	ID2D1RadialGradientBrush* RadialBrush;
	ID2D1LinearGradientBrush* LinearBrush;
	ID2D1LinearGradientBrush* GUIStyleLinearBrush;

	ID2D1LinearGradientBrush* LinearReflectBrushHigh;
	ID2D1LinearGradientBrush* LinearReflectBrush;
	ID2D1LinearGradientBrush* BackgroundBrush;

	D2DSubView* MainSubView;
};

