#include "pch.h"
#include "D3D11GraphicsEngine.h"
#include "D3D11VertexBuffer.h"
#include "D3D11ShaderManager.h"
#include "D3D11PShader.h"
#include "D3D11VShader.h"
#include "D3D11ConstantBuffer.h"
#include "D3D11Texture.h"
#include "Engine.h"
#include "GothicAPI.h"
#include "Logger.h"
#include "RenderToTextureBuffer.h"
#include "GSky.h"
#include "GMesh.h"
#include "D3D11PfxRenderer.h"
#include "ConstantBufferStructs.h"
#include "BaseAntTweakBar.h"
#include "zCVob.h"
#include "zCVobLight.h"
#include "zCMaterial.h"
#include "zCLightmap.h"
#include "D3D11LineRenderer.h"
#include "D3D7\MyDirectDrawSurface7.h"
#include "zCView.h"
#include "D2DView.h"

//#include "MemoryTracker.h"

#pragma comment( lib, "dxguid.lib")

const int RES_UPSCALE = 1;
const INT2 DEFAULT_RESOLUTION = INT2(1920 * RES_UPSCALE, 1080*  RES_UPSCALE);
const int WORLD_SHADOWMAP_SIZE = 4096;

const float DEFAULT_FAR_PLANE = 40000.0f;
//#define DEBUG_D3D11

D3D11GraphicsEngine::D3D11GraphicsEngine(void)
{
	Resolution = DEFAULT_RESOLUTION;

	SwapChain = NULL;
	DXGIFactory = NULL;
	DXGIAdapter = NULL;
	OutputWindow = NULL;
	BackbufferRTV = NULL;
	DepthStencilBuffer = NULL;
	Device = NULL;
	Context = NULL;
	DistortionTexture = NULL;
	DeferredContext = NULL;
	UIView = NULL;

	ShadowmapSamplerState = NULL;
	DefaultSamplerState = NULL;
	HUDRasterizerState = NULL;
	WorldRasterizerState = NULL;
	DefaultDepthStencilState = NULL;

	FFRasterizerState = NULL;
	FFBlendState = NULL;
	FFDepthStencilState = NULL;

	ShaderManager = NULL;
	DynamicInstancingBuffer = NULL;
	TempVertexBuffer = NULL;
	NoiseTexture = NULL;

	ActivePS = NULL;
	PfxRenderer = NULL;
	CloudBuffer = NULL;
	BackbufferSRV = NULL;

	InverseUnitSphereMesh = NULL;

	GBuffer1_Normals_SpecIntens_SpecPower = NULL;
	GBuffer0_Diffuse = NULL;
	WorldShadowmap1 = NULL;
	HDRBackBuffer = NULL;

	RenderingStage = DES_MAIN;

	PresentPending = false;

	LineRenderer = new D3D11LineRenderer;


	RECT desktopRect;
	GetClientRect(GetDesktopWindow(), &desktopRect);

	// Match the resolution with the current desktop resolution
	Resolution = INT2(desktopRect.right * RES_UPSCALE, desktopRect.bottom * RES_UPSCALE);
}

D3D11GraphicsEngine::~D3D11GraphicsEngine(void)
{
	delete LineRenderer;
	delete DepthStencilBuffer;
	delete DynamicInstancingBuffer;
	delete PfxRenderer;
	delete CloudBuffer;
	delete DistortionTexture;
	delete NoiseTexture;
	delete ShaderManager;
	delete TempVertexBuffer;
	delete GBuffer1_Normals_SpecIntens_SpecPower;
	delete GBuffer0_Diffuse;
	delete HDRBackBuffer;
	delete WorldShadowmap1;
	delete InverseUnitSphereMesh;
	delete UIView;

	if(ShadowmapSamplerState)ShadowmapSamplerState->Release();
	if (FFRasterizerState)FFRasterizerState->Release();
	if (FFBlendState)FFBlendState->Release();
	if (FFDepthStencilState)FFDepthStencilState->Release();
	if(BackbufferSRV)BackbufferSRV->Release();
	if (DefaultSamplerState)DefaultSamplerState->Release();
	if (HUDRasterizerState)HUDRasterizerState->Release();
	if (WorldRasterizerState)WorldRasterizerState->Release();
	if (DefaultDepthStencilState)DefaultDepthStencilState->Release();
	
	if (BackbufferRTV)BackbufferRTV->Release();
	if (SwapChain)SwapChain->Release();
	
	if (Context)Context->Release();
	//if (DeferredContext)DeferredContext->Release();

	ID3D11Debug* d3dDebug;
	Device->QueryInterface(__uuidof(ID3D11Debug), reinterpret_cast<void**>(&d3dDebug));

	if(d3dDebug)
	{
		d3dDebug->ReportLiveDeviceObjects(D3D11_RLDO_DETAIL);
		d3dDebug->Release();
	}

	if(Device)Device->Release();
	
	if (DXGIAdapter)DXGIAdapter->Release();
	if (DXGIFactory)DXGIFactory->Release();

	//MemTrackerFinalReport();
}

/** Called when the game created it's window */
XRESULT D3D11GraphicsEngine::Init()
{
	HRESULT hr;

	LogInfo() << "Initializing Device...";

	// Create DXGI factory
	LE(CreateDXGIFactory(__uuidof(IDXGIFactory), (void **)&DXGIFactory));
	LE(DXGIFactory->EnumAdapters(0, &DXGIAdapter)); // Get first adapter

	// Find out what we are rendering on to write it into the logfile
	DXGI_ADAPTER_DESC adpDesc;
	DXGIAdapter->GetDesc(&adpDesc);

	std::wstring wDeviceDescription(adpDesc.Description);
	std::string deviceDescription(wDeviceDescription.begin(), wDeviceDescription.end());
	LogInfo() << "Rendering on: " << deviceDescription.c_str();

	int flags = D3D11_CREATE_DEVICE_BGRA_SUPPORT;

	// Create D3D11-Device
#ifndef DEBUG_D3D11
	LE(D3D11CreateDevice(DXGIAdapter, D3D_DRIVER_TYPE_UNKNOWN, NULL, flags, NULL, 0, D3D11_SDK_VERSION, &Device, NULL, &Context));
#else
	LE(D3D11CreateDevice(DXGIAdapter, D3D_DRIVER_TYPE_UNKNOWN, NULL, flags | D3D11_CREATE_DEVICE_DEBUG, NULL, 0, D3D11_SDK_VERSION, &Device, NULL, &Context));
#endif
	LE(Device->CreateDeferredContext(0, &DeferredContext)); // Used for multithreaded texture loading
	
	LogInfo() << "Creating ShaderManager";

	ShaderManager = new D3D11ShaderManager();
	ShaderManager->Init();
	ShaderManager->LoadShaders();

	TempVertexBuffer = new D3D11VertexBuffer();
	TempVertexBuffer->Init(NULL, DRAWVERTEXARRAY_BUFFER_SIZE, BaseVertexBuffer::B_VERTEXBUFFER, BaseVertexBuffer::U_DYNAMIC, BaseVertexBuffer::CA_WRITE);

	DynamicInstancingBuffer = new D3D11VertexBuffer();
	DynamicInstancingBuffer->Init(NULL, INSTANCING_BUFFER_SIZE, BaseVertexBuffer::B_VERTEXBUFFER, BaseVertexBuffer::U_DYNAMIC, BaseVertexBuffer::CA_WRITE);


	D3D11_SAMPLER_DESC samplerDesc;
	samplerDesc.Filter = D3D11_FILTER_ANISOTROPIC;
	samplerDesc.AddressU = D3D11_TEXTURE_ADDRESS_WRAP;
	samplerDesc.AddressV = D3D11_TEXTURE_ADDRESS_WRAP;
	samplerDesc.AddressW = D3D11_TEXTURE_ADDRESS_CLAMP;
	samplerDesc.MipLODBias = 0;
	samplerDesc.MaxAnisotropy = 16;
	samplerDesc.ComparisonFunc = D3D11_COMPARISON_NEVER;
	samplerDesc.BorderColor[0] = 1.0f;
	samplerDesc.BorderColor[1] = 1.0f;
	samplerDesc.BorderColor[2] = 1.0f;
	samplerDesc.BorderColor[3] = 1.0f;
	samplerDesc.MinLOD = -3.402823466e+38F; // -FLT_MAX
	samplerDesc.MaxLOD = 3.402823466e+38F; // FLT_MAX

	LE(Device->CreateSamplerState(&samplerDesc, &DefaultSamplerState));
	Context->PSSetSamplers(0, 1, &DefaultSamplerState);
	Context->VSSetSamplers(0, 1, &DefaultSamplerState);

	samplerDesc.Filter = D3D11_FILTER_COMPARISON_MIN_MAG_MIP_LINEAR;
	samplerDesc.ComparisonFunc = D3D11_COMPARISON_LESS_EQUAL;
	Device->CreateSamplerState(&samplerDesc, &ShadowmapSamplerState);
	Context->PSSetSamplers(2, 1, &ShadowmapSamplerState);

	D3D11_RASTERIZER_DESC rasterizerDesc;
	rasterizerDesc.CullMode = D3D11_CULL_BACK;
	rasterizerDesc.FillMode = D3D11_FILL_SOLID;
	rasterizerDesc.FrontCounterClockwise = true; // Gothics world vertices are CCW. That get's set by the GAPI-Graphics-State as well but is hardcoded here
	rasterizerDesc.DepthBias = 0;
	rasterizerDesc.DepthBiasClamp = 0;
	rasterizerDesc.SlopeScaledDepthBias = 0;
	rasterizerDesc.DepthClipEnable = false;
	rasterizerDesc.ScissorEnable = false;
	rasterizerDesc.MultisampleEnable = false;
	rasterizerDesc.AntialiasedLineEnable = true;

	LE(Device->CreateRasterizerState(&rasterizerDesc, &WorldRasterizerState));
	Context->RSSetState(WorldRasterizerState);

	rasterizerDesc.FrontCounterClockwise = false;
	LE(Device->CreateRasterizerState(&rasterizerDesc, &HUDRasterizerState));

	D3D11_DEPTH_STENCIL_DESC depthStencilDesc;
	// Depth test parameters
	depthStencilDesc.DepthEnable = true;
	depthStencilDesc.DepthWriteMask = D3D11_DEPTH_WRITE_MASK_ALL;
	depthStencilDesc.DepthFunc = D3D11_COMPARISON_LESS_EQUAL;

	// Stencil test parameters
	depthStencilDesc.StencilEnable = false;
	depthStencilDesc.StencilReadMask = 0xFF;
	depthStencilDesc.StencilWriteMask = 0xFF;

	// Stencil operations if pixel is front-facing
	depthStencilDesc.FrontFace.StencilFailOp = D3D11_STENCIL_OP_KEEP;
	depthStencilDesc.FrontFace.StencilDepthFailOp = D3D11_STENCIL_OP_INCR;
	depthStencilDesc.FrontFace.StencilPassOp = D3D11_STENCIL_OP_KEEP;
	depthStencilDesc.FrontFace.StencilFunc = D3D11_COMPARISON_ALWAYS;

	// Stencil operations if pixel is back-facing
	depthStencilDesc.BackFace.StencilFailOp = D3D11_STENCIL_OP_KEEP;
	depthStencilDesc.BackFace.StencilDepthFailOp = D3D11_STENCIL_OP_DECR;
	depthStencilDesc.BackFace.StencilPassOp = D3D11_STENCIL_OP_KEEP;
	depthStencilDesc.BackFace.StencilFunc = D3D11_COMPARISON_ALWAYS;

	LE(Device->CreateDepthStencilState(&depthStencilDesc, &DefaultDepthStencilState));

	SetActivePixelShader("PS_Simple");
	SetActiveVertexShader("VS_Ex");

	CreateTexture(&DistortionTexture);
	DistortionTexture->Init("system\\GD3D11\\textures\\distortion2.dds");

	CreateTexture(&NoiseTexture);
	NoiseTexture->Init("system\\GD3D11\\textures\\noise.png");

	InverseUnitSphereMesh = new GMesh;
	InverseUnitSphereMesh->LoadMesh("system\\GD3D11\\meshes\\icoSphere.obj");


	return XR_SUCCESS;
}

/** Called when the game created its window */
XRESULT D3D11GraphicsEngine::SetWindow(HWND hWnd)
{
	LogInfo() << "Creating swapchain";
	OutputWindow = hWnd;

	OnResize(Resolution);

	return XR_SUCCESS;
}

/** Called on window resize/resolution change */
XRESULT D3D11GraphicsEngine::OnResize(INT2 newSize)
{
	HRESULT hr;

	Resolution = newSize;

	SetWindowPos(OutputWindow, NULL, 0, 0, newSize.x, newSize.y, 0);

	// Release all referenced buffer resources before we can resize the swapchain
	if (BackbufferRTV)BackbufferRTV->Release();
	if (BackbufferSRV)BackbufferSRV->Release();
	delete DepthStencilBuffer;

	if(UIView)UIView->PrepareResize();

	if (!SwapChain)
	{
		LogInfo() << "Creating new swapchain! (Format: DXGI_FORMAT_R8G8B8A8_UNORM)";

		DXGI_SWAP_CHAIN_DESC scd;
		ZeroMemory(&scd, sizeof(DXGI_SWAP_CHAIN_DESC));

		scd.BufferCount = 1;
		scd.BufferDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
		scd.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT | DXGI_USAGE_SHADER_INPUT;
		scd.OutputWindow = OutputWindow;
		scd.SampleDesc.Count = 1;
		scd.SampleDesc.Quality = 0;
		scd.BufferDesc.Height = newSize.y;
		scd.BufferDesc.Width = newSize.x;
		scd.SwapEffect = DXGI_SWAP_EFFECT_DISCARD;
#ifdef PUBLIC_RELEASE
		scd.Windowed = false;
#else
		scd.Windowed = true;
#endif

		LE(DXGIFactory->CreateSwapChain(Device, &scd, &SwapChain));
		 
		if (!SwapChain)
		{
			LogError() << "Failed to create Swapchain! Program will now exit!";
			exit(0);
		}

		// Need to init AntTweakBar now that we have a working swapchain
		XLE(Engine::AntTweakBar->Init());
	}
	else
	{
		LogInfo() << "Resizing swapchain  (Format: DXGI_FORMAT_R8G8B8A8_UNORM)";

		if (FAILED(SwapChain->ResizeBuffers(1, newSize.x, newSize.y, DXGI_FORMAT_R8G8B8A8_UNORM, 0)))
		{
			LogError() << "Failed to resize swapchain!";
			return XR_FAILED;
		}
	}

	// Successfully resized swapchain, re-get buffers
	ID3D11Texture2D* backbuffer = NULL;
	SwapChain->GetBuffer(0, __uuidof(ID3D11Texture2D), (void **)&backbuffer);

	// Recreate RenderTargetView
	LE(Device->CreateRenderTargetView(backbuffer, nullptr, &BackbufferRTV));
	LE(Device->CreateShaderResourceView(backbuffer, nullptr, &BackbufferSRV));

	// Create main view
	if(!UIView)
	{
		UIView = new D2DView;
		UIView->Init(Resolution, backbuffer);
	}
	else
	{
		UIView->Resize(Resolution, backbuffer);
	}

	backbuffer->Release();

	// Recreate DepthStencilBuffer
	DepthStencilBuffer = new RenderToDepthStencilBuffer(Device, newSize.x, newSize.y, DXGI_FORMAT_R32_TYPELESS, NULL, DXGI_FORMAT_D32_FLOAT, DXGI_FORMAT_R32_FLOAT);

	// Bind our newly created resources
	Context->OMSetRenderTargets(1, &BackbufferRTV, DepthStencilBuffer->GetDepthStencilView());

	// Set the viewport
	D3D11_VIEWPORT viewport;
	ZeroMemory(&viewport, sizeof(D3D11_VIEWPORT));

	viewport.TopLeftX = 0;
	viewport.TopLeftY = 0;
	viewport.Width = (float)newSize.x;
	viewport.Height = (float)newSize.y;
	viewport.MinDepth = 0.0f;
	viewport.MaxDepth = 1.0f;

	Context->RSSetViewports(1, &viewport);

	// Create PFX-Renderer
	if(!PfxRenderer)
		PfxRenderer = new D3D11PfxRenderer;

	PfxRenderer->OnResize(newSize);

	delete CloudBuffer;
	CloudBuffer = new RenderToTextureBuffer(Device, newSize.x, newSize.y, DXGI_FORMAT_R8G8B8A8_UNORM);

	delete GBuffer1_Normals_SpecIntens_SpecPower;
	GBuffer1_Normals_SpecIntens_SpecPower = new RenderToTextureBuffer(Device, newSize.x, newSize.y, DXGI_FORMAT_R16G16B16A16_FLOAT);

	delete GBuffer0_Diffuse;
	GBuffer0_Diffuse = new RenderToTextureBuffer(Device, newSize.x, newSize.y, DXGI_FORMAT_R16G16B16A16_FLOAT);

	delete HDRBackBuffer;
	HDRBackBuffer = new RenderToTextureBuffer(Device, newSize.x, newSize.y, DXGI_FORMAT_R16G16B16A16_FLOAT);

	delete WorldShadowmap1;
	WorldShadowmap1 = new RenderToDepthStencilBuffer(Device, WORLD_SHADOWMAP_SIZE, WORLD_SHADOWMAP_SIZE, DXGI_FORMAT_R32_TYPELESS, NULL, DXGI_FORMAT_D32_FLOAT, DXGI_FORMAT_R32_FLOAT);


	
	
	return XR_SUCCESS;
}

/** Called when the game wants to render a new frame */
XRESULT D3D11GraphicsEngine::OnBeginFrame()
{
	// Check resolution
	RECT desktopRect;
	GetClientRect(GetDesktopWindow(), &desktopRect);
	if(desktopRect.right *RES_UPSCALE != GetResolution().x || desktopRect.bottom * RES_UPSCALE != GetResolution().y)
	{
		// Match the resolution with the current desktop resolution
		OnResize(INT2(desktopRect.right * RES_UPSCALE, desktopRect.bottom * RES_UPSCALE));
	}

	// Force the mode
	zCView::SetMode(desktopRect.right * RES_UPSCALE, desktopRect.bottom * RES_UPSCALE, 32);

	//return XR_SUCCESS;
	// Enter the critical section for safety while executing the deferred command list
	Engine::GAPI->EnterResourceCriticalSection();
	ID3D11CommandList* dc_cl = NULL;
	DeferredContext->FinishCommandList(true, &dc_cl);
	Engine::GAPI->LeaveResourceCriticalSection();
	if (dc_cl)
	{
		//LogInfo() << "Executing command list";
		Context->ExecuteCommandList(dc_cl, true);
		dc_cl->Release();
	}

	// Notify the shader manager
	ShaderManager->OnFrameStart();

	// Clear using the fog color
	//Clear(float4(0,0,0,0));
	Clear(float4(Engine::GAPI->GetRendererState()->GraphicsState.FF_FogColor, 0.0f));

	// Bind GBuffers
	//ID3D11RenderTargetView* rtvs[] = {GBuffer0_Diffuse->GetRenderTargetView(), GBuffer1_Normals_SpecIntens_SpecPower->GetRenderTargetView()};
	//Context->OMSetRenderTargets(2, rtvs, DepthStencilBuffer->GetDepthStencilView());
	Context->OMSetRenderTargets(1, HDRBackBuffer->GetRenderTargetViewPtr(), DepthStencilBuffer->GetDepthStencilView());

	SetActivePixelShader("PS_Simple");
	SetActiveVertexShader("VS_Ex");
	return XR_SUCCESS;
}

/** Called when the game ended it's frame */
XRESULT D3D11GraphicsEngine::OnEndFrame()
{
	Present();
	return XR_SUCCESS;
}

/** Called when the game wants to clear the bound rendertarget */
XRESULT D3D11GraphicsEngine::Clear(const float4& color)
{
	Context->ClearRenderTargetView(BackbufferRTV, (float *)&color);
	Context->ClearDepthStencilView(DepthStencilBuffer->GetDepthStencilView(), D3D11_CLEAR_DEPTH, 1.0f, 0);
	
	Context->ClearRenderTargetView(GBuffer0_Diffuse->GetRenderTargetView(), (float *)&color);
	Context->ClearRenderTargetView(GBuffer1_Normals_SpecIntens_SpecPower->GetRenderTargetView(), (float *)&D3DXVECTOR4(0,0,0,0));
	Context->ClearRenderTargetView(HDRBackBuffer->GetRenderTargetView(), (float *)&D3DXVECTOR4(0,0,0,0));
	
	return XR_SUCCESS;
}

/** Creates a vertexbuffer object (Not registered inside) */
XRESULT D3D11GraphicsEngine::CreateVertexBuffer(BaseVertexBuffer** outBuffer)
{
	*outBuffer = new D3D11VertexBuffer;
	return XR_SUCCESS;
}

/** Creates a texture object (Not registered inside) */
XRESULT D3D11GraphicsEngine::CreateTexture(BaseTexture** outTexture)
{
	*outTexture = new D3D11Texture;
	return XR_SUCCESS;
}

/** Creates a constantbuffer object (Not registered inside) */
XRESULT D3D11GraphicsEngine::CreateConstantBuffer(BaseConstantBuffer** outCB, void* data, int size)
{
	*outCB = new D3D11ConstantBuffer(size, data);
	return XR_SUCCESS;
}

/** Returns a list of available display modes */
XRESULT D3D11GraphicsEngine::GetDisplayModeList(std::vector<DisplayModeInfo>* modeList)
{
	HRESULT hr;
	UINT numModes = 0;
	DXGI_MODE_DESC* displayModes = NULL;
	DXGI_FORMAT format = DXGI_FORMAT_R8G8B8A8_UNORM;
	IDXGIOutput* output;

	if(!DXGIAdapter)
		return XR_FAILED;

	DXGIAdapter->EnumOutputs(0, &output);

	if(!output)
		return XR_FAILED;

	hr = output->GetDisplayModeList(format, 0, &numModes, NULL);

	displayModes = new DXGI_MODE_DESC[numModes];

	// Get the list
	hr = output->GetDisplayModeList(format, 0, &numModes, displayModes);

	for (unsigned int i = 0; i<numModes; i++)
	{
		if (displayModes[i].Format != format)
			continue;

		DisplayModeInfo info;
		info.Height = displayModes[i].Height;
		info.Width = displayModes[i].Width;
		info.Bpp = 32;

		modeList->push_back(info);
	}

	delete[] displayModes;

	output->Release();

	return XR_SUCCESS;
}

/** Presents the current frame to the screen */
XRESULT D3D11GraphicsEngine::Present()
{
	//Context->ClearRenderTargetView(PfxRenderer->GetTempBuffer()->GetRenderTargetView(), (float *)&float4(0,0,0,1));
	/*PfxRenderer->CopyTextureToRTV(BackbufferSRV, PfxRenderer->GetTempBuffer()->GetRenderTargetView());
	PfxRenderer->BlurTexture(PfxRenderer->GetTempBuffer());

	//Clear(float4(0,1,0,0));
	PfxRenderer->CopyTextureToRTV(PfxRenderer->GetTempBuffer()->GetShaderResView(), BackbufferRTV);*/



	// Copy HDR scene to backbuffer
	SetDefaultStates();
	SetActivePixelShader("PS_PFX_GammaCorrectInv");
	ActivePS->Apply();
	PfxRenderer->CopyTextureToRTV(HDRBackBuffer->GetShaderResView(), BackbufferRTV, INT2(0,0), true);

	Context->OMSetRenderTargets(1, &BackbufferRTV, NULL);

	// Draw ant tweak bar
	Engine::AntTweakBar->Draw();
	//LineRenderer->ClearCache();

	UIView->Render(Engine::GAPI->GetFrameTimeSec());

	Engine::GAPI->EnterResourceCriticalSection();
	if(SwapChain->Present(0, 0) == DXGI_ERROR_DEVICE_REMOVED)
	{
		switch(Device->GetDeviceRemovedReason())
		{
		case DXGI_ERROR_DEVICE_HUNG:
			LogInfoBox() << "Device Removed! (DXGI_ERROR_DEVICE_HUNG)";
			exit(0);
			break;

		case DXGI_ERROR_DEVICE_REMOVED:
			LogInfoBox() << "Device Removed! (DXGI_ERROR_DEVICE_REMOVED)";
			exit(0);
			break;

		case DXGI_ERROR_DEVICE_RESET:
			LogInfoBox() << "Device Removed! (DXGI_ERROR_DEVICE_RESET)";
			exit(0);
			break;

		case DXGI_ERROR_DRIVER_INTERNAL_ERROR:
			LogInfoBox() << "Device Removed! (DXGI_ERROR_DRIVER_INTERNAL_ERROR)";
			exit(0);
			break;

		case DXGI_ERROR_INVALID_CALL:
			LogInfoBox() << "Device Removed! (DXGI_ERROR_INVALID_CALL)";
			exit(0);
			break;

		case S_OK:
			LogInfo() << "Device removed, but we're fine!";
			break;

		default:
			LogInfoBox() << "Device Removed! (Unknown reason)";
		}
	}
	Engine::GAPI->LeaveResourceCriticalSection();

	PresentPending = false;

	return XR_SUCCESS;
}

/** Called to set the current viewport */
XRESULT D3D11GraphicsEngine::SetViewport(const ViewportInfo& viewportInfo)
{
	// Set the viewport
	D3D11_VIEWPORT viewport;
	ZeroMemory(&viewport, sizeof(D3D11_VIEWPORT));

	viewport.TopLeftX = (float)viewportInfo.TopLeftX;
	viewport.TopLeftY = (float)viewportInfo.TopLeftY;
	viewport.Width = (float)viewportInfo.Width;
	viewport.Height = (float)viewportInfo.Height;
	viewport.MinDepth = viewportInfo.MinZ;
	viewport.MaxDepth = viewportInfo.MaxZ;

	Context->RSSetViewports(1, &viewport);

	return XR_SUCCESS;
}

/** Draws a vertexbuffer, non-indexed (World)*/
XRESULT D3D11GraphicsEngine::DrawVertexBuffer(BaseVertexBuffer* vb, unsigned int numVertices, unsigned int stride)
{
	UINT offset = 0;
	UINT uStride = stride;
	ID3D11Buffer* buffer = ((D3D11VertexBuffer *)vb)->GetVertexBuffer();
	Context->IASetVertexBuffers(0, 1, &buffer, &uStride, &offset);

	//Draw the mesh
	Context->Draw(numVertices, 0);

	Engine::GAPI->GetRendererState()->RendererInfo.FrameDrawnTriangles += numVertices;

	return XR_SUCCESS;
}

/** Draws a vertexbuffer, non-indexed (VOBs)*/
XRESULT D3D11GraphicsEngine::DrawVertexBufferIndexed(BaseVertexBuffer* vb, BaseVertexBuffer* ib, unsigned int numIndices)
{
	UINT offset = 0;
	UINT uStride = sizeof(ExVertexStruct);
	ID3D11Buffer* buffer = ((D3D11VertexBuffer *)vb)->GetVertexBuffer();
	Context->IASetVertexBuffers(0, 1, &buffer, &uStride, &offset);

	if(sizeof(VERTEX_INDEX) == sizeof(unsigned short))
	{
		Context->IASetIndexBuffer(((D3D11VertexBuffer *)ib)->GetVertexBuffer(), DXGI_FORMAT_R16_UINT, 0);
	}else
	{
		Context->IASetIndexBuffer(((D3D11VertexBuffer *)ib)->GetVertexBuffer(), DXGI_FORMAT_R32_UINT, 0);
	}

	//Draw the mesh
	Context->DrawIndexed(numIndices, 0, 0);

	Engine::GAPI->GetRendererState()->RendererInfo.FrameDrawnTriangles += numIndices / 3;

	return XR_SUCCESS;
}

/** Binds viewport information to the given constantbuffer slot */
XRESULT D3D11GraphicsEngine::BindViewportInformation(const std::string& shader, int slot) 
{
	D3D11_VIEWPORT vp;
	UINT num = 1;
	Context->RSGetViewports(&num, &vp);

	// Update viewport information
	Temp2Float2[0].x = vp.TopLeftX;
	Temp2Float2[0].y = vp.TopLeftY;
	Temp2Float2[1].x = vp.Width;
	Temp2Float2[1].y = vp.Height;

	D3D11PShader* ps = ShaderManager->GetPShader(shader);
	D3D11VShader* vs = ShaderManager->GetVShader(shader);

	if(vs)
	{
		vs->GetConstantBuffer()[slot]->UpdateBuffer(Temp2Float2);
		vs->GetConstantBuffer()[slot]->BindToVertexShader(slot);
	}

	if(ps)
	{
		ps->GetConstantBuffer()[slot]->UpdateBuffer(Temp2Float2);
		ps->GetConstantBuffer()[slot]->BindToVertexShader(slot);
	}

	return XR_SUCCESS;
}

/** Draws a vertexarray, non-indexed (HUD, 2D)*/
XRESULT D3D11GraphicsEngine::DrawVertexArray(ExVertexStruct* vertices, unsigned int numVertices, unsigned int startVertex, unsigned int stride)
{
	UpdateRenderStates();
	D3D11VShader* vShader = ActiveVS;//ShaderManager->GetVShader("VS_TransformedEx");
	
	// Bind the FF-Info to the first PS slot
	ActivePS->GetConstantBuffer()[0]->UpdateBuffer(&Engine::GAPI->GetRendererState()->GraphicsState);
	ActivePS->GetConstantBuffer()[0]->BindToPixelShader(0);

	SetupVS_ExMeshDrawCall();

	TempVertexBuffer->UpdateBuffer(vertices, stride * numVertices);

	UINT offset = 0;
	UINT uStride = stride;
	ID3D11Buffer* buffer = TempVertexBuffer->GetVertexBuffer();
	Context->IASetVertexBuffers(0, 1, &buffer, &uStride, &offset);

	//Draw the mesh
	Context->Draw(numVertices, startVertex);

	Engine::GAPI->GetRendererState()->RendererInfo.FrameDrawnTriangles += numVertices;

	return XR_SUCCESS;
}

/** Draws a vertexbuffer, non-indexed, binding the FF-Pipe values */
XRESULT D3D11GraphicsEngine::DrawVertexBufferFF(BaseVertexBuffer* vb, unsigned int numVertices, unsigned int startVertex, unsigned int stride)
{
	SetupVS_ExMeshDrawCall();

	// Bind the FF-Info to the first PS slot
	ActivePS->GetConstantBuffer()[0]->UpdateBuffer(&Engine::GAPI->GetRendererState()->GraphicsState);
	ActivePS->GetConstantBuffer()[0]->BindToPixelShader(0);

	UINT offset = 0;
	UINT uStride = stride;
	ID3D11Buffer* buffer = ((D3D11VertexBuffer *)vb)->GetVertexBuffer();
	Context->IASetVertexBuffers(0, 1, &buffer, &uStride, &offset);

	//Draw the mesh
	Context->Draw(numVertices, startVertex);

	Engine::GAPI->GetRendererState()->RendererInfo.FrameDrawnTriangles += numVertices;

	return XR_SUCCESS;
}

/** Draws a skeletal mesh */
XRESULT D3D11GraphicsEngine::DrawSkeletalMesh(BaseVertexBuffer* vb, BaseVertexBuffer* ib, unsigned int numIndices, const std::vector<D3DXMATRIX>& transforms)
{
	Context->RSSetState(WorldRasterizerState);
	Context->OMSetDepthStencilState(DefaultDepthStencilState, 0);
	SetActiveVertexShader("VS_ExSkeletal");

	SetActivePixelShader("PS_AtmosphereGround");
	D3D11PShader* nrmPS = ActivePS;
	SetActivePixelShader("PS_World");
	D3D11PShader* defaultPS = ActivePS;

	// Querry the second texture slot to see if there is a normalmap bound
	ID3D11ShaderResourceView* nrmmap;
	Context->PSGetShaderResources(1,1, &nrmmap);
	if(!nrmmap)
	{
		if(ActivePS != defaultPS)
		{
			ActivePS = defaultPS;
			ActivePS->Apply();
		}
	}else
	{
		if(ActivePS != nrmPS)
		{
			ActivePS = nrmPS;
			ActivePS->Apply();
		}
		nrmmap->Release();

		// Get currently bound texture name
		zCTexture* tex = Engine::GAPI->GetBoundTexture(0);

		if(tex)
		{
			MaterialInfo* info = Engine::GAPI->GetMaterialInfoFrom(tex);
			ActivePS->GetConstantBuffer()[2]->UpdateBuffer(info);
			ActivePS->GetConstantBuffer()[2]->BindToPixelShader(2);
		}
	}

	D3DXMATRIX& world = Engine::GAPI->GetRendererState()->TransformState.TransformWorld;
	D3DXMATRIX& view = Engine::GAPI->GetRendererState()->TransformState.TransformView;
	D3DXMATRIX& proj = Engine::GAPI->GetProjectionMatrix();

	SetupVS_ExMeshDrawCall();
	SetupVS_ExConstantBuffer();

	VS_ExConstantBuffer_PerInstance cb2;
	cb2.World = world;

	ActiveVS->GetConstantBuffer()[1]->UpdateBuffer(&cb2);
	ActiveVS->GetConstantBuffer()[1]->BindToVertexShader(1);

	memcpy(TempBonesD3DXmatrix, &transforms[0], sizeof(D3DXMATRIX) * transforms.size());
	ActiveVS->GetConstantBuffer()[2]->UpdateBuffer(TempBonesD3DXmatrix);
	ActiveVS->GetConstantBuffer()[2]->BindToVertexShader(2);

	ActiveVS->Apply();
	ActivePS->Apply();

	Context->IASetPrimitiveTopology(D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
	UINT offset = 0;
	UINT uStride = sizeof(ExSkelVertexStruct);
	ID3D11Buffer* buffer = ((D3D11VertexBuffer *)vb)->GetVertexBuffer();
	Context->IASetVertexBuffers(0, 1, &buffer, &uStride, &offset);

	if(sizeof(VERTEX_INDEX) == sizeof(unsigned short))
	{
		Context->IASetIndexBuffer(((D3D11VertexBuffer *)ib)->GetVertexBuffer(), DXGI_FORMAT_R16_UINT, 0);
	}else
	{
		Context->IASetIndexBuffer(((D3D11VertexBuffer *)ib)->GetVertexBuffer(), DXGI_FORMAT_R32_UINT, 0);
	}

	//Draw the mesh
	Context->DrawIndexed(numIndices, 0, 0);

	return XR_SUCCESS;
}

/** Draws a batch of instanced geometry */
XRESULT D3D11GraphicsEngine::DrawInstanced(BaseVertexBuffer* vb, BaseVertexBuffer* ib, unsigned int numIndices, void* instanceData, unsigned int instanceDataStride, unsigned int numInstances, unsigned int vertexStride)
{
	UpdateRenderStates();

	// Check buffersize
	D3D11_BUFFER_DESC desc;
	((D3D11VertexBuffer *)DynamicInstancingBuffer)->GetVertexBuffer()->GetDesc(&desc);

	if(desc.ByteWidth < instanceDataStride * numInstances)
	{
		LogInfo() << "Instancing buffer too small (" << desc.ByteWidth << "), need " << instanceDataStride * numInstances << " bytes. Recreating buffer.";

		// Buffer too small, recreate it
		delete DynamicInstancingBuffer;
		DynamicInstancingBuffer = new D3D11VertexBuffer();
		DynamicInstancingBuffer->Init(NULL, instanceDataStride * numInstances, BaseVertexBuffer::B_VERTEXBUFFER, BaseVertexBuffer::U_DYNAMIC, BaseVertexBuffer::CA_WRITE);
	}

	// Update the vertexbuffer
	DynamicInstancingBuffer->UpdateBuffer(instanceData, instanceDataStride * numInstances);

	// Bind shader and pipeline flags
	//Context->RSSetState(WorldRasterizerState);
	//Context->OMSetDepthStencilState(DefaultDepthStencilState, 0);
	D3D11VShader* vShader = ShaderManager->GetVShader("VS_ExInstanced");
	SetActivePixelShader("PS_Simple");

	D3DXMATRIX& world = Engine::GAPI->GetRendererState()->TransformState.TransformWorld;
	D3DXMATRIX& view = Engine::GAPI->GetRendererState()->TransformState.TransformView;
	D3DXMATRIX& proj = Engine::GAPI->GetProjectionMatrix();

	
	VS_ExConstantBuffer_PerFrame cb;
	cb.View = view;
	cb.Projection = proj;

	VS_ExConstantBuffer_PerInstance cb2;
	cb2.World = world;




	vShader->GetConstantBuffer()[0]->UpdateBuffer(&cb);
	vShader->GetConstantBuffer()[0]->BindToVertexShader(0);

	//vShader->GetConstantBuffer()[1]->UpdateBuffer(&cb2);
	//vShader->GetConstantBuffer()[1]->BindToVertexShader(1);

	vShader->Apply();
	ActivePS->Apply();

	Context->IASetPrimitiveTopology(D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
	UINT offset[] = {0,0};
	UINT uStride[] = {vertexStride, instanceDataStride};
	ID3D11Buffer* buffers[] = {((D3D11VertexBuffer *)vb)->GetVertexBuffer(),
		((D3D11VertexBuffer *)DynamicInstancingBuffer)->GetVertexBuffer()};
	Context->IASetVertexBuffers(0, 2, buffers, uStride, offset);

	if(sizeof(VERTEX_INDEX) == sizeof(unsigned short))
	{
		Context->IASetIndexBuffer(((D3D11VertexBuffer *)ib)->GetVertexBuffer(), DXGI_FORMAT_R16_UINT, 0);
	}else
	{
		Context->IASetIndexBuffer(((D3D11VertexBuffer *)ib)->GetVertexBuffer(), DXGI_FORMAT_R32_UINT, 0);
	}

	//Draw the batch
	Context->DrawIndexedInstanced(numIndices, numInstances, 0, 0, 0);

	return XR_SUCCESS;
}

/** Draws a batch of instanced geometry */
XRESULT D3D11GraphicsEngine::DrawInstanced(BaseVertexBuffer* vb, BaseVertexBuffer* ib, unsigned int numIndices, BaseVertexBuffer* instanceData, unsigned int instanceDataStride, unsigned int numInstances, unsigned int vertexStride)
{
	// Bind shader and pipeline flags
	UINT offset[] = {0,0};
	UINT uStride[] = {vertexStride, instanceDataStride};
	ID3D11Buffer* buffers[] = {((D3D11VertexBuffer *)vb)->GetVertexBuffer(),
		((D3D11VertexBuffer *)instanceData)->GetVertexBuffer()};
	Context->IASetVertexBuffers(0, 2, buffers, uStride, offset);

	if(sizeof(VERTEX_INDEX) == sizeof(unsigned short))
	{
		Context->IASetIndexBuffer(((D3D11VertexBuffer *)ib)->GetVertexBuffer(), DXGI_FORMAT_R16_UINT, 0);
	}else
	{
		Context->IASetIndexBuffer(((D3D11VertexBuffer *)ib)->GetVertexBuffer(), DXGI_FORMAT_R32_UINT, 0);
	}

	unsigned int max = Engine::GAPI->GetRendererState()->RendererSettings.MaxNumFaces * 3;
	numIndices = max != 0 ? (numIndices < max ? numIndices : max) : numIndices;

	//Draw the batch
	Context->DrawIndexedInstanced(numIndices, numInstances, 0, 0, 0);

	Engine::GAPI->GetRendererState()->RendererInfo.FrameDrawnVobs++;

	return XR_SUCCESS;
}

/** Sets the active pixel shader object */
XRESULT D3D11GraphicsEngine::SetActivePixelShader(const std::string& shader)
{
	ActivePS = ShaderManager->GetPShader(shader);

	return XR_SUCCESS;
}

XRESULT D3D11GraphicsEngine::SetActiveVertexShader(const std::string& shader)
{
	ActiveVS = ShaderManager->GetVShader(shader);

	return XR_SUCCESS;
}

/** Unbinds the texture at the given slot */
XRESULT D3D11GraphicsEngine::UnbindTexture(int slot)
{
	ID3D11ShaderResourceView* srv = NULL;
	Context->PSSetShaderResources(slot, 1, &srv);
	Context->VSSetShaderResources(slot, 1, &srv);

	return XR_SUCCESS;
}

/** Recreates the renderstates */
XRESULT D3D11GraphicsEngine::UpdateRenderStates()
{
	if (Engine::GAPI->GetRendererState()->BlendStateDirty)
	{
		if (FFBlendState)FFBlendState->Release();

		GothicBlendStateInfo& bs = Engine::GAPI->GetRendererState()->BlendState;

		D3D11_BLEND_DESC blendDesc;
		// Set to default
		blendDesc.AlphaToCoverageEnable = bs.AlphaToCoverage;
		blendDesc.IndependentBlendEnable = FALSE;
		blendDesc.RenderTarget[0].RenderTargetWriteMask = D3D11_COLOR_WRITE_ENABLE_RED | 
			D3D11_COLOR_WRITE_ENABLE_BLUE | 
			D3D11_COLOR_WRITE_ENABLE_GREEN |
			D3D11_COLOR_WRITE_ENABLE_ALPHA;
		blendDesc.RenderTarget[0].SrcBlend = (D3D11_BLEND)bs.SrcBlend;
		blendDesc.RenderTarget[0].DestBlend = (D3D11_BLEND)bs.DestBlend;
		blendDesc.RenderTarget[0].BlendOp = (D3D11_BLEND_OP)bs.BlendOp;
		blendDesc.RenderTarget[0].SrcBlendAlpha = (D3D11_BLEND)bs.SrcBlendAlpha;
		blendDesc.RenderTarget[0].DestBlendAlpha = (D3D11_BLEND)bs.DestBlendAlpha;
		blendDesc.RenderTarget[0].BlendOpAlpha = (D3D11_BLEND_OP)bs.BlendOpAlpha;
		blendDesc.RenderTarget[0].BlendEnable = bs.BlendEnabled;

		Device->CreateBlendState(&blendDesc, &FFBlendState);
		

		Engine::GAPI->GetRendererState()->BlendStateDirty = false;
		Context->OMSetBlendState(FFBlendState, (float *)&D3DXVECTOR4(0, 0, 0, 0), 0xFFFFFFFF);
	}

	

	if (Engine::GAPI->GetRendererState()->RasterizerStateDirty)
	{
		if (FFRasterizerState)FFRasterizerState->Release();

		GothicRasterizerStateInfo& rs = Engine::GAPI->GetRendererState()->RasterizerState;

		D3D11_RASTERIZER_DESC rasterizerDesc;
		rasterizerDesc.CullMode = (D3D11_CULL_MODE)rs.CullMode;

		if(rs.Wireframe)
			rasterizerDesc.FillMode = D3D11_FILL_WIREFRAME;
		else
			rasterizerDesc.FillMode = D3D11_FILL_SOLID;

		rasterizerDesc.FrontCounterClockwise = rs.FrontCounterClockwise;
		rasterizerDesc.DepthBias = rs.ZBias;
		rasterizerDesc.DepthBiasClamp = 0;
		rasterizerDesc.SlopeScaledDepthBias = 0;
		rasterizerDesc.DepthClipEnable = false;
		rasterizerDesc.ScissorEnable = false;
		rasterizerDesc.MultisampleEnable = false;
		rasterizerDesc.AntialiasedLineEnable = true;

		Device->CreateRasterizerState(&rasterizerDesc, &FFRasterizerState);
		

		Engine::GAPI->GetRendererState()->RasterizerStateDirty = false;
		Context->RSSetState(FFRasterizerState);
	}

	

	if (Engine::GAPI->GetRendererState()->DepthStateDirty)
	{
		if (FFDepthStencilState)FFDepthStencilState->Release();

		GothicDepthBufferStateInfo& ds = Engine::GAPI->GetRendererState()->DepthState;

		D3D11_DEPTH_STENCIL_DESC depthStencilDesc;

		// Depth test parameters
		depthStencilDesc.DepthEnable = ds.DepthBufferEnabled;

		if(ds.DepthWriteEnabled)
		{
			depthStencilDesc.DepthWriteMask = D3D11_DEPTH_WRITE_MASK_ALL;
		}else
		{
			depthStencilDesc.DepthWriteMask = D3D11_DEPTH_WRITE_MASK_ZERO;
		}
		depthStencilDesc.DepthFunc = (D3D11_COMPARISON_FUNC)ds.DepthBufferCompareFunc;

		// Stencil test parameters
		depthStencilDesc.StencilEnable = false;
		depthStencilDesc.StencilReadMask = 0xFF;
		depthStencilDesc.StencilWriteMask = 0xFF;

		// Stencil operations if pixel is front-facing
		depthStencilDesc.FrontFace.StencilFailOp = D3D11_STENCIL_OP_KEEP;
		depthStencilDesc.FrontFace.StencilDepthFailOp = D3D11_STENCIL_OP_INCR;
		depthStencilDesc.FrontFace.StencilPassOp = D3D11_STENCIL_OP_KEEP;
		depthStencilDesc.FrontFace.StencilFunc = D3D11_COMPARISON_ALWAYS;

		// Stencil operations if pixel is back-facing
		depthStencilDesc.BackFace.StencilFailOp = D3D11_STENCIL_OP_KEEP;
		depthStencilDesc.BackFace.StencilDepthFailOp = D3D11_STENCIL_OP_DECR;
		depthStencilDesc.BackFace.StencilPassOp = D3D11_STENCIL_OP_KEEP;
		depthStencilDesc.BackFace.StencilFunc = D3D11_COMPARISON_ALWAYS;

		Device->CreateDepthStencilState(&depthStencilDesc, &FFDepthStencilState);

		Engine::GAPI->GetRendererState()->DepthStateDirty = false;
		Context->OMSetDepthStencilState(FFDepthStencilState, 0);
	}

	

	return XR_SUCCESS;
}

/** Called when we started to render the world */
XRESULT D3D11GraphicsEngine::OnStartWorldRendering()
{
	//Clear(float4(0,0,0,0));
	//Clear(float4(0xFF44AEFF));

	//return XR_SUCCESS;
	if(PresentPending)
		return XR_SUCCESS;

	ID3D11RenderTargetView* rtvs[] = {GBuffer0_Diffuse->GetRenderTargetView(), GBuffer1_Normals_SpecIntens_SpecPower->GetRenderTargetView()};
	Context->OMSetRenderTargets(2, rtvs, DepthStencilBuffer->GetDepthStencilView());

	Engine::GAPI->SetFarPlane(DEFAULT_FAR_PLANE);

	// Clear textures from the last frame
	FrameTextures.clear();

	Engine::GAPI->GetRendererState()->RasterizerState.FrontCounterClockwise = false;
	Engine::GAPI->GetRendererState()->RasterizerStateDirty = true;

	if(Engine::GAPI->GetRendererState()->RendererSettings.DrawSky)
	{
		// Draw back of the sky
		DrawSky();
	}

	// Draw world
	Engine::GAPI->DrawWorldMeshNaive();

	SetDefaultStates();
	
	if(Engine::GAPI->GetRendererState()->RendererSettings.DrawFog)
		PfxRenderer->RenderHeightfog();

	//PfxRenderer->RenderDistanceBlur();
	
	SetActivePixelShader("PS_Simple");
	SetActiveVertexShader("VS_Ex");

	if(Engine::GAPI->GetRendererState()->RendererSettings.DrawSky)
	{
		// Draw clouds
		//Draw3DClouds();
	}

	SetActivePixelShader("PS_Simple");
	SetActiveVertexShader("VS_Ex");

	// Update editor
	UIView->Update(Engine::GAPI->GetFrameTimeSec());

	SetDefaultStates();
	// Draw debug lines
	LineRenderer->Flush();
	
	if(Engine::GAPI->GetRendererState()->RendererSettings.EnableHDR)
		PfxRenderer->RenderHDR();

	//PfxRenderer->CopyTextureToRTV(GBuffer1_Normals_SpecIntens_SpecPower->GetShaderResView(), BackbufferRTV, INT2(Resolution.x / 4, Resolution.y / 4));

	PresentPending = true;

	return XR_SUCCESS;
}


void D3D11GraphicsEngine::SetupVS_ExMeshDrawCall()
{
	UpdateRenderStates();

	ActiveVS->Apply();
	ActivePS->Apply();

	Context->IASetPrimitiveTopology(D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
}

void D3D11GraphicsEngine::SetupVS_ExConstantBuffer()
{
	D3DXMATRIX& world = Engine::GAPI->GetRendererState()->TransformState.TransformWorld;
	D3DXMATRIX& view = Engine::GAPI->GetRendererState()->TransformState.TransformView;
	D3DXMATRIX& proj = Engine::GAPI->GetProjectionMatrix();

	VS_ExConstantBuffer_PerFrame cb;
	cb.View = view;
	cb.Projection = proj;
	cb.ViewProj = proj * view;

	ActiveVS->GetConstantBuffer()[0]->UpdateBuffer(&cb);
	ActiveVS->GetConstantBuffer()[0]->BindToVertexShader(0);
}

void D3D11GraphicsEngine::SetupVS_ExPerInstanceConstantBuffer()
{
	D3DXMATRIX& world = Engine::GAPI->GetRendererState()->TransformState.TransformWorld;

	VS_ExConstantBuffer_PerInstance cb;
	cb.World = world;

	ActiveVS->GetConstantBuffer()[1]->UpdateBuffer(&cb);
	ActiveVS->GetConstantBuffer()[1]->BindToVertexShader(1);
}

bool SectionRenderlistSortCmp(std::pair<float, WorldMeshSectionInfo*>& a, std::pair<float, WorldMeshSectionInfo*>& b)
{
	return a.first < b.first;
}

/** Draws the world mesh */
XRESULT D3D11GraphicsEngine::DrawWorldMesh(bool noTextures)
{
	if(!Engine::GAPI->GetRendererState()->RendererSettings.DrawWorldMesh)
		return XR_SUCCESS;


	D3DXVECTOR3 camPos = Engine::GAPI->GetCameraPosition();

	//Engine::GAPI->SetFarPlane(DEFAULT_FAR_PLANE);

	INT2 camSection = WorldConverter::GetSectionOfPos(camPos);

	D3DXMATRIX view;
	Engine::GAPI->GetViewMatrix(&view);

	// Setup renderstates
	Engine::GAPI->GetRendererState()->RasterizerState.SetDefault();
	Engine::GAPI->GetRendererState()->RasterizerState.CullMode = GothicRasterizerStateInfo::CM_CULL_BACK;
	Engine::GAPI->GetRendererState()->RasterizerStateDirty = true;

	Engine::GAPI->GetRendererState()->DepthState.SetDefault();
	Engine::GAPI->GetRendererState()->DepthStateDirty = true;

	

	Engine::GAPI->ResetWorldTransform();
	Engine::GAPI->SetViewTransform(view);

	// Set shader
	SetActivePixelShader("PS_AtmosphereGround");
	D3D11PShader* nrmPS = ActivePS;
	SetActivePixelShader("PS_World");
	D3D11PShader* defaultPS = ActivePS;
	SetActiveVertexShader("VS_Ex");

	// Set constant buffer
	ActivePS->GetConstantBuffer()[0]->UpdateBuffer(&Engine::GAPI->GetRendererState()->GraphicsState);
	ActivePS->GetConstantBuffer()[0]->BindToPixelShader(0);

	GSky* sky = Engine::GAPI->GetSky();
	ActivePS->GetConstantBuffer()[1]->UpdateBuffer(&sky->GetAtmosphereCB());
	ActivePS->GetConstantBuffer()[1]->BindToPixelShader(1);

	if(Engine::GAPI->GetRendererState()->RendererSettings.WireframeWorld)
	{
		Engine::GAPI->GetRendererState()->RasterizerState.Wireframe = true;
	}

	// Init drawcalls
	SetupVS_ExMeshDrawCall();
	SetupVS_ExConstantBuffer();

	D3DXMATRIX id;
	D3DXMatrixIdentity(&id);
	ActiveVS->GetConstantBuffer()[1]->UpdateBuffer(&id);
	ActiveVS->GetConstantBuffer()[1]->BindToVertexShader(1);

	int numSections = 0;
	int sectionViewDist = Engine::GAPI->GetRendererState()->RendererSettings.SectionDrawRadius;

	std::list<WorldMeshSectionInfo*> renderList;

	Engine::GAPI->CollectVisibleSections(renderList);

	// Sort the list to help pixel-culling
	//renderList.sort(SectionRenderlistSortCmp);


	//noTextures = true;
	std::hash_map<zCMaterial*, std::list<MeshInfo*>> meshesByMaterial;
	for(std::list<WorldMeshSectionInfo*>::iterator itr = renderList.begin(); itr != renderList.end(); itr++)
	{
		numSections++;

		//GetLineRenderer()->AddAABBMinMax((*itr)->BoundingBox.Min, (*itr)->BoundingBox.Max);

		// Draw world mesh
		/*DistortionTexture->BindToPixelShader(0);
		if((*itr)->FullStaticMesh)
			Engine::GAPI->DrawMeshInfo(NULL, (*itr)->FullStaticMesh);

		continue;*/

		for(std::map<MeshKey, MeshInfo*>::iterator it = (*itr)->WorldMeshes.begin(); it != (*itr)->WorldMeshes.end();it++)
		{
			meshesByMaterial[(*it).first.Material].push_back((*it).second);
		}
	}

	for(std::hash_map<zCMaterial*, std::list<MeshInfo*>>::iterator it = meshesByMaterial.begin(); it != meshesByMaterial.end();it++)
	{
		if(!(*it).first || !(*it).first->GetTexture())
		{
			DistortionTexture->BindToPixelShader(0);
		}else
		{
			FrameTextures.insert((*it).first->GetTexture());

			/*MaterialInfo* info = Engine::GAPI->GetMaterialInfoFrom((*it).first->GetTexture());
			ActivePS->GetConstantBuffer()[2]->UpdateBuffer(info);
			ActivePS->GetConstantBuffer()[2]->BindToPixelShader(2);*/

			// Bind texture
			if((*it).first->GetTexture()->CacheIn(0.6f) == zRES_CACHED_IN)
				(*it).first->GetTexture()->Bind(0);

			// TODO: TESTCODE
			/*if(strnicmp((*it).first.Material->GetTexture()->GetNameWithoutExt().c_str(), "NW_Harbour_Stairs", strlen("NW_Harbour_Stairs")) == 0)
			{
			SetActiveVertexShader("VS_ExDisplace");
			ActiveVS->Apply();
			}else
			{
			SetActiveVertexShader("VS_Ex");
			ActiveVS->Apply();
			}*/


			// Querry the second texture slot to see if there is a normalmap bound
			ID3D11ShaderResourceView* nrmmap;
			Context->PSGetShaderResources(1,1, &nrmmap);
			if(!nrmmap)
			{
				if(ActivePS != defaultPS)
				{
					ActivePS = defaultPS;
					ActivePS->Apply();
				}
			}else
			{
				if(ActivePS != nrmPS)
				{
					ActivePS = nrmPS;
					ActivePS->Apply();
				}
				nrmmap->Release();
			}
		}

		for(std::list<MeshInfo*>::iterator itr = (*it).second.begin(); itr != (*it).second.end(); itr++)
		{
			Engine::GAPI->DrawMeshInfo((*it).first, (*itr));
		}
	}

	if(Engine::GAPI->GetRendererState()->RendererSettings.WireframeWorld)
	{
		Engine::GAPI->GetRendererState()->RasterizerState.Wireframe = false;
	}

	Engine::GAPI->GetRendererState()->RendererInfo.FrameNumSectionsDrawn = numSections;
	Engine::GAPI->GetRendererState()->RasterizerState.CullMode = GothicRasterizerStateInfo::CM_CULL_FRONT;
	Engine::GAPI->GetRendererState()->RasterizerStateDirty = true;

	return XR_SUCCESS;
}

/** Draws everything around the given position */
void D3D11GraphicsEngine::DrawWorldAround(const D3DXVECTOR3& position, int sectionRange, float vobXZRange)
{
	// Setup renderstates
	Engine::GAPI->GetRendererState()->RasterizerState.SetDefault();
	Engine::GAPI->GetRendererState()->RasterizerState.CullMode = GothicRasterizerStateInfo::CM_CULL_BACK;
	Engine::GAPI->GetRendererState()->RasterizerStateDirty = true;

	Engine::GAPI->GetRendererState()->DepthState.SetDefault();
	Engine::GAPI->GetRendererState()->DepthStateDirty = true;

	D3DXMATRIX view;
	Engine::GAPI->GetViewMatrix(&view);

	Engine::GAPI->ResetWorldTransform();
	Engine::GAPI->SetViewTransform(view);

	// Set shader
	SetActivePixelShader("PS_AtmosphereGround");
	D3D11PShader* nrmPS = ActivePS;
	SetActivePixelShader("PS_World");
	D3D11PShader* defaultPS = ActivePS;
	SetActiveVertexShader("VS_Ex");

	// Set constant buffer
	ActivePS->GetConstantBuffer()[0]->UpdateBuffer(&Engine::GAPI->GetRendererState()->GraphicsState);
	ActivePS->GetConstantBuffer()[0]->BindToPixelShader(0);

	GSky* sky = Engine::GAPI->GetSky();
	ActivePS->GetConstantBuffer()[1]->UpdateBuffer(&sky->GetAtmosphereCB());
	ActivePS->GetConstantBuffer()[1]->BindToPixelShader(1);

	// Init drawcalls
	SetupVS_ExMeshDrawCall();
	SetupVS_ExConstantBuffer();

	D3DXMATRIX id;
	D3DXMatrixIdentity(&id);
	ActiveVS->GetConstantBuffer()[1]->UpdateBuffer(&id);
	ActiveVS->GetConstantBuffer()[1]->BindToVertexShader(1);

	// Update and bind buffer of PS
	PerObjectState ocb;
	ocb.OS_AmbientColor = float3(1,1,1);
	ActivePS->GetConstantBuffer()[3]->UpdateBuffer(&ocb);
	ActivePS->GetConstantBuffer()[3]->BindToPixelShader(3);

	INT2 s = WorldConverter::GetSectionOfPos(position);
	D3DXVECTOR2 camXZ = D3DXVECTOR2(position.x, position.z);

	float vobOutdoorDist = Engine::GAPI->GetRendererState()->RendererSettings.OutdoorVobDrawRadius;
	float vobOutdoorSmallDist = Engine::GAPI->GetRendererState()->RendererSettings.OutdoorSmallVobDrawRadius;
	float vobSmallSize = Engine::GAPI->GetRendererState()->RendererSettings.SmallVobSize;

	DistortionTexture->BindToPixelShader(0);

	for(std::map<int, std::map<int, WorldMeshSectionInfo>>::iterator itx = Engine::GAPI->GetWorldSections().begin(); itx != Engine::GAPI->GetWorldSections().end(); itx++)
	{
		for(std::map<int, WorldMeshSectionInfo>::iterator ity = (*itx).second.begin(); ity != (*itx).second.end(); ity++)
		{
			D3DXVECTOR2 a = D3DXVECTOR2((float)((*itx).first - s.x), (float)((*ity).first - s.y));
			if(D3DXVec2Length(&a) < sectionRange)
			{
				WorldMeshSectionInfo& section = (*ity).second;

				D3DXMATRIX id;
				D3DXMatrixIdentity(&id);
				ActiveVS->GetConstantBuffer()[1]->UpdateBuffer(&id);
				ActiveVS->GetConstantBuffer()[1]->BindToVertexShader(1);

				if(Engine::GAPI->GetRendererState()->RendererSettings.FastShadows)
				{
					// Draw world mesh
					if(section.FullStaticMesh)
						Engine::GAPI->DrawMeshInfo(NULL, section.FullStaticMesh);
				}else
				{
					for(std::map<MeshKey, MeshInfo*>::iterator it = section.WorldMeshes.begin(); it != section.WorldMeshes.end();it++)
					{
						// Bind texture			
						if((*it).first.Material && (*it).first.Material->GetTexture())
						{
							if((*it).first.Material->GetAlphaFunc() != zMAT_ALPHA_FUNC_FUNC_NONE || 
								(*it).first.Material->GetAlphaFunc() != zMAT_ALPHA_FUNC_FUNC_MAT_DEFAULT)
							{
								if((*it).first.Material->GetTexture()->CacheIn(0.6f) == zRES_CACHED_IN)
								{
									(*it).first.Material->GetTexture()->Bind(0);
								}
							}else
							{
								DistortionTexture->BindToPixelShader(0);
							}
						}

						Engine::GAPI->DrawMeshInfo((*it).first.Material, (*it).second);
					}

					// Draw all vobs here
					for(std::list<VobInfo*>::iterator it = section.Vobs.begin(); it != section.Vobs.end(); it++)
					{
						D3DXVECTOR2 xz = D3DXVECTOR2((*it)->LastRenderPosition.x, (*it)->LastRenderPosition.z);

						// Check vob range
						float range = D3DXVec2Length(&(camXZ - xz));
						if(range > vobOutdoorDist || ((*it)->VisualInfo->MeshSize < vobSmallSize && range > vobOutdoorSmallDist))
							continue;

						// Check for inside vob
						if((*it)->IsIndoorVob)
							continue;

						// Bind per-instance buffer
						((D3D11ConstantBuffer *)(*it)->VobConstantBuffer)->BindToVertexShader(1);

						// Draw the vob
						for(std::map<zCMaterial *, std::vector<MeshInfo*>>::iterator itm = (*it)->VisualInfo->Meshes.begin(); itm != (*it)->VisualInfo->Meshes.end();itm++)
						{
							if((*itm).first && (*itm).first->GetTexture())
							{
								if((*itm).first->GetAlphaFunc() != zMAT_ALPHA_FUNC_FUNC_NONE || 
									(*itm).first->GetAlphaFunc() != zMAT_ALPHA_FUNC_FUNC_MAT_DEFAULT)
								{
									if((*itm).first->GetTexture()->CacheIn(0.6f) == zRES_CACHED_IN)
									{
										(*itm).first->GetTexture()->Bind(0);
									}
								}else
								{
									DistortionTexture->BindToPixelShader(0);
								}
							}

							for(unsigned int i=0;i<(*itm).second.size();i++)
							{
								Engine::GraphicsEngine->DrawVertexBufferIndexed((*itm).second[i]->MeshVertexBuffer, (*itm).second[i]->MeshIndexBuffer, (*itm).second[i]->Indices.size());
							}
						}
					}
				}
			}
		}
	}

	// Draw skeletal meshes
	for(std::list<SkeletalVobInfo *>::iterator it = Engine::GAPI->GetSkeletalMeshVobs().begin(); it != Engine::GAPI->GetSkeletalMeshVobs().end(); it++)
	{
		INT2 s = WorldConverter::GetSectionOfPos((*it)->Vob->GetPositionWorld());

		if(D3DXVec3Length(&((*it)->Vob->GetPositionWorld() - position)) > Engine::GAPI->GetRendererState()->RendererSettings.SkeletalMeshDrawRadius / 2)
			continue; // Skip out of range

		Engine::GAPI->DrawSkeletalMeshVob((*it));
	}
}

/** Draws the static vobs instanced */
XRESULT D3D11GraphicsEngine::DrawVOBsInstanced()
{
	D3DXVECTOR3 camPos = Engine::GAPI->GetCameraPosition();
	INT2 camSection = WorldConverter::GetSectionOfPos(camPos);

	int sectionViewDist = Engine::GAPI->GetRendererState()->RendererSettings.SectionDrawRadius;
	for(std::map<int, std::map<int, WorldMeshSectionInfo>>::iterator itx = Engine::GAPI->GetWorldSections().begin(); itx != Engine::GAPI->GetWorldSections().end(); itx++)
	{
		for(std::map<int, WorldMeshSectionInfo>::iterator ity = (*itx).second.begin(); ity != (*itx).second.end(); ity++)
		{
			if(abs((*itx).first - camSection.x) < sectionViewDist && 
				abs((*ity).first - camSection.y) < sectionViewDist)
			{
				WorldMeshSectionInfo& section = (*ity).second;

				// Draw all registered batches

				for(std::map<MeshVisualInfo *, BaseVertexBuffer*>::iterator it = section.InstanceCache.InstanceCache.begin(); it != section.InstanceCache.InstanceCache.end(); it++)
				{
					for(std::map<zCMaterial *, std::vector<MeshInfo*>>::iterator itm = (*it).first->Meshes.begin(); itm != (*it).first->Meshes.end();itm++)
					{
						if((*itm).first && (*itm).first->GetTexture())
						{
							if((*itm).first->GetTexture()->CacheIn(0.6f) == zRES_CACHED_IN)
								(*itm).first->GetTexture()->Bind(0);
						}

						for(unsigned int i=0;i<(*itm).second.size();i++)
						{
							// Draw instances
							DrawInstanced((*itm).second[i]->MeshVertexBuffer, 
								(*itm).second[i]->MeshIndexBuffer, 
								(*itm).second[i]->Indices.size(), 
								(*it).second, 
								sizeof(VS_ExConstantBuffer_PerInstance),  
								section.InstanceCache.InstanceCache.size());
						}
					}
				}
			}
		}
	}

	return XR_SUCCESS;
}

/** Draws the static VOBs */
XRESULT D3D11GraphicsEngine::DrawVOBs(bool noTextures)
{
	

	//Engine::GAPI->SetFarPlane(DEFAULT_FAR_PLANE);

	// bind shader
	SetActivePixelShader("PS_AtmosphereGround");
	D3D11PShader* nrmPS = ActivePS;
	SetActivePixelShader("PS_World");
	D3D11PShader* defaultPS = ActivePS;
	SetActiveVertexShader("VS_Ex");

	// Set constant buffer
	ActivePS->GetConstantBuffer()[0]->UpdateBuffer(&Engine::GAPI->GetRendererState()->GraphicsState);
	ActivePS->GetConstantBuffer()[0]->BindToPixelShader(0);

	GSky* sky = Engine::GAPI->GetSky();
	ActivePS->GetConstantBuffer()[1]->UpdateBuffer(&sky->GetAtmosphereCB());
	ActivePS->GetConstantBuffer()[1]->BindToPixelShader(1);

	// Use default material info for now
	MaterialInfo defInfo;
	ActivePS->GetConstantBuffer()[2]->UpdateBuffer(&defInfo);
	ActivePS->GetConstantBuffer()[2]->BindToPixelShader(2);

	D3DXVECTOR3 camPos = Engine::GAPI->GetCameraPosition();
	INT2 camSection = WorldConverter::GetSectionOfPos(camPos);

	D3DXMATRIX view;
	Engine::GAPI->GetViewMatrix(&view);
	Engine::GAPI->SetViewTransform(view);

	
	

	if(Engine::GAPI->GetRendererState()->RendererSettings.WireframeVobs)
	{
		Engine::GAPI->GetRendererState()->RasterizerState.Wireframe = true;
	}

	// Init drawcalls
	SetupVS_ExMeshDrawCall();
	SetupVS_ExConstantBuffer();

	//DrawVOBsInstanced();

	std::list<VobInfo *> vobs;
	std::list<VobLightInfo *> lights;
	Engine::GAPI->CollectVisibleVobs(vobs, lights);

	if(Engine::GAPI->GetRendererState()->RendererSettings.DrawVOBs)
	{
		float vobIndoorDist = Engine::GAPI->GetRendererState()->RendererSettings.IndoorVobDrawRadius;
		float vobOutdoorDist = Engine::GAPI->GetRendererState()->RendererSettings.OutdoorVobDrawRadius;
		float vobOutdoorSmallDist = Engine::GAPI->GetRendererState()->RendererSettings.OutdoorSmallVobDrawRadius;
		float vobSmallSize = Engine::GAPI->GetRendererState()->RendererSettings.SmallVobSize;
		int sectionViewDist = Engine::GAPI->GetRendererState()->RendererSettings.SectionDrawRadius;


		//noTextures = true;
		float3 indoorAmbient = DEFAULT_INDOOR_VOB_AMBIENT;

		// Sort them per material
		std::hash_map<zCMaterial*, std::list<VobInfo*>> meshesPerMaterial;

		for(std::list<VobInfo*>::iterator itv = vobs.begin(); itv != vobs.end();itv++)
		{
			// Reset this vob for CollectVisibleVobs
			(*itv)->VisibleInRenderPass = false;

			if(!(*itv)->Vob->GetVisual())
				continue;

			if((*itv)->VisualInfo->UnloadedSomething)
				continue; // Skip the whole mesh if we're missing something

			// Check for repositioning
			if(memcmp(&(*itv)->LastRenderPosition, (*itv)->Vob->GetPositionWorld(), sizeof(D3DXVECTOR3)) != 0)
			{
				(*itv)->LastRenderPosition = (*itv)->Vob->GetPositionWorld();
				(*itv)->UpdateVobConstantBuffer();

				Engine::GAPI->GetRendererState()->RendererInfo.FrameVobUpdates++;
			}

			for(std::map<zCMaterial *, std::vector<MeshInfo*>>::iterator itm = (*itv)->VisualInfo->Meshes.begin(); itm != (*itv)->VisualInfo->Meshes.end();itm++)
			{
				meshesPerMaterial[(*itm).first].push_back((*itv));
			}

			Engine::GAPI->GetRendererState()->RendererInfo.FrameDrawnVobs++;
		}

		for(std::hash_map<zCMaterial*, std::list<VobInfo*>>::iterator itm = meshesPerMaterial.begin(); itm != meshesPerMaterial.end();itm++)
		{
			// Bind texture
			if((*itm).first && (*itm).first->GetTexture())
			{
				if((*itm).first->GetTexture()->CacheIn(0.6f) == zRES_CACHED_IN)
					(*itm).first->GetTexture()->Bind(0);
			}

			// Querry the second texture slot to see if there is a normalmap bound
			ID3D11ShaderResourceView* nrmmap;
			Context->PSGetShaderResources(1,1, &nrmmap);
			if(!nrmmap)
			{
				if(ActivePS != defaultPS)
				{
					ActivePS = defaultPS;
					ActivePS->Apply();
				}
			}else
			{
				MaterialInfo* info = Engine::GAPI->GetMaterialInfoFrom((*itm).first->GetTexture());
				ActivePS->GetConstantBuffer()[2]->UpdateBuffer(info);
				ActivePS->GetConstantBuffer()[2]->BindToPixelShader(2);

				if(ActivePS != nrmPS)
				{
					ActivePS = nrmPS;
					ActivePS->Apply();
				}
				nrmmap->Release();
			}

			// Draw everything which uses this material
			for(std::list<VobInfo*>::iterator itv = (*itm).second.begin(); itv != (*itm).second.end();itv++)
			{
				// Bind per-instance buffer
				((D3D11ConstantBuffer *)(*itv)->VobConstantBuffer)->BindToVertexShader(1);

				// Update and bind buffer of PS
				/*PerObjectState ocb;
				if((*itv)->IsIndoorVob)
				{
					ocb.OS_AmbientColor = indoorAmbient;
				}else
				{
					ocb.OS_AmbientColor = float3(1,1,1);
				}
				ActivePS->GetConstantBuffer()[3]->UpdateBuffer(&ocb);
				ActivePS->GetConstantBuffer()[3]->BindToPixelShader(3);*/

				

				std::vector<MeshInfo *>& mlist = (*itv)->VisualInfo->Meshes[(*itm).first];
				for(unsigned int i=0;i<mlist.size();i++)
				{
					MeshInfo* mi = mlist[i];

					Engine::GraphicsEngine->DrawVertexBufferIndexed(mi->MeshVertexBuffer, mi->MeshIndexBuffer, mi->Indices.size());
				}
			}
		}

		/*for(std::list<VobInfo*>::iterator itv = vobs.begin(); itv != vobs.end();itv++)
		{
			

			

			// Push the vob
			for(std::map<zCMaterial *, std::vector<MeshInfo*>>::iterator itm = (*itv)->VisualInfo->Meshes.begin(); itm != (*itv)->VisualInfo->Meshes.end();itm++)
			{
				if(!noTextures)
				{
					
				}

				for(unsigned int i=0;i<(*itm).second.size();i++)
				{
					// Draw instances
					Engine::GraphicsEngine->DrawVertexBufferIndexed((*itm).second[i]->MeshVertexBuffer, (*itm).second[i]->MeshIndexBuffer, (*itm).second[i]->Indices.size());
				}
			}
		}*/
	}

	if(Engine::GAPI->GetRendererState()->RendererSettings.WireframeVobs)
	{
		Engine::GAPI->GetRendererState()->RasterizerState.Wireframe = false;
	}

	if(RenderingStage == DES_MAIN)
	{
		// Draw lighting, since everything is drawn by now and we have the lights here
		DrawLighting(lights);
	}

	return XR_SUCCESS;
}


/** Returns the current size of the backbuffer */
INT2 D3D11GraphicsEngine::GetResolution()
{
	return Resolution;
}


/** Sets up the default rendering state */
void D3D11GraphicsEngine::SetDefaultStates()
{
	Engine::GAPI->GetRendererState()->RasterizerState.SetDefault();
	Engine::GAPI->GetRendererState()->BlendState.SetDefault();
	Engine::GAPI->GetRendererState()->DepthState.SetDefault();
	Engine::GAPI->GetRendererState()->SamplerState.SetDefault();

	Engine::GAPI->GetRendererState()->RasterizerStateDirty = true;
	Engine::GAPI->GetRendererState()->BlendStateDirty = true;
	Engine::GAPI->GetRendererState()->DepthStateDirty = true;
	Engine::GAPI->GetRendererState()->SamplerStateDirty = true;

	UpdateRenderStates();
}

/** Draws the 3D-Cloud-Sky */
void D3D11GraphicsEngine::Draw3DClouds()
{
	GSky* sky = Engine::GAPI->GetSky();

	SetDefaultStates();

	// Copy the scene to the cloud-buffer without alpha
	SetActivePixelShader("PS_PFX_Copy_NoAlpha");
	ActivePS->Apply();

	Context->ClearRenderTargetView(CloudBuffer->GetRenderTargetView(), (float *)&float4(0,0,0,0));
	PfxRenderer->CopyTextureToRTV(BackbufferSRV, CloudBuffer->GetRenderTargetView(), INT2(0,0), true);

	Context->OMSetRenderTargets(1, CloudBuffer->GetRenderTargetViewPtr(), NULL);

	// Draw cloud meshes
	DrawCloudMeshes();

	Context->OMSetRenderTargets(1, &BackbufferRTV, DepthStencilBuffer->GetDepthStencilView());

	// Blur the cloud-buffer
	PfxRenderer->BlurTexture(CloudBuffer, true);

	// Blend it to the backbuffer
	Engine::GAPI->GetRendererState()->BlendState.BlendEnabled = true;
	Engine::GAPI->GetRendererState()->BlendStateDirty = true;
	UpdateRenderStates();

	SetActivePixelShader("PS_SkyPlane");
	SetActiveVertexShader("VS_Ex");
	ActiveVS->Apply();
	ActivePS->Apply();

	ViewportInfoConstantBuffer vpi;
	vpi.VPI_ViewportSize = float2((float)CloudBuffer->GetSizeX(), (float)CloudBuffer->GetSizeY());
	ActivePS->GetConstantBuffer()[0]->UpdateBuffer(&vpi);
	ActivePS->GetConstantBuffer()[0]->BindToPixelShader(0);

	Context->OMSetRenderTargets(1, &BackbufferRTV, DepthStencilBuffer->GetDepthStencilView());

	PfxRenderer->GetTempBufferDS4_2()->BindToPixelShader(Context, 2);
	NoiseTexture->BindToPixelShader(1);
	CloudBuffer->BindToPixelShader(Context, 0);

	D3DXMATRIX view;
	Engine::GAPI->GetViewMatrix(&view);
	Engine::GAPI->SetWorldTransform(view, true);

	Engine::GAPI->DrawMeshInfo(NULL, sky->GetSkyPlane());
	//PfxRenderer->CopyTextureToRTV(CloudBuffer->GetShaderResView(), BackbufferRTV, INT2(0,0), true);
}

/** Draws the sky using the GSky-Object */
XRESULT D3D11GraphicsEngine::DrawSky()
{
	GSky* sky = Engine::GAPI->GetSky();
	sky->RenderSky();

	//Context->OMSetRenderTargets(1, &BackbufferRTV, NULL);

	if(!Engine::GAPI->GetRendererState()->RendererSettings.AtmosphericScattering)
		return XR_SUCCESS;

	// Create a rotaion only view-matrix
	D3DXMATRIX invView;
	Engine::GAPI->GetInverseViewMatrix(&invView);

	D3DXMATRIX view;
	Engine::GAPI->GetViewMatrix(&view);



	// Remove translation
	/*invView(0,3) = 0;
	invView(1,3) = 0;
	invView(2,3) = 0;*/

	D3DXMATRIX scale;
	D3DXMatrixScaling(&scale, sky->GetAtmoshpereSettings().OuterRadius,
		sky->GetAtmoshpereSettings().OuterRadius,
		sky->GetAtmoshpereSettings().OuterRadius); // Upscale it a huge amount. Gothics world is big.

	D3DXMATRIX world;
	D3DXMatrixTranslation(&world, sky->GetAtmoshpereSettings().SpherePosition.x, 
		sky->GetAtmoshpereSettings().SpherePosition.y + sky->GetAtmoshpereSettings().SphereOffsetY, 
		sky->GetAtmoshpereSettings().SpherePosition.z);
	 
	world = scale * world;
	D3DXMatrixTranspose(&world, &world);

	// Apply world matrix
	Engine::GAPI->SetWorldTransform(world);
	Engine::GAPI->SetViewTransform(view);

	if(sky->GetAtmosphereCB().AC_CameraHeight > sky->GetAtmosphereCB().AC_OuterRadius)
	{
		SetActivePixelShader("PS_AtmosphereOuter");
	}else
	{
		SetActivePixelShader("PS_Atmosphere");
	}

	
	SetActiveVertexShader("VS_ExWS");

	ActivePS->GetConstantBuffer()[0]->UpdateBuffer(&sky->GetAtmosphereCB());
	ActivePS->GetConstantBuffer()[0]->BindToPixelShader(1);

	VS_ExConstantBuffer_PerInstance cbi;
	cbi.World = world;
	ActiveVS->GetConstantBuffer()[1]->UpdateBuffer(&cbi);
	ActiveVS->GetConstantBuffer()[1]->BindToVertexShader(1);


	// Update sky CB
	/*SkyConstantBuffer scb;
	scb.SC_TextureWeight = factor;
	ActivePS->GetConstantBuffer()[0]->UpdateBuffer(&scb),
	ActivePS->GetConstantBuffer()[0]->BindToPixelShader(0);
	*/
	Engine::GAPI->GetRendererState()->BlendState.SetDefault();
	Engine::GAPI->GetRendererState()->BlendState.BlendEnabled = true;

	Engine::GAPI->GetRendererState()->DepthState.SetDefault();
	Engine::GAPI->GetRendererState()->DepthState.DepthBufferEnabled = true;
	Engine::GAPI->GetRendererState()->RasterizerState.SetDefault();
	Engine::GAPI->GetRendererState()->DepthStateDirty = true;
	Engine::GAPI->GetRendererState()->BlendStateDirty = true;
	Engine::GAPI->GetRendererState()->RasterizerState.CullMode = GothicRasterizerStateInfo::CM_CULL_FRONT;
	Engine::GAPI->GetRendererState()->RasterizerStateDirty = true;

	SetupVS_ExMeshDrawCall();
	SetupVS_ExConstantBuffer();


	if(sky->GetSkyDome())
		sky->GetSkyDome()->DrawMesh();

	Engine::GAPI->GetRendererState()->RasterizerState.CullMode = GothicRasterizerStateInfo::CM_CULL_BACK;
	Engine::GAPI->GetRendererState()->RasterizerStateDirty = true;

	Engine::GAPI->GetRendererState()->DepthState.DepthBufferEnabled = true;
	Engine::GAPI->GetRendererState()->DepthStateDirty = true;

	Engine::GAPI->GetRendererState()->BlendState.BlendEnabled = false;
	Engine::GAPI->GetRendererState()->BlendStateDirty = true;

	//Context->OMSetRenderTargets(1, &BackbufferRTV, DepthStencilBuffer->GetDepthStencilView());

	//SetActivePixelShader("PS_World");

	return XR_SUCCESS;
}

/** Renders the cloudmeshes */
void D3D11GraphicsEngine::DrawCloudMeshes()
{
	GSky* sky = Engine::GAPI->GetSky();

	// Always use the same seed for this
	const int SEED = 1000;
	srand(SEED);
	const int NUM_CLOUDS = 100;

	Engine::GAPI->GetRendererState()->DepthState.DepthWriteEnabled = false;
	Engine::GAPI->GetRendererState()->DepthStateDirty = true;
	UpdateRenderStates();

	D3DXMATRIX view;
	Engine::GAPI->GetViewMatrix(&view);

	SetActivePixelShader("PS_Cloud");
	SetActiveVertexShader("VS_Ex");
	ActivePS->Apply();
	ActiveVS->Apply();


	ActivePS->GetConstantBuffer()[1]->UpdateBuffer(&sky->GetAtmosphereCB());
	ActivePS->GetConstantBuffer()[1]->BindToPixelShader(1);

	D3DXVECTOR3 tempSunPos = D3DXVECTOR3(0,20000,0);

	D3DXVECTOR3 minCloud = D3DXVECTOR3(-80000, 10000, -80000);
	D3DXVECTOR3 maxCloud = D3DXVECTOR3(80000, 10000, 80000);

	D3DXVECTOR3 minSize = D3DXVECTOR3(300, 300, 300);
	D3DXVECTOR3 maxSize = D3DXVECTOR3(1600, 600, 1600);

	for(int i=0;i<NUM_CLOUDS;i++)
	{
		D3DXVECTOR3 cp = D3DXVECTOR3(Toolbox::lerp(minCloud.x, maxCloud.x, Toolbox::frand()),
			Toolbox::lerp(minCloud.y, maxCloud.y, Toolbox::frand()),
			Toolbox::lerp(minCloud.z, maxCloud.z, Toolbox::frand()));

		D3DXVECTOR3 cs = D3DXVECTOR3(Toolbox::lerp(minSize.x, maxSize.x, Toolbox::frand()),
			Toolbox::lerp(minSize.y, maxSize.y, Toolbox::frand()),
			Toolbox::lerp(minSize.z, maxSize.z, Toolbox::frand()));

		CloudConstantBuffer ccb;
		D3DXVECTOR3 sp = cp - tempSunPos;
		D3DXVec3Normalize(&sp, &sp);

		D3DXVec3TransformNormal(ccb.C_LightDirection.toD3DXVECTOR3(), &sky->GetAtmoshpereSettings().LightDirection, &view);

		ccb.C_CloudPosition = cp;
		ActivePS->GetConstantBuffer()[0]->UpdateBuffer(&ccb);
		ActivePS->GetConstantBuffer()[0]->BindToPixelShader(0);

		D3DXMATRIX scale;
		D3DXMatrixScaling(&scale, cs.x, cs.y, cs.z); // Upscale

		D3DXMATRIX world;
		D3DXMatrixTranslation(&world, cp.x, cp.y, cp.z);
		world = scale * world;

		// Apply world matrix
		Engine::GAPI->SetWorldTransform(world);
		Engine::GAPI->SetViewTransform(view);

		// Draw
		sky->GetCloudMeshes()[rand() % 2]->DrawMesh();
	}

	srand(timeGetTime());
}

/** Returns the shadermanager */
D3D11ShaderManager* D3D11GraphicsEngine::GetShaderManager()
{
	return ShaderManager;
}

/** Called when a key got pressed */
XRESULT D3D11GraphicsEngine::OnKeyDown(unsigned int key)
{
	switch (key)
	{
	case VK_NUMPAD0:
		ShaderManager->ReloadShaders();
		break;

	default:
		break;
	}

	return XR_SUCCESS;
}

/** Returns the line renderer object */
BaseLineRenderer* D3D11GraphicsEngine::GetLineRenderer()
{
	return LineRenderer;
}

/** Applys the lighting to the scene */
XRESULT D3D11GraphicsEngine::DrawLighting(std::list<VobLightInfo*>& lights)
{
	// ********************************
	// Draw world shadows
	// ********************************
	CameraReplacement cr;

	D3DXVECTOR3 dir = *Engine::GAPI->GetSky()->GetAtmosphereCB().AC_LightPos.toD3DXVECTOR3();

	D3DXVECTOR3 cameraPosition = Engine::GAPI->GetCameraPosition();
	D3DXVECTOR3 p = cameraPosition;

	// Get the section we are currently in
	INT2 cameraSection = WorldConverter::GetSectionOfPos(cameraPosition);
	WorldMeshSectionInfo& section = Engine::GAPI->GetWorldSections()[cameraSection.x][cameraSection.y];

	// Set the camera height to the highest point in this section
	//p.y = 0;
	p += dir * 6000.0f;

	D3DXVECTOR3 lookAt = p;
	lookAt -= dir;

	// Create shadowmap view-matrix
	D3DXMatrixLookAtLH(&cr.ViewReplacement, &p, &lookAt, &D3DXVECTOR3(0,1,0));
	D3DXMatrixTranspose(&cr.ViewReplacement, &cr.ViewReplacement);

	D3DXMatrixOrthoLH(&cr.ProjectionReplacement, WORLD_SHADOWMAP_SIZE * Engine::GAPI->GetRendererState()->RendererSettings.WorldShadowRangeScale, 
												 WORLD_SHADOWMAP_SIZE * Engine::GAPI->GetRendererState()->RendererSettings.WorldShadowRangeScale, 1, 40000.0f);
	D3DXMatrixTranspose(&cr.ProjectionReplacement, &cr.ProjectionReplacement);

	cr.PositionReplacement = p;
	cr.LookAtReplacement = lookAt;

	// Replace gothics camera
	Engine::GAPI->SetCameraReplacementPtr(&cr);

	RenderShadowmaps(cameraPosition);

	// Restore gothics camera
	Engine::GAPI->SetCameraReplacementPtr(NULL);

	D3DXMATRIX view;
	Engine::GAPI->GetViewMatrix(&view);
	Engine::GAPI->SetViewTransform(view);

	// ********************************
	// Draw direct lighting
	// ********************************
	SetActiveVertexShader("VS_ExPointLight");
	SetActivePixelShader("PS_DS_PointLight");

	Engine::GAPI->SetFarPlane(DEFAULT_FAR_PLANE);

	Engine::GAPI->GetRendererState()->BlendState.SetAdditiveBlending();
	Engine::GAPI->GetRendererState()->BlendStateDirty = true;

	Engine::GAPI->GetRendererState()->DepthState.DepthWriteEnabled = false;
	Engine::GAPI->GetRendererState()->DepthStateDirty = true;

	Engine::GAPI->GetRendererState()->RasterizerState.CullMode = GothicRasterizerStateInfo::CM_CULL_BACK;
	Engine::GAPI->GetRendererState()->RasterizerStateDirty = true;

	SetupVS_ExMeshDrawCall();
	SetupVS_ExConstantBuffer();

	// Set the main rendertarget
	Context->OMSetRenderTargets(1, HDRBackBuffer->GetRenderTargetViewPtr(), NULL);
	//Context->ClearRenderTargetView(HDRBackBuffer->GetRenderTargetView(), (float *)D3DXVECTOR4(0,0,0,0));

	
	D3DXMatrixTranspose(&view, &view);

	DS_PointLightConstantBuffer plcb;
	D3DXMatrixInverse(&plcb.PL_InvProj, NULL, &Engine::GAPI->GetProjectionMatrix());
	D3DXMatrixInverse(&plcb.PL_InvView, NULL, &Engine::GAPI->GetRendererState()->TransformState.TransformView);
	plcb.PL_ViewportSize = float2((float)Resolution.x, (float)Resolution.y);

	GBuffer0_Diffuse->BindToPixelShader(Context, 0);
	GBuffer1_Normals_SpecIntens_SpecPower->BindToPixelShader(Context, 1);
	DepthStencilBuffer->BindToPixelShader(Context, 2);

	bool lastOutside = true;

	// Draw all lights
	for(std::list<VobLightInfo*>::iterator itv = lights.begin(); itv != lights.end();itv++)
	{
		zCVobLight* vob = (*itv)->Vob;

		// Reset state from CollectVisibleVobs
		(*itv)->VisibleInRenderPass = false;

		if(!vob->IsEnabled())
			continue;

		plcb.PL_Color = float4(vob->GetLightColor());
		plcb.PL_Range = vob->GetLightRange();
		plcb.Pl_PositionWorld = vob->GetPositionWorld();
		
		//zTBBox3D bb = (*itv)->GetBBoxLocal();
		//GetLineRenderer()->AddAABBMinMax(bb.Min + (*itv)->GetPositionWorld(), bb.Max + (*itv)->GetPositionWorld(), *(D3DXVECTOR4 *)&float4((*itv)->GetLightColor()));
		

		// Need that in view space
		D3DXVec3TransformCoord(plcb.Pl_PositionView.toD3DXVECTOR3(), plcb.Pl_PositionWorld.toD3DXVECTOR3(), &view);
		D3DXVec3TransformCoord(plcb.PL_LightScreenPos.toD3DXVECTOR3(), plcb.Pl_PositionWorld.toD3DXVECTOR3(), &Engine::GAPI->GetProjectionMatrix());

		plcb.PL_LightScreenPos.x = plcb.PL_LightScreenPos.x/2.0f +0.5f;
		plcb.PL_LightScreenPos.y = plcb.PL_LightScreenPos.y/-2.0f +0.5f;

		// Apply the constantbuffer to vs and PS
		ActivePS->GetConstantBuffer()[0]->UpdateBuffer(&plcb);
		ActivePS->GetConstantBuffer()[0]->BindToPixelShader(0);
		ActivePS->GetConstantBuffer()[0]->BindToVertexShader(1); // Bind this instead of the usual per-instance buffer

		// Draw the mesh
		InverseUnitSphereMesh->DrawMesh();
	}


	Engine::GAPI->GetRendererState()->RasterizerState.CullMode = GothicRasterizerStateInfo::CM_CULL_BACK;
	Engine::GAPI->GetRendererState()->RasterizerStateDirty = true;

	// Draw atmospheric scattering
	SetActivePixelShader("PS_DS_AtmosphericScattering");
	SetActiveVertexShader("VS_PFX");

	SetupVS_ExMeshDrawCall();

	GSky* sky = Engine::GAPI->GetSky();
	ActivePS->GetConstantBuffer()[1]->UpdateBuffer(&sky->GetAtmosphereCB());
	ActivePS->GetConstantBuffer()[1]->BindToPixelShader(1);

	DS_ScreenQuadConstantBuffer scb;
	scb.SQ_InvProj = plcb.PL_InvProj;
	scb.SQ_InvView = plcb.PL_InvView;
	D3DXVec3TransformNormal(scb.SQ_LightDirectionVS.toD3DXVECTOR3(), sky->GetAtmosphereCB().AC_LightPos.toD3DXVECTOR3(), &view);

	float3 sunColor = Engine::GAPI->GetRendererState()->RendererSettings.SunLightColor;
	scb.SQ_LightColor = float4(sunColor.x, sunColor.y, sunColor.z,1);

	scb.SQ_ShadowView = cr.ViewReplacement;
	scb.SQ_ShadowProj = cr.ProjectionReplacement;
	scb.SQ_ShadowmapSize = WORLD_SHADOWMAP_SIZE;
	ActivePS->GetConstantBuffer()[0]->UpdateBuffer(&scb);
	ActivePS->GetConstantBuffer()[0]->BindToPixelShader(0);

	WorldShadowmap1->BindToPixelShader(Context, 3);

	PfxRenderer->DrawFullScreenQuad();

	// Reset state
	Context->OMSetRenderTargets(1, HDRBackBuffer->GetRenderTargetViewPtr(), DepthStencilBuffer->GetDepthStencilView());

	Engine::GAPI->GetRendererState()->BlendState.SetDefault();
	Engine::GAPI->GetRendererState()->BlendStateDirty = true;

	Engine::GAPI->GetRendererState()->BlendState.SetDefault();
	Engine::GAPI->GetRendererState()->BlendStateDirty = true;

	Engine::GAPI->GetRendererState()->DepthState.DepthWriteEnabled = true;
	Engine::GAPI->GetRendererState()->DepthStateDirty = true;

	Engine::GAPI->GetRendererState()->RasterizerState.CullMode = GothicRasterizerStateInfo::CM_CULL_FRONT;
	Engine::GAPI->GetRendererState()->RasterizerStateDirty = true;

	UpdateRenderStates();
	return XR_SUCCESS;
}

/** Renders the shadowmaps for the sun */
void D3D11GraphicsEngine::RenderShadowmaps(const D3DXVECTOR3& cameraPosition)
{
	D3D11_VIEWPORT oldVP;
	UINT n = 1;
	Context->RSGetViewports(&n, &oldVP);

	// Apply new viewport
	D3D11_VIEWPORT vp;
	vp.TopLeftY = 0;
	vp.TopLeftY = 0;
	vp.MinDepth = 0.0f;
	vp.MaxDepth = 1.0f;
	vp.Width = WORLD_SHADOWMAP_SIZE;
	vp.Height = WORLD_SHADOWMAP_SIZE;
	Context->RSSetViewports(1, &vp);

	// Set the rendering stage
	D3D11ENGINE_RENDER_STAGE oldStage = RenderingStage;
	SetRenderingStage(DES_SHADOWMAP);

	// Clear and Bind the shadowmap
	Context->ClearDepthStencilView(WorldShadowmap1->GetDepthStencilView(), D3D11_CLEAR_DEPTH, 1.0f, 0);
	//Context->ClearRenderTargetView(GBuffer1_Normals_SpecIntens_SpecPower->GetRenderTargetView(), (float *)&D3DXVECTOR4(1,0,0,0));
	Context->OMSetRenderTargets(0, NULL, WorldShadowmap1->GetDepthStencilView());
	//Context->OMSetRenderTargets(1, GBuffer1_Normals_SpecIntens_SpecPower->GetRenderTargetViewPtr(), WorldShadowmap1->GetDepthStencilView());

	Engine::GAPI->SetFarPlane(20000.0f);

	// Dont render shadows from the sun when it isn't on the sky
	if (Engine::GAPI->GetSky()->GetAtmoshpereSettings().LightDirection.y > 0 && 
		Engine::GAPI->GetRendererState()->RendererSettings.DrawShadowGeometry)
	{
		// Draw the world mesh without textures
		DrawWorldAround(cameraPosition, 2, 10000.0f);
	}

	// Restore state
	SetRenderingStage(oldStage);
	Context->RSSetViewports(1, &oldVP);
}

/** Draws a fullscreenquad, copying the given texture to the viewport */
void D3D11GraphicsEngine::DrawQuad(INT2 position, INT2 size)
{
	ID3D11ShaderResourceView* srv;
	Context->PSGetShaderResources(0, 1, &srv);

	if(srv)
	{
		PfxRenderer->CopyTextureToRTV(srv, BackbufferRTV, size, false, position);
		srv->Release();
	}
}


/** Sets the current rendering stage */
void D3D11GraphicsEngine::SetRenderingStage(D3D11ENGINE_RENDER_STAGE stage)
{
	RenderingStage = stage;
}

/** Draws a single VOB */
void D3D11GraphicsEngine::DrawVobSingle(VobInfo* vob)
{
	vob->UpdateVobConstantBuffer();
	vob->VobConstantBuffer->BindToVertexShader(1);

	SetActivePixelShader("PS_AtmosphereGround");
	SetActiveVertexShader("VS_Ex");

	SetupVS_ExMeshDrawCall();
	SetupVS_ExConstantBuffer();
	//SetupVS_ExPerInstanceConstantBuffer();
	vob->VobConstantBuffer->BindToVertexShader(1);

	for(std::map<zCMaterial *, std::vector<MeshInfo*>>::iterator itm = vob->VisualInfo->Meshes.begin(); itm != vob->VisualInfo->Meshes.end();itm++)
	{
		for(unsigned int i=0;i<(*itm).second.size();i++)
		{
			// Cache texture
			if((*itm).first)
			{
				if((*itm).first->GetTexture())
				{
					if((*itm).first->GetTexture()->CacheIn(0.6f) == zRES_CACHED_IN)
						(*itm).first->GetTexture()->Bind(0);
				}
			}

			// Draw instances
			Engine::GraphicsEngine->DrawVertexBufferIndexed((*itm).second[i]->MeshVertexBuffer, (*itm).second[i]->MeshIndexBuffer, (*itm).second[i]->Indices.size());
		}
	}
}

/** Message-Callback for the main window */
LRESULT D3D11GraphicsEngine::OnWindowMessage(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	UIView->OnWindowMessage(hWnd, msg, wParam, lParam);

	return 0;
}