#include "pch.h"
#include "D3D11PfxRenderer.h"
#include "RenderToTextureBuffer.h"
#include "Engine.h"
#include "D3D11GraphicsEngine.h"
#include "D3D11FullscreenQuad.h"
#include "D3D11ShaderManager.h"
#include "D3D11PShader.h"
#include "D3D11VShader.h"
#include "D3D11PFX_Blur.h"
#include "D3D11PFX_HeightFog.h"
#include "D3D11PFX_DistanceBlur.h"
#include "D3D11PFX_HDR.h"

D3D11PfxRenderer::D3D11PfxRenderer(void)
{
	TempBuffer = NULL;
	TempBufferDS4_1 = NULL;
	TempBufferDS4_2 = NULL;

	FX_Blur = NULL;

	D3D11GraphicsEngine* engine = (D3D11GraphicsEngine *)Engine::GraphicsEngine;
	ScreenQuad = new D3D11FullscreenQuad;
	ScreenQuad->CreateQuad(engine->GetDevice());

	FX_Blur = new D3D11PFX_Blur(this);
	FX_HeightFog = new D3D11PFX_HeightFog(this);
	//FX_DistanceBlur = new D3D11PFX_DistanceBlur(this);
	FX_HDR = new D3D11PFX_HDR(this);
}


D3D11PfxRenderer::~D3D11PfxRenderer(void)
{
	delete TempBuffer;
	delete TempBufferDS4_1;
	delete TempBufferDS4_2;
	delete ScreenQuad;
	delete FX_Blur;
	delete FX_HeightFog;
	//delete FX_DistanceBlur;
	delete FX_HDR;
} 

/** Renders the distance blur effect */
XRESULT D3D11PfxRenderer::RenderDistanceBlur()
{
	FX_DistanceBlur->Render(NULL);
	return XR_SUCCESS;
}

/** Blurs the given texture */
XRESULT D3D11PfxRenderer::BlurTexture(RenderToTextureBuffer* texture, bool leaveResultInD4_2)
{
	FX_Blur->RenderBlur(texture, leaveResultInD4_2);
	return XR_SUCCESS;
}

/** Renders the heightfog */
XRESULT D3D11PfxRenderer::RenderHeightfog()
{
	return FX_HeightFog->Render(NULL);
} 

/** Renders the HDR-Effect */
XRESULT D3D11PfxRenderer::RenderHDR()
{
	return FX_HDR->Render(NULL);
}

/** Draws a fullscreenquad */
XRESULT D3D11PfxRenderer::DrawFullScreenQuad()
{
	D3D11GraphicsEngine* engine = (D3D11GraphicsEngine *)Engine::GraphicsEngine;
	engine->UpdateRenderStates();

	engine->GetContext()->IASetPrimitiveTopology(D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST);

	UINT offset = 0;
	UINT uStride = sizeof(SimpleVertexStruct);
	ID3D11Buffer* buffers = ScreenQuad->GetBuffer();
	engine->GetContext()->IASetVertexBuffers( 0, 1, &buffers, &uStride, &offset );

	ID3D11Buffer* cb = NULL;
	engine->GetContext()->VSSetConstantBuffers(0, 1, &cb);

	//Draw the mesh
	engine->GetContext()->Draw(6, 0 );
	
	return XR_SUCCESS;
}

/** Unbinds texturesamplers from the pixel-shader */
XRESULT D3D11PfxRenderer::UnbindPSResources(int num)
{
	ID3D11ShaderResourceView** srv = new ID3D11ShaderResourceView*[num];
	ZeroMemory(srv, sizeof(ID3D11ShaderResourceView*) * num);

	D3D11GraphicsEngine* engine = (D3D11GraphicsEngine *)Engine::GraphicsEngine;
	engine->GetContext()->PSSetShaderResources(0, num, srv);

	delete[] srv;
	
	return XR_SUCCESS;
}

/** Copies the given texture to the given RTV */
XRESULT D3D11PfxRenderer::CopyTextureToRTV(ID3D11ShaderResourceView* texture, ID3D11RenderTargetView* rtv, INT2 targetResolution, bool useCustomPS, INT2 offset)
{
	D3D11GraphicsEngine* engine = (D3D11GraphicsEngine *)Engine::GraphicsEngine;
	
	D3D11_VIEWPORT oldVP;
	if(targetResolution.x != 0 && targetResolution.y != 0)
	{
		UINT n=1;
		engine->GetContext()->RSGetViewports(&n, &oldVP);

		D3D11_VIEWPORT vp;
		vp.TopLeftX = (float)offset.x;
		vp.TopLeftY = (float)offset.y;
		vp.MinDepth = 0.0f;
		vp.MaxDepth = 0.0f;
		vp.Width = (float)targetResolution.x;
		vp.Height = (float)targetResolution.y;

		engine->GetContext()->RSSetViewports(1, &vp);
	}

	// Save old rendertargets
	ID3D11RenderTargetView* oldRTV = NULL;
	ID3D11DepthStencilView* oldDSV = NULL;
	engine->GetContext()->OMGetRenderTargets(1, &oldRTV, &oldDSV);

	// Bind shaders
	if(!useCustomPS)
	{
		D3D11PShader* simplePS = engine->GetShaderManager()->GetPShader("PS_PFX_Simple");
		simplePS->Apply();
	}

	engine->GetShaderManager()->GetVShader("VS_PFX")->Apply();
	
	ID3D11ShaderResourceView* srv = NULL;
	engine->GetContext()->PSSetShaderResources(0,1,&srv);

	engine->GetContext()->OMSetRenderTargets(1, &rtv, NULL);

	if(texture)
		engine->GetContext()->PSSetShaderResources(0,1, &texture);

	DrawFullScreenQuad();

	engine->GetContext()->OMSetRenderTargets(1, &oldRTV, oldDSV);
	if(oldRTV)oldRTV->Release();
	if(oldDSV)oldDSV->Release();
	
	if(targetResolution.x != 0 && targetResolution.y != 0)
	{
		engine->GetContext()->RSSetViewports(1, &oldVP);
	}

	return XR_SUCCESS;
}

/** Called on resize */
XRESULT D3D11PfxRenderer::OnResize(const INT2& newResolution)
{
	D3D11GraphicsEngine* engine = (D3D11GraphicsEngine *)Engine::GraphicsEngine;

	// Create temp-buffer
	delete TempBuffer;
	TempBuffer = new RenderToTextureBuffer(engine->GetDevice(), newResolution.x, newResolution.y, DXGI_FORMAT_R16G16B16A16_FLOAT, NULL);

	delete TempBufferDS4_1;
	TempBufferDS4_1 = new RenderToTextureBuffer(engine->GetDevice(), newResolution.x / 4, newResolution.y / 4, DXGI_FORMAT_R16G16B16A16_FLOAT, NULL);

	delete TempBufferDS4_2;
	TempBufferDS4_2 = new RenderToTextureBuffer(engine->GetDevice(), newResolution.x / 4, newResolution.y / 4, DXGI_FORMAT_R16G16B16A16_FLOAT, NULL);
	
	return XR_SUCCESS;
}