#include "pch.h"
#include "D3D11ShaderManager.h"
#include "D3D11Vshader.h"
#include "D3D11PShader.h"
#include "D3D11ConstantBuffer.h"
#include "GothicGraphicsState.h"
#include "ConstantBufferStructs.h"
#include "GothicAPI.h"

const int NUM_MAX_BONES = 64;

D3D11ShaderManager::D3D11ShaderManager()
{
	ReloadShadersNextFrame = false;
}

D3D11ShaderManager::~D3D11ShaderManager()
{
	DeleteShaders();
}

/** Creates list with ShaderInfos */
XRESULT D3D11ShaderManager::Init()
{
	Shaders = std::vector<ShaderInfo>();
	VShaders = std::unordered_map<std::string, D3D11VShader*>();
	PShaders = std::unordered_map<std::string, D3D11PShader*>();

	Shaders.push_back(ShaderInfo("VS_Ex", "VS_Ex.hlsl", "v", 1));
	Shaders.back().cBufferSizes.push_back(sizeof(VS_ExConstantBuffer_PerFrame));
	Shaders.back().cBufferSizes.push_back(sizeof(VS_ExConstantBuffer_PerInstance));

	Shaders.push_back(ShaderInfo("VS_ExWS", "VS_ExWS.hlsl", "v", 1));
	Shaders.back().cBufferSizes.push_back(sizeof(VS_ExConstantBuffer_PerFrame));
	Shaders.back().cBufferSizes.push_back(sizeof(VS_ExConstantBuffer_PerInstance));

	Shaders.push_back(ShaderInfo("VS_ExDisplace", "VS_ExDisplace.hlsl", "v", 1));
	Shaders.back().cBufferSizes.push_back(sizeof(VS_ExConstantBuffer_PerFrame));
	Shaders.back().cBufferSizes.push_back(sizeof(VS_ExConstantBuffer_PerInstance));

	Shaders.push_back(ShaderInfo("VS_Obj", "VS_Obj.hlsl", "v", 8));
	Shaders.back().cBufferSizes.push_back(sizeof(VS_ExConstantBuffer_PerFrame));
	Shaders.back().cBufferSizes.push_back(sizeof(VS_ExConstantBuffer_PerInstance));
	Shaders.push_back(ShaderInfo("VS_ExSkeletal", "VS_ExSkeletal.hlsl", "v", 3));
	Shaders.back().cBufferSizes.push_back(sizeof(VS_ExConstantBuffer_PerFrame));
	Shaders.back().cBufferSizes.push_back(sizeof(VS_ExConstantBuffer_PerInstance));
	Shaders.back().cBufferSizes.push_back(NUM_MAX_BONES * sizeof(D3DXMATRIX));
	Shaders.push_back(ShaderInfo("VS_TransformedEx", "VS_TransformedEx.hlsl", "v", 1));
	Shaders.back().cBufferSizes.push_back(2 * sizeof(float2));

	Shaders.push_back(ShaderInfo("VS_ExPointLight", "VS_ExPointLight.hlsl", "v", 1));
	Shaders.back().cBufferSizes.push_back(sizeof(VS_ExConstantBuffer_PerFrame));
	Shaders.back().cBufferSizes.push_back(sizeof(DS_PointLightConstantBuffer));

	Shaders.push_back(ShaderInfo("VS_XYZRHW_DIF_T1", "VS_XYZRHW_DIF_T1.hlsl", "v", 7));
	Shaders.back().cBufferSizes.push_back(2 * sizeof(float2));

	Shaders.push_back(ShaderInfo("VS_ExInstanced", "VS_ExInstanced.hlsl", "v", 4));
	Shaders.back().cBufferSizes.push_back(sizeof(VS_ExConstantBuffer_PerFrame));
	Shaders.back().cBufferSizes.push_back(sizeof(GrassConstantBuffer));

	Shaders.push_back(ShaderInfo("VS_GrassInstanced", "VS_GrassInstanced.hlsl", "v", 9));
	Shaders.back().cBufferSizes.push_back(sizeof(VS_ExConstantBuffer_PerFrame));

	Shaders.push_back(ShaderInfo("VS_Lines", "VS_Lines.hlsl", "v", 6));
	Shaders.back().cBufferSizes.push_back(sizeof(VS_ExConstantBuffer_PerFrame));
	Shaders.back().cBufferSizes.push_back(sizeof(VS_ExConstantBuffer_PerInstance));

	Shaders.push_back(ShaderInfo("PS_Lines", "PS_Lines.hlsl", "p"));

	//Shaders.push_back(ShaderInfo("FixedFunctionPipelineEmulationPS", "FixedFunctionPipelineEmulationPS.hlsl", "p", 1));
	Shaders.push_back(ShaderInfo("PS_Simple", "PS_Simple.hlsl", "p"));
	Shaders.push_back(ShaderInfo("PS_World", "PS_World.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(GothicGraphicsState));
	Shaders.back().cBufferSizes.push_back(sizeof(AtmosphereConstantBuffer));
	Shaders.back().cBufferSizes.push_back(sizeof(MaterialInfo));
	Shaders.back().cBufferSizes.push_back(sizeof(PerObjectState));

	Shaders.push_back(ShaderInfo("PS_Grass", "PS_Grass.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(MaterialInfo));

	Shaders.push_back(ShaderInfo("PS_Sky", "PS_Sky.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(SkyConstantBuffer));

	Shaders.push_back(ShaderInfo("VS_PFX", "VS_PFX.hlsl", "v", 5));
	Shaders.back().cBufferSizes.push_back(sizeof(PFXVS_ConstantBuffer));

	Shaders.push_back(ShaderInfo("PS_PFX_Simple", "PS_PFX_Simple.hlsl", "p"));
	Shaders.push_back(ShaderInfo("PS_PFX_GammaCorrectInv", "PS_PFX_GammaCorrectInv.hlsl", "p"));

	Shaders.push_back(ShaderInfo("PS_PFX_GaussBlur", "PS_PFX_GaussBlur.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(BlurConstantBuffer));

	Shaders.push_back(ShaderInfo("PS_PFX_Heightfog", "PS_PFX_Heightfog.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(HeightfogConstantBuffer));

	Shaders.push_back(ShaderInfo("PS_Cloud", "PS_Cloud.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(CloudConstantBuffer));
	Shaders.back().cBufferSizes.push_back(sizeof(AtmosphereConstantBuffer));

	Shaders.push_back(ShaderInfo("PS_PFX_Copy_NoAlpha", "PS_PFX_Copy_NoAlpha.hlsl", "p"));
	Shaders.push_back(ShaderInfo("PS_PFX_Blend", "PS_PFX_Blend.hlsl", "p"));
	Shaders.push_back(ShaderInfo("PS_PFX_DistanceBlur", "PS_PFX_DistanceBlur.hlsl", "p"));
	Shaders.push_back(ShaderInfo("PS_PFX_LumConvert", "PS_PFX_LumConvert.hlsl", "p"));
	Shaders.push_back(ShaderInfo("PS_PFX_LumAdapt", "PS_PFX_LumAdapt.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(LumAdaptConstantBuffer));
	Shaders.push_back(ShaderInfo("PS_PFX_HDR", "PS_PFX_HDR.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(HDRSettingsConstantBuffer));

	Shaders.push_back(ShaderInfo("PS_PFX_Tonemap", "PS_PFX_Tonemap.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(HDRSettingsConstantBuffer));

	Shaders.push_back(ShaderInfo("PS_SkyPlane", "PS_SkyPlane.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(ViewportInfoConstantBuffer));

	Shaders.push_back(ShaderInfo("PS_AtmosphereGround", "PS_AtmosphereGround.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(GothicGraphicsState));
	Shaders.back().cBufferSizes.push_back(sizeof(AtmosphereConstantBuffer));
	Shaders.back().cBufferSizes.push_back(sizeof(MaterialInfo));
	Shaders.back().cBufferSizes.push_back(sizeof(PerObjectState));

	Shaders.push_back(ShaderInfo("PS_Atmosphere", "PS_Atmosphere.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(AtmosphereConstantBuffer));

	Shaders.push_back(ShaderInfo("PS_AtmosphereOuter", "PS_AtmosphereOuter.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(AtmosphereConstantBuffer));

	Shaders.push_back(ShaderInfo("PS_WorldLightmapped", "PS_WorldLightmapped.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(GothicGraphicsState));

	Shaders.push_back(ShaderInfo("PS_FixedFunctionPipe", "PS_FixedFunctionPipe.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(GothicGraphicsState));

	Shaders.push_back(ShaderInfo("PS_DS_PointLight", "PS_DS_PointLight.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(DS_PointLightConstantBuffer));

	Shaders.push_back(ShaderInfo("PS_DS_AtmosphericScattering", "PS_DS_AtmosphericScattering.hlsl", "p"));
	Shaders.back().cBufferSizes.push_back(sizeof(DS_ScreenQuadConstantBuffer));
	Shaders.back().cBufferSizes.push_back(sizeof(AtmosphereConstantBuffer));
	
	
	
	return XR_SUCCESS;
}

/** Loads/Compiles Shaderes from list */
XRESULT D3D11ShaderManager::LoadShaders()
{
	for (unsigned int i = 0; i < Shaders.size(); i++)
	{
		//Check if shader src-file exists
		std::string fileName = "system\\GD3D11\\shaders\\" + Shaders[i].fileName;
		if (FILE* f = fopen(fileName.c_str(), "r"))
		{
			//Check shader's type
			if (Shaders[i].type == "v")
			{
				// See if this is a reload
				if(VShaders.count(Shaders[i].name) > 0)
				{
					LogInfo() << "Reloading shader: " << Shaders[i].name;

					D3D11VShader* vs = new D3D11VShader();
					if(XR_SUCCESS != vs->LoadShader(("system\\GD3D11\\shaders\\" + Shaders[i].fileName).c_str(), Shaders[i].layout))
					{
						LogError() << "Failed to reload shader: " << Shaders[i].fileName;

						delete vs;
					}else
					{
						// Compilation succeeded, switch the shader
						delete VShaders[Shaders[i].name];
						VShaders[Shaders[i].name] = vs;

						for (unsigned int j = 0; j < Shaders[i].cBufferSizes.size(); j++)
						{
							VShaders[Shaders[i].name]->GetConstantBuffer().push_back(new D3D11ConstantBuffer(Shaders[i].cBufferSizes[j], NULL));
						}
					}
				}else
				{
					LogInfo() << "Reloading shader: " << Shaders[i].name;

					VShaders[Shaders[i].name] = new D3D11VShader();
					XLE(VShaders[Shaders[i].name]->LoadShader(("system\\GD3D11\\shaders\\" + Shaders[i].fileName).c_str(), Shaders[i].layout));
					for (unsigned int j = 0; j < Shaders[i].cBufferSizes.size(); j++)
					{
						VShaders[Shaders[i].name]->GetConstantBuffer().push_back(new D3D11ConstantBuffer(Shaders[i].cBufferSizes[j], NULL));
					}
				}
			}
			else if (Shaders[i].type == "p")
			{
				// See if this is a reload
				if(PShaders.count(Shaders[i].name) > 0)
				{
					D3D11PShader* ps = new D3D11PShader();
					if(XR_SUCCESS != ps->LoadShader(("system\\GD3D11\\shaders\\" + Shaders[i].fileName).c_str()))
					{
						LogError() << "Failed to reload shader: " << Shaders[i].fileName;

						delete ps;
					}else
					{
						// Compilation succeeded, switch the shader
						delete PShaders[Shaders[i].name];
						PShaders[Shaders[i].name] = ps;

						for (unsigned int j = 0; j < Shaders[i].cBufferSizes.size(); j++)
						{
							PShaders[Shaders[i].name]->GetConstantBuffer().push_back(new D3D11ConstantBuffer(Shaders[i].cBufferSizes[j], NULL));
						}
					}
				}else
				{
					PShaders[Shaders[i].name] = new D3D11PShader();
					XLE(PShaders[Shaders[i].name]->LoadShader(("system\\GD3D11\\shaders\\" + Shaders[i].fileName).c_str()));
					for (unsigned int j = 0; j < Shaders[i].cBufferSizes.size(); j++)
					{
						PShaders[Shaders[i].name]->GetConstantBuffer().push_back(new D3D11ConstantBuffer(Shaders[i].cBufferSizes[j], NULL));
					}
				}
			}
			fclose(f);
		}

	}

	return XR_SUCCESS;
}

/** Deletes all shaders and loads them again */
XRESULT D3D11ShaderManager::ReloadShaders()
{
	ReloadShadersNextFrame = true;

	return XR_SUCCESS;
}

/** Called on frame start */
XRESULT D3D11ShaderManager::OnFrameStart()
{
	if(ReloadShadersNextFrame)
	{
		LoadShaders();
		ReloadShadersNextFrame = false;
	}

	return XR_SUCCESS;
}

/** Deletes all shaders */
XRESULT D3D11ShaderManager::DeleteShaders()
{
	std::unordered_map<std::string, D3D11VShader*>::iterator vIter;
	for (vIter = VShaders.begin(); vIter != VShaders.end(); vIter++)
	{
		delete vIter->second;
	}
	VShaders.clear();

	std::unordered_map<std::string, D3D11PShader*>::iterator pIter;
	for (pIter = PShaders.begin(); pIter != PShaders.end(); pIter++)
	{
		delete pIter->second;
	}
	PShaders.clear();

	return XR_SUCCESS;
}

/** Return a specific shader */
D3D11VShader* D3D11ShaderManager::GetVShader(std::string shader)
{
	return VShaders[shader];
}
D3D11PShader* D3D11ShaderManager::GetPShader(std::string shader)
{
	return PShaders[shader];
}