#pragma once
#include <unordered_map>

/** Struct holds initial shader data for load operation*/
struct ShaderInfo
{
public:
	std::string name;				//Shader's name, used as key in map
	std::string fileName;			//Shader's filename (without 'system\\shaders\\')
	std::string type;				//Shader's type: 'v' vertexShader, 'p' pixelShader
	int layout;						//Shader's input layout
	std::vector<int> cBufferSizes;	//Vector with size for each constant buffer to be created for this shader

	//Constructor
	ShaderInfo(std::string n, std::string fn, std::string t, int l = 0)
	{
		name = n;
		fileName = fn;
		type = t;
		layout = l;
		cBufferSizes = std::vector<int>();
	}
};

class D3D11PShader;
class D3D11VShader;

class D3D11ShaderManager
{
public:
	D3D11ShaderManager();
	~D3D11ShaderManager();

	/** Creates list with ShaderInfos */
	XRESULT Init();

	/** Loads/Compiles Shaderes from list */
	XRESULT LoadShaders();

	/** Deletes all shaders and loads them again */
	XRESULT ReloadShaders();

	/** Called on frame start */
	XRESULT OnFrameStart();

	/** Deletes all shaders */
	XRESULT DeleteShaders();

	/** Return a specific shader */
	D3D11VShader* GetVShader(std::string shader);
	D3D11PShader* GetPShader(std::string shader);

private:
	std::vector<ShaderInfo> Shaders;							//Initial shader list for loading
	std::unordered_map<std::string, D3D11VShader*> VShaders;
	std::unordered_map<std::string, D3D11PShader*> PShaders;

	/** Whether we need to reload the shaders next frame or not */
	bool ReloadShadersNextFrame;
};