#pragma once
#ifndef MYDIRECT3DVERTEXBUFFER7_H
#define MYDIRECT3DVERTEXBUFFER7_H

#include "../pch.h"
#include <d3d.h>
#include "../Logger.h"
#include <vector>
#include "../BaseGraphicsEngine.h"
#include "../BaseVertexBuffer.h"
#include "../Engine.h"



class MyDirect3DVertexBuffer7 : public IDirect3DVertexBuffer7 {
public:
    MyDirect3DVertexBuffer7(const D3DVERTEXBUFFERDESC& originalDesc) {
        DebugWrite("MyDirect3DVertexBuffer7::MyDirect3DVertexBuffer7\n");

		// Save original desc
		OriginalDesc = originalDesc;

		// Create our own buffer
		XLE(Engine::GraphicsEngine->CreateVertexBuffer(&VertexBuffer));

		// Initialize it
		XLE(VertexBuffer->Init(NULL, OriginalDesc.dwNumVertices * ComputeFVFSize(OriginalDesc.dwFVF), BaseVertexBuffer::EBindFlags::B_VERTEXBUFFER, BaseVertexBuffer::EUsageFlags::U_DYNAMIC, BaseVertexBuffer::ECPUAccessFlags::CA_WRITE));

		// Start with 1 reference
		RefCount = 1;
    }

	
    /*** IUnknown methods ***/
    HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void** ppvObj) {
        //DebugWrite("MyDirect3DVertexBuffer7::QueryInterface\n");
        //return this->direct3DVertexBuffer7->QueryInterface(riid, ppvObj);

		LogError() << "QueryInterface on Vertexbuffer not supported!";
		// Lets hope this never gets called
		return S_OK;
    }

    ULONG STDMETHODCALLTYPE AddRef() {
        DebugWrite("MyDirect3DVertexBuffer7::AddRef\n");
        RefCount++;

		return RefCount;
    }

    ULONG STDMETHODCALLTYPE Release() {
        DebugWrite("MyDirect3DVertexBuffer7::Release\n");

		RefCount--;

        ULONG count = RefCount;
        if (0 == count) {
            delete this;
			return 0;
        }

        return count;
    }

    /*** IDirect3DVertexBuffer7 methods ***/
    HRESULT STDMETHODCALLTYPE GetVertexBufferDesc(LPD3DVERTEXBUFFERDESC lpVBDesc) {
        DebugWrite("MyDirect3DVertexBuffer7::GetVertexBufferDesc\n");

		if(lpVBDesc)*lpVBDesc = OriginalDesc;
        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE Lock(DWORD dwFlags, LPVOID* lplpData, LPDWORD lpdwSize) {
        DebugWrite("MyDirect3DVertexBuffer7::Lock\n");

		// Pass the lock-call through to our engine
		UINT size = 0;
		VertexBuffer->Map(BaseVertexBuffer::EMapFlags::M_WRITE_DISCARD, lplpData, &size);
		if(lpdwSize)*lpdwSize = size;

        return S_OK;
    }

    HRESULT STDMETHODCALLTYPE Optimize(LPDIRECT3DDEVICE7 lpD3DDevice, DWORD dwFlags)
	{
		// Not needed
		return S_OK;
	}

    HRESULT STDMETHODCALLTYPE ProcessVertices(DWORD dwVertexOp, DWORD dwDestIndex, DWORD dwCount, LPDIRECT3DVERTEXBUFFER7 lpSrcBuffer, DWORD dwSrcIndex, LPDIRECT3DDEVICE7 lpD3DDevice, DWORD dwFlags)
	{
		LogWarn() << "Unimplemented method: MyDirect3DVertexBuffer7::ProcessVertices";
		return S_OK;
	}

    HRESULT STDMETHODCALLTYPE ProcessVerticesStrided(DWORD dwVertexOp, DWORD dwDestIndex, DWORD dwCount, LPD3DDRAWPRIMITIVESTRIDEDDATA lpVertexArray, DWORD dwSrcIndex, LPDIRECT3DDEVICE7 lpD3DDevice, DWORD dwFlags)
	{
		LogWarn() << "Unimplemented method: MyDirect3DVertexBuffer7::ProcessVerticesStrided";
		return S_OK;
	}

    HRESULT STDMETHODCALLTYPE Unlock() {
        DebugWrite("MyDirect3DVertexBuffer7::Unlock\n");
		VertexBuffer->Unmap();
        return S_OK;
    }

	/** Returns the number of vertices inside this buffer */
	int GetNumVertices()
	{
		return OriginalDesc.dwNumVertices;
	}

	/** Returns the actual vertex buffer */
	BaseVertexBuffer* GetVertexBuffer()
	{
		return VertexBuffer;
	}

private:

	/** Original desc D3D7 created the buffer with */
	D3DVERTEXBUFFERDESC OriginalDesc;

	/** Our own vertex buffer */
	BaseVertexBuffer* VertexBuffer;

	/** Referencecount on this */
	int RefCount;
};

#endif
