#include "pch.h"
#include "GMesh.h"
#include "assimp--3.0.1270-sdk\include\assimp\Importer.hpp"
#include "assimp--3.0.1270-sdk\include\assimp\scene.h"
#include "assimp--3.0.1270-sdk\include\assimp\postprocess.h"
#include "assimp--3.0.1270-sdk\include\assimp\material.h"
#include "Engine.h"
#include "BaseGraphicsEngine.h"
#include "GothicAPI.h"

#pragma comment(lib, "assimp.lib")

using namespace Assimp;

GMesh::GMesh(void)
{
}


GMesh::~GMesh(void)
{
}


/** Load a mesh from file */
XRESULT GMesh::LoadMesh(const std::string& file)
{
	char dir[260];
	GetCurrentDirectoryA(260, dir);
	LogInfo() << "Loading custom mesh " << dir << "\\" << file;

	Importer imp;
	const aiScene* s = imp.ReadFile(file, aiProcessPreset_TargetRealtime_Fast);

	if(!s)
	{
		LogError() << "Failed to open custom Mesh: " << file;
		LogError() << " - " << imp.GetErrorString();
		return XR_FAILED;
	}

	LogInfo() << "Loading " << s->mNumMeshes << " submeshes";

	int startIndex = 0;
	for(unsigned int i=0; i < s->mNumMeshes; i++)
	{
		aiString texture;
		s->mMaterials[s->mMeshes[i]->mMaterialIndex]->GetTexture(aiTextureType::aiTextureType_DIFFUSE, 0, &texture);

		LogInfo() << " - Submesh: (Num Vertices: " << s->mMeshes[i]->mNumVertices << ") (Texture: " << texture.C_Str() << ")";

		MeshInfo* mi = new MeshInfo;

		ExVertexStruct* vertices = new ExVertexStruct[s->mMeshes[i]->mNumVertices];
		VERTEX_INDEX* indices = new VERTEX_INDEX[s->mMeshes[i]->mNumFaces * 3];

		for(unsigned int n=0;n<s->mMeshes[i]->mNumVertices;n++)
		{
			if(s->mMeshes[i]->HasNormals())
			{
				vertices[n].Normal = float3(s->mMeshes[i]->mNormals[n].x, s->mMeshes[i]->mNormals[n].y, s->mMeshes[i]->mNormals[n].z);
			}

			if(s->mMeshes[i]->HasTextureCoords(0))
			{
				vertices[n].TexCoord = float2(s->mMeshes[i]->mTextureCoords[0][n].x, s->mMeshes[i]->mTextureCoords[0][n].y);
			}

			if(s->mMeshes[i]->HasTextureCoords(1))
			{
				vertices[n].TexCoord2 = float2(s->mMeshes[i]->mTextureCoords[1][n].x, s->mMeshes[i]->mTextureCoords[1][n].y);
			}

			vertices[n].Color = 0xFFFFFFFF;
			vertices[n].Position = float3(s->mMeshes[i]->mVertices[n].x, s->mMeshes[i]->mVertices[n].y, s->mMeshes[i]->mVertices[n].z);
		}

		for(unsigned int n=0;n<s->mMeshes[i]->mNumFaces;n++)
		{
			if(s->mMeshes[i]->mFaces[n].mNumIndices != 3)
			{
				LogError() << "Mesh not triangulated!";
				continue;
			}

			indices[3*n] = s->mMeshes[i]->mFaces[n].mIndices[0] - startIndex;
			indices[3*n+1] = s->mMeshes[i]->mFaces[n].mIndices[2] - startIndex;
			indices[3*n+2] = s->mMeshes[i]->mFaces[n].mIndices[1] - startIndex;
		}

		std::string stex = texture.C_Str();
		std::string ext;
		std::string name = stex;

		// Extract the file extension and its name
		int extpos = stex.find_last_of(".");
		if(extpos >= 0)
		{
			ext = &stex[extpos + 1];
			//LogInfo() << "Got file ext: " << ext;

			name.resize(name.size() - (ext.size() + 1)); // Strip file extension
			//LogInfo() << "Got file name: " << name;
		}

		mi->Create(vertices, s->mMeshes[i]->mNumVertices, indices, s->mMeshes[i]->mNumFaces * 3);
		Meshes.push_back(mi);

		Textures.push_back(std::string(name.c_str()));

		delete[] vertices;
		delete[] indices;

		//startIndex += s->mMeshes[i]->mNumFaces * 3;
	}

	return XR_SUCCESS;
}

/** Draws all buffers this holds */
void GMesh::DrawMesh()
{
	for(unsigned int i=0;i<Meshes.size();i++)
	{
		Engine::GAPI->DrawMeshInfo(NULL, Meshes[i]);
	}
}