#include "pch.h"
#include "GothicAPI.h"
#include "Engine.h"
#include "BaseGraphicsEngine.h"
#include "zCPolygon.h"
#include "WorldConverter.h"
#include "HookedFunctions.h"
#include "zCMaterial.h"
#include "zCTexture.h"
#include "zCVisual.h"
#include "zCVob.h"
#include "zCProgMeshProto.h"
#include "zCCamera.h"
#include "oCGame.h"
#include "zCModel.h"
//#include "ReferenceD3D11GraphicsEngine.h"
#include "zCMorphMesh.h"
#include "zCParticleFX.h"
#include "GSky.h"
#include "GInventory.h"

#define DIRECTINPUT_VERSION 0x0700
#include <dinput.h>
#include "BaseAntTweakBar.h"
#include "zCInput.h"
#include "zCBspTree.h"
#include "BaseLineRenderer.h"
#include "D3D7\MyDirect3DDevice7.h"
#include "GVegetationBox.h"

/** Writes this info to a file */
void MaterialInfo::WriteToFile(const std::string& name)
{
	FILE* f = fopen(("system\\GD3D11\\textures\\infos\\" + name + ".mi").c_str(), "wb");

	if(!f)
	{
		LogError() << "Failed to open file '" << ("system\\GD3D11\\textures\\infos\\" + name + ".mi") << "' for writing! Make sure the game runs in Admin mode "
					  " to get the rights to write to that directory!";

		return;
	}

	// Write the version first
	fwrite(&MATERIALINFO_VERSION, sizeof(MATERIALINFO_VERSION), 1, f);

	// Then the data
	fwrite(this, sizeof(MaterialInfo), 1, f);

	fclose(f);
}

/** Loads this info from a file */
void MaterialInfo::LoadFromFile(const std::string& name)
{
	FILE* f = fopen(("system\\GD3D11\\textures\\infos\\" + name + ".mi").c_str(), "rb");

	if(!f)
		return;

	// Write the version first
	int version;
	fread(&version, sizeof(int), 1, f);

	// Then the data
	fread(this, sizeof(MaterialInfo), 1, f);

	fclose(f);
}

GothicAPI::GothicAPI(void)
{
	LoadedWorldInfo = new WorldInfo;
	LoadedWorldInfo->HighestVertex = 2;
	LoadedWorldInfo->LowestVertex = 3;
	LoadedWorldInfo->MidPoint = D3DXVECTOR2(4,5);

	// Get start directory
	char dir[MAX_PATH];
	GetCurrentDirectoryA(MAX_PATH, dir);
	StartDirectory = dir;

	InitializeCriticalSection(&ResourceCriticalSection);

	SkyRenderer = new GSky;
	SkyRenderer->InitSky();

	Inventory = new GInventory; 

	OriginalGothicWndProc = 0;

	TextureTestBindMode = false;

	ZeroMemory(BoundTextures, sizeof(BoundTextures));

	CameraReplacementPtr = NULL;
}


GothicAPI::~GothicAPI(void)
{
	delete SkyRenderer;
	delete Inventory;
	delete LoadedWorldInfo;

	ResetWorld();


}

/** Called to update the world, before rendering */
void GothicAPI::OnWorldUpdate()
{
	if(Engine::AntTweakBar->GetActive())
		SetEnableGothicInput(false);

	RendererState.RendererInfo.Reset();
	RendererState.RendererInfo.FPS = GetFramesPerSecond();
	RendererState.GraphicsState.FF_Time = GetTimeSeconds();

	if(zCCamera::GetCamera())
	{
		RendererState.RendererInfo.FarPlane = zCCamera::GetCamera()->GetFarPlane();
		RendererState.RendererInfo.NearPlane = zCCamera::GetCamera()->GetNearPlane();
	}
}

/** Returns gothics fps-counter */
int GothicAPI::GetFramesPerSecond()
{
	return ((vidGetFPSRate)GothicMemoryLocations::Functions::vidGetFPSRate)();
}

/** Returns global time */
float GothicAPI::GetTimeSeconds()
{
	if(zCTimer::GetTimer())
		return zCTimer::GetTimer()->totalTimeFloatSecs;

	return 0.0f;
}

/** Returns the current frame time */
float GothicAPI::GetFrameTimeSec()
{
	if(zCTimer::GetTimer())
		return zCTimer::GetTimer()->frameTimeFloatSecs;

	return -1.0f;
}

/** Disables the input from gothic */
void GothicAPI::SetEnableGothicInput(bool value)
{
	zCInput* input = zCInput::GetInput();

	if(!input)
		return;

	// zMouse, false
	input->SetDeviceEnabled(2, value);
	input->SetDeviceEnabled(1, value);

	// GOTHIC 2 SPECIFIC
	IDirectInputDevice7A* dInputMouse = *(IDirectInputDevice7A **)GothicMemoryLocations::GlobalObjects::DInput7DeviceMouse;
	IDirectInputDevice7A* dInputKeyboard = *(IDirectInputDevice7A **)GothicMemoryLocations::GlobalObjects::DInput7DeviceKeyboard;
	if(dInputMouse)
	{
		//LogInfo() << "Got DInput (Mouse)!";
		if(!value)
			dInputMouse->Unacquire();
	}

	if(dInputKeyboard)
	{
		//LogInfo() << "Got DInput (Keyboard)!";
		if(!value)
			dInputKeyboard->Unacquire();
	}

}


/** Called when the window got set */
void GothicAPI::OnSetWindow(HWND hWnd)
{
	if(OriginalGothicWndProc || !hWnd)
		return; // Dont do that twice

	OutputWindow = hWnd;

	// Start here, create our engine
	Engine::GraphicsEngine->SetWindow(hWnd);

	OriginalGothicWndProc = GetWindowLongPtrA(hWnd, GWL_WNDPROC);
	SetWindowLongPtrA(hWnd, GWL_WNDPROC, (LONG)GothicWndProc);
}

/** Returns the GraphicsState */
GothicRendererState* GothicAPI::GetRendererState()
{
	return &RendererState;
}


/** Spawns a vegetationbox at the camera */
GVegetationBox* GothicAPI::SpawnVegetationBoxAt(const D3DXVECTOR3& position, 
												const D3DXVECTOR3& min, 
												const D3DXVECTOR3& max, 
												float density,
												const std::string& restrictByTexture)
{
	GVegetationBox* v = new GVegetationBox;
	v->InitVegetationBox(min + position, max + position, "", density, 1.0f, restrictByTexture);

	VegetationBoxes.push_back(v);

	return v;
}


/** Adds a vegetationbox to the world */
void GothicAPI::AddVegetationBox(GVegetationBox* box)
{
	VegetationBoxes.push_back(box);
}

/** Resets the object, like at level load */
void GothicAPI::ResetWorld()
{
	WorldSections.clear();

	for(std::map<zCProgMeshProto*, MeshVisualInfo*>::iterator it = StaticMeshVisuals.begin(); it != StaticMeshVisuals.end();it++)
	{
		delete (*it).second;
	}
	StaticMeshVisuals.clear();

	for(std::map<zCModel*, SkeletalMeshVisualInfo*>::iterator it = SkeletalMeshVisuals.begin(); it != SkeletalMeshVisuals.end();it++)
	{
		delete (*it).second;
	}
	SkeletalMeshVisuals.clear();

	for(std::list<GVegetationBox *>::iterator it = VegetationBoxes.begin(); it != VegetationBoxes.end();it++)
	{
		delete (*it);
	}
	VegetationBoxes.clear();

	/*for(std::list<SkeletalVobInfo *>::iterator it = SkeletalMeshVobs.begin(); it != SkeletalMeshVobs.end(); it++)
	{
		delete (*it);
	}
	SkeletalMeshVobs.clear();*/

	//ParticleEffectVobs.clear();

	RegisteredVobs.clear();
	VobMap.clear();
	BspLeafVobLists.clear();

	for(std::list<SkeletalVobInfo *>::iterator it = SkeletalMeshVobs.begin(); it != SkeletalMeshVobs.end(); it++)
	{
		delete (*it);
	}

	for(std::hash_map<zCVobLight*, VobLightInfo*>::iterator it = VobLightMap.begin(); it != VobLightMap.end(); it++)
	{
		delete (*it).second;
	}


	// Clear inventory too?
}

/** Called when the game loaded a new level */
void GothicAPI::OnGeometryLoaded(zCPolygon** polys, unsigned int numPolygons)
{
	LogInfo() << "Extracting world";
	 
	ResetWorld();

	// Convert world to our own format
	WorldConverter::ConvertWorldMesh(polys, numPolygons, &WorldSections, LoadedWorldInfo);

	/*for(unsigned int i=0;i<numPolygons;i++)
	{
		zCPolygon* poly = polys[i];

		OnMaterialCreated(poly->GetMaterial());
	}

	WorldConverter::LoadWorldMeshFromFile("system\\GD3D11\\meshes\\GRM.3DS", &WorldSections, LoadedWorldInfo);*/

	LogInfo() << "Done extracting world!";
}

/** Called when the game is done loading the world */
void GothicAPI::OnWorldLoaded()
{
	LogInfo() << "Collecting vobs...";
	
	// Get all VOBs
	zCTree<zCVob>* vobTree = oCGame::GetGame()->_zCSession_world->GetGlobalVobTree();
	TraverseVobTree(vobTree);
	 
	// Build instancing cache for the static vobs for each section
	BuildStaticMeshInstancingCache();

	// Build vob info cache for the bsp-leafs
	BuildBspVobMapCache();

	LogInfo() << "Done!";
}



/** Goes through the given zCTree and registeres all found vobs */
void GothicAPI::TraverseVobTree(zCTree<zCVob>* tree)
{
	// Iterate through the nodes
	if(tree->FirstChild != NULL)
	{
		TraverseVobTree(tree->FirstChild);
	}

	if(tree->Next != NULL)
	{
		TraverseVobTree(tree->Next);
	}

	// Add the vob if it exists and has a visual
	if(tree->Data)
	{
		if(tree->Data->GetVisual())
			OnAddVob(tree->Data, (zCWorld *)oCGame::GetGame()->_zCSession_world);
	}
}

/** Returns in which directory we started in */
const std::string& GothicAPI::GetStartDirectory()
{
	return StartDirectory;
}

/** Builds the static mesh instancing cache */
void GothicAPI::BuildStaticMeshInstancingCache()
{
	for(std::map<zCProgMeshProto*, MeshVisualInfo*>::iterator it = StaticMeshVisuals.begin(); it != StaticMeshVisuals.end(); it++)
	{
		(*it).second->StartNewFrame();
	}

	// Cache everything
	for(std::map<int, std::map<int, WorldMeshSectionInfo>>::iterator itx = Engine::GAPI->GetWorldSections().begin(); itx != Engine::GAPI->GetWorldSections().end(); itx++)
	{
		for(std::map<int, WorldMeshSectionInfo>::iterator ity = (*itx).second.begin(); ity != (*itx).second.end(); ity++)
		{
			WorldMeshSectionInfo& section = (*ity).second;

			// Cache positions for the vobs
			for(std::list<VobInfo*>::iterator itv = section.Vobs.begin(); itv != section.Vobs.end();itv++)
			{
				section.InstanceCache.ClearCacheForStatic((*itv)->VisualInfo);
			}

			// Cache positions for the vobs
			for(std::list<VobInfo*>::iterator itv = section.Vobs.begin(); itv != section.Vobs.end();itv++)
			{
				VS_ExConstantBuffer_PerInstance cb;
				cb.World = *(*itv)->Vob->GetWorldMatrixPtr();

				section.InstanceCache.InstanceCacheData[(*itv)->VisualInfo].push_back(cb);
			}

			// Generate constantbuffers
			for(std::map<MeshVisualInfo *, std::vector<VS_ExConstantBuffer_PerInstance>>::iterator it = section.InstanceCache.InstanceCacheData.begin(); it != section.InstanceCache.InstanceCacheData.end(); it++)
			{				
				Engine::GraphicsEngine->CreateVertexBuffer(&section.InstanceCache.InstanceCache[(*it).first]);
				
				section.InstanceCache.InstanceCache[(*it).first]->Init(&(*it).second[0], (*it).second.size() * sizeof(VS_ExConstantBuffer_PerInstance));
			}

			// Generate full section mesh for the current section
			//WorldConverter::GenerateFullSectionMesh(section);
		}
	}
}

/** Draws the world-mesh */
void GothicAPI::DrawWorldMeshNaive()
{
	if(!zCCamera::GetCamera() || !oCGame::GetGame())
		return;

	FrameParticleInfo.clear();
	FrameParticles.clear();
	FrameMeshInstances.clear();

	Engine::GraphicsEngine->DrawWorldMesh();
	
	for(std::list<GVegetationBox *>::iterator it = VegetationBoxes.begin(); it != VegetationBoxes.end();it++)
	{
		(*it)->RenderVegetation(GetCameraPosition());
	}


	//Engine::GraphicsEngine->SetActivePixelShader("PS_Simple");

	// Get camera position (TODO: Make a function for this)
	
	D3DXVECTOR3 camPos = GetCameraPosition();

	INT2 camSection = WorldConverter::GetSectionOfPos(camPos);
	//camSection.x += 1;
	//camSection.y += 1;

	D3DXMATRIX view;
	GetViewMatrix(&view);

	// Clear instances
	int sectionViewDist = Engine::GAPI->GetRendererState()->RendererSettings.SectionDrawRadius;


	if(RendererState.RendererSettings.DrawSkeletalMeshes)
	{
		Engine::GraphicsEngine->SetActivePixelShader("PS_World");

		// Set up frustum for the camera
		zCCamera::GetCamera()->Activate();

		for(std::list<SkeletalVobInfo *>::iterator it = SkeletalMeshVobs.begin(); it != SkeletalMeshVobs.end(); it++)
		{
			INT2 s = WorldConverter::GetSectionOfPos((*it)->Vob->GetPositionWorld());

			//if(abs(s.x - camSection.x) > sectionViewDist || abs(s.y - camSection.y) > sectionViewDist)
			//	continue;

			if(D3DXVec3Length(&((*it)->Vob->GetPositionWorld() - camPos)) > RendererState.RendererSettings.SkeletalMeshDrawRadius)
				continue; // Skip out of range

			zTBBox3D bb = (*it)->Vob->GetBBoxLocal();
			zCCamera::GetCamera()->SetTransform(zCCamera::ETransformType::TT_WORLD, *(*it)->Vob->GetWorldMatrixPtr());

			//Engine::GraphicsEngine->GetLineRenderer()->AddAABBMinMax(bb.Min, bb.Max, D3DXVECTOR4(1,1,1,1));

			int clipFlags = 15; // No far clip
			if(zCCamera::GetCamera()->BBox3DInFrustum(bb, clipFlags) == ZTCAM_CLIPTYPE_OUT)
				continue;

			DrawSkeletalMeshVob((*it));
		}
	}

	RendererState.RasterizerState.CullMode = GothicRasterizerStateInfo::CM_CULL_FRONT;
	RendererState.RasterizerStateDirty = true;


	// Draw vobs in view
	Engine::GraphicsEngine->DrawVOBs();

	if(RendererState.RendererSettings.DrawParticleEffects)
	{
		for(std::list<zCVob *>::iterator it = ParticleEffectVobs.begin(); it != ParticleEffectVobs.end(); it++)
		{
			INT2 s = WorldConverter::GetSectionOfPos((*it)->GetPositionWorld());

			//if(abs(s.x - camSection.x) > 2  || abs(s.y - camSection.y) > 2)
			//	continue;

			float dist = D3DXVec3Length(&((*it)->GetPositionWorld() - camPos));
			if(dist > RendererState.RendererSettings.OutdoorSmallVobDrawRadius)
				continue;

			if((*it)->GetVisual())
			{
				DrawParticleFX((*it), (zCParticleFX *)(*it)->GetVisual());
			}
		}

		DrawTestInstances();
	}

	//DebugDrawBSPTree();

	ResetWorldTransform();
	ResetViewTransform();

	/*for(int i=0;i<SkeletalMeshVobs.size();i++)
	{
		DrawSkeletalMeshVob(SkeletalMeshVobs[i]);
	}*/
}

/** Called when a material got removed */
void GothicAPI::OnMaterialDeleted(zCMaterial* mat)
{
	LoadedMaterials.erase(mat);
	if(!mat)
		return;

	/** Map for static mesh visuals */
	for(std::map<zCProgMeshProto*, MeshVisualInfo*>::iterator it = StaticMeshVisuals.begin(); it != StaticMeshVisuals.end(); it++)
	{
		for(std::map<zCMaterial *, std::vector<MeshInfo*>>::iterator itm = (*it).second->Meshes.begin(); itm != (*it).second->Meshes.end(); itm++)
		{
			if((*itm).first == mat)
			{
				(*it).second->UnloadedSomething = true;
			}
		}
	}
	
}

/** Called when a material got removed */
void GothicAPI::OnMaterialCreated(zCMaterial* mat)
{
	LoadedMaterials.insert(mat);
}

/** Returns if the material is currently active */
bool GothicAPI::IsMaterialActive(zCMaterial* mat)
{
	std::set<zCMaterial *>::iterator it = LoadedMaterials.find(mat);
	if(it != LoadedMaterials.end())
	{
		return true;
	}

	return false;
}

/** Called when a visual got removed */
void GothicAPI::OnVisualDeleted(zCVisual* visual)
{
	std::vector<std::string> extv;
	

	// Get the visuals possible file extensions
	int e=0;
	while(strlen(visual->GetFileExtension(e)) > 0)
	{
		extv.push_back(visual->GetFileExtension(e));
		e++;
	}

	// Check every extension
	for(unsigned int i=0; i<extv.size();i++)
	{
		std::string ext = extv[i];

		// Delete according to the type
		if(ext == ".3DS")
		{
			// Clear the visual from all vobs (FIXME: This may be slow!)
			for(std::hash_map<zCVob*, VobInfo*>::iterator it = VobMap.begin();it != VobMap.end();it++)
			{
				if(!(*it).second->VisualInfo) // This happens sometimes, so get rid of it
				{
					it = VobMap.erase(it);
					continue;
				}

				if((*it).second->VisualInfo->Visual == (zCProgMeshProto *)visual)
				{
					(*it).second->VisualInfo = NULL;
				}
			}

			delete StaticMeshVisuals[(zCProgMeshProto *)visual];
			StaticMeshVisuals.erase((zCProgMeshProto *)visual);

			
			
			break;
		}else if(ext == ".MDS" || ext == ".ASC")
		{
			delete SkeletalMeshVisuals[(zCModel *)visual];
			SkeletalMeshVisuals.erase((zCModel *)visual);
			break;
		}
	}
}
/** Draws a MeshInfo */
void GothicAPI::DrawMeshInfo(zCMaterial* mat, MeshInfo* msh)
{
	// Check for material and bind the texture if it exists
	if(mat)
	{
		if(mat->GetTexture())
		{
			/*if((*it).first->GetTexture()->GetCacheState() != zRES_CACHED_IN)
			{
			(*it).first->GetTexture()->CacheIn(0.6f);
			}else
			{
			(*it).first->GetTexture()->Bind(0);
			}*/

			// Setup alphatest //FIXME: This has to be done earlier!
			if(mat->GetAlphaFunc() == zRND_ALPHA_FUNC_TEST)
				RendererState.GraphicsState.FF_GSwitches |= GSWITCH_ALPHAREF;
			else
				RendererState.GraphicsState.FF_GSwitches &= ~GSWITCH_ALPHAREF;
		}
	}

	if(!msh->MeshIndexBuffer)
	{
		Engine::GraphicsEngine->DrawVertexBuffer(msh->MeshVertexBuffer, msh->Vertices.size());
	}else
	{
		Engine::GraphicsEngine->DrawVertexBufferIndexed(msh->MeshVertexBuffer, msh->MeshIndexBuffer, msh->Indices.size());
	}
}

/** Draws a SkeletalMeshInfo */
void GothicAPI::DrawSkeletalMeshInfo(zCMaterial* mat, SkeletalMeshInfo* msh, std::vector<D3DXMATRIX>& transforms)
{
	// Check for material and bind the texture if it exists
	if(mat)
	{
		if(mat->GetTexture())
		{
			if(mat->GetAniTexture()->CacheIn(0.6f) == zRES_CACHED_IN)
				mat->GetAniTexture()->Bind(0);
		}
	}

	Engine::GraphicsEngine->DrawSkeletalMesh(msh->MeshVertexBuffer, msh->MeshIndexBuffer, msh->Indices.size(), transforms);
}

/** Locks the resource CriticalSection */
void GothicAPI::EnterResourceCriticalSection()
{
	EnterCriticalSection(&ResourceCriticalSection);
}

/** Unlocks the resource CriticalSection */
void GothicAPI::LeaveResourceCriticalSection()
{
	LeaveCriticalSection(&ResourceCriticalSection);
}

/** Called when a VOB got removed from the world */
void GothicAPI::OnRemovedVob(zCVob* vob, zCWorld* world)
{
	// Erase the vob from the global set
	RegisteredVobs.erase(vob);
	
	// Check if this was in some inventory
	if(Inventory->OnRemovedVob(vob, world))
		return; // Don't search in the other lists since it wont be in it anyways

	if(world != LoadedWorldInfo->MainWorld)
		return; // *should* be already deleted from the inventory here. But watch out for dem leaks, dragons be here!

	VobInfo* vi = VobMap[vob];
	SkeletalVobInfo* svi = SkeletalVobMap[vob];

	// Erase it from the particle-effect list
	ParticleEffectVobs.remove(vob);

	for(std::hash_map<zCBspBase *, BspVobInfo>::iterator it = BspLeafVobLists.begin(); it != BspLeafVobLists.end(); it++)
	{
		(*it).second.IndoorVobs.remove(vi);
		(*it).second.SmallVobs.remove(vi);
		(*it).second.Vobs.remove(vi);
	}

	SkeletalVobMap.erase(vob);

	// Erase the vob from the section

	if(vi && vi->VobSection)
	{
		vi->VobSection->Vobs.remove(vi);
	}
	
	// Erase it from the skeletal vob-list
	for(std::list<SkeletalVobInfo *>::iterator it = SkeletalMeshVobs.begin(); it != SkeletalMeshVobs.end(); it++)
	{
		if((*it)->Vob == vob)
		{
			SkeletalVobInfo* vi = (*it);
			SkeletalMeshVobs.erase(it);

			delete vi;
			break;
		}
	}


	std::hash_map<zCVob*, VobInfo*>::iterator vit = VobMap.find(vob);
	if(vit != VobMap.end())
	{
		delete (*vit).second;
		VobMap.erase(vob);
	}

	/*for(std::list<VobInfo *>::iterator it = WorldSections[s.x][s.y].Vobs.begin(); it != WorldSections[s.x][s.y].Vobs.end(); it++)
	{
		if((*it)->Vob == vob)
		{
			VobInfo* vi = (*it);
			WorldSections[s.x][s.y].Vobs.remove((*it));

			delete vi;
			break;
		}
	}

	for(std::list<VobInfo *>::iterator it = WorldSections[s.x][s.y].Vobs.begin(); it != WorldSections[s.x][s.y].Vobs.end(); it++)
	{
		if((*it)->Vob == vob)
		{
			VobInfo* vi = (*it);
			WorldSections[s.x][s.y].Vobs.remove((*it));

			delete vi;
			break;
		}
	}*/


}

/** Called when a VOB got added to the BSP-Tree */
void GothicAPI::OnAddVob(zCVob* vob, zCWorld* world)
{
	// Add the vob to the set
	std::set<zCVob *>::iterator it = RegisteredVobs.find(vob);
	if(it != RegisteredVobs.end())
	{
		// Already got that
		return;
	}

	RegisteredVobs.insert(vob);

	
	std::vector<std::string> extv;
	
	int e=0;
	while(strlen(vob->GetVisual()->GetFileExtension(e)) > 0)
	{
		extv.push_back(vob->GetVisual()->GetFileExtension(e));
		e++;
	}

	for(unsigned int i=0; i<extv.size();i++)
	{
		std::string ext = extv[i];

		if(ext == ".3DS")
		{
			if(StaticMeshVisuals.count((zCProgMeshProto *)vob->GetVisual()) == 0)
			{
				// Load the new visual
				MeshVisualInfo* mi = new MeshVisualInfo;

				LogInfo() << "Loading: " << vob->GetVisual()->GetObjectName();
				if (strncmp(vob->GetVisual()->GetObjectName(), "ITAR_", 5) == 0)
				{
					LogInfo() << "OMG!";
					return;
				}

				WorldConverter::Extract3DSMeshFromVisual2((zCProgMeshProto *)vob->GetVisual(), mi);

				StaticMeshVisuals[(zCProgMeshProto *)vob->GetVisual()] = mi;
			}

			INT2 section = WorldConverter::GetSectionOfPos(vob->GetPositionWorld());

			VobInfo* vi = new VobInfo;
			vi->Vob = vob;
			vi->VisualInfo = StaticMeshVisuals[(zCProgMeshProto *)vob->GetVisual()];
			Engine::GraphicsEngine->CreateConstantBuffer(&vi->VobConstantBuffer, NULL, sizeof(VS_ExConstantBuffer_PerInstance));

			vi->UpdateVobConstantBuffer();

			

			// Check for mainworld
			if(world == oCGame::GetGame()->_zCSession_world)
			{
				VobMap[vob] = vi;
				WorldSections[section.x][section.y].Vobs.push_back(vi);

				vi->VobSection = &WorldSections[section.x][section.y];
			}else
			{
				// Must be inventory
				Inventory->OnAddVob(vi, world);
			}
			break;
		}else if(ext == ".MDS" || ext == ".ASC")
		{
			if(SkeletalMeshVisuals.count((zCModel *)vob->GetVisual()) == 0)
			{
				// Load the new visual
				SkeletalMeshVisualInfo* mi = new SkeletalMeshVisualInfo;
				WorldConverter::ExtractSkeletalMeshFromVob((zCModel *)vob->GetVisual(), mi);

				SkeletalMeshVisuals[(zCModel *)vob->GetVisual()] = mi;
			}
		
			// Add vob to the skeletal list
			SkeletalVobInfo* vi = new SkeletalVobInfo;
			vi->Vob = vob;
			vi->VisualInfo = SkeletalMeshVisuals[(zCModel *)vob->GetVisual()];

			// Check for mainworld
			if(world == oCGame::GetGame()->_zCSession_world)
			{
				SkeletalMeshVobs.push_back(vi);
				SkeletalVobMap[vob] = vi;
			}else
			{
				// Must be inventory
				//Inventory->OnAddVob(vi, world);
			}
			break;
		}else if(ext == ".PFX")
		{
			if(!BspLeafVobLists.empty())
				return; // FIXME: Don't add particle effects after the loading time, since some effects (like landing from a jump?) will crash the game!

			ParticleEffectVobs.push_back(vob);

			tempParticleNames[vob] = ((zCParticleFX *)vob->GetVisual())->GetEmitter()->particleFXName.ToChar() + std::string(" | ") + ((zCParticleFX *)vob->GetVisual())->GetEmitter()->visName_S.ToChar();
			break;
		}
	}
}

// FIXME: REMOVE!!
#include "D3D11GraphicsEngine.h"

/** Draws a skeletal mesh-vob */
void GothicAPI::DrawSkeletalMeshVob(SkeletalVobInfo* vi)
{
	std::vector<D3DXMATRIX> transforms;
	zCModel* model = (zCModel *)vi->Vob->GetVisual();

	if(!model)
		return; // Gothic fortunately sets this to 0 when it throws the model out of the cache


	D3DXMATRIX world;
	vi->Vob->GetWorldMatrix(&world);
	zCCamera::GetCamera()->SetTransform(zCCamera::TT_WORLD, world);

	// Get the bone transforms
	model->GetBoneTransforms(&transforms, vi->Vob);

	// Update textures
	/*for(int i=0;i<model->GetMeshLibList()->NumInArray;i++)
	{
		model->GetMeshLibList()->Array[i]->TexAniState.UpdateTexList();
	}*/

	// Update attachments
	model->UpdateAttachedVobs();

	D3DXMATRIX view;
	GetViewMatrix(&view);
	SetWorldViewTransform(world, view);

	for(unsigned int i=0;i<transforms.size();i++)
	{
		//D3DXMatrixTranspose(&transforms[i], &transforms[i]);
	}

	// Draw submeshes
	for(std::map<zCMaterial *, std::vector<SkeletalMeshInfo*>>::iterator itm = vi->VisualInfo->Meshes.begin(); itm != vi->VisualInfo->Meshes.end();itm++)
	{
		for(unsigned int i=0;i<(*itm).second.size();i++)
			DrawSkeletalMeshInfo((*itm).first, (*itm).second[i], transforms);
	}

	/*D3DXMATRIX identity;
	D3DXMatrixIdentity(&identity);
	zCCamera::GetCamera()->SetTransform(zCCamera::TT_WORLD, identity);*/

	// FIXME: Put this into the renderer!!
	D3D11GraphicsEngine* g = (D3D11GraphicsEngine *)Engine::GraphicsEngine;
	g->SetActiveVertexShader("VS_Ex");
	
	for(unsigned int i=0;i<transforms.size();i++)
	{
		// Check for new visual
		zCModel* visual = (zCModel *)vi->Vob->GetVisual();
		zCModelNodeInst* node = visual->GetNodeList()->Array[i];

		if(!node->NodeVisual)
			continue; // Happens when you pull your sword for example

		// Check if this is loaded
		if(node->NodeVisual && vi->VisualInfo->NodeAttachments.find(i) == vi->VisualInfo->NodeAttachments.end())
		{
			// It's not, extract it
			WorldConverter::ExtractNodeVisual(i, node, vi->VisualInfo);
		}

		// Check for changed visual
		/*if(vi->VisualInfo->NodeAttachments[i].size() && node->NodeVisual != (zCVisual *)vi->VisualInfo->NodeAttachments[i][0]->Visual)
		{
			// Visual changed
			delete vi->VisualInfo->NodeAttachments[i][0];

			if(vi->VisualInfo->NodeAttachments[i].size() > 1)
				LogWarn() << "MEMLEAK: Node has more than one attachment!";

			vi->VisualInfo->NodeAttachments[i].clear();

			// Load the new one
			WorldConverter::ExtractNodeVisual(i, node, vi->VisualInfo);
		}*/

		if(vi->VisualInfo->NodeAttachments.count(i))
		{
			//RendererState.TransformState.TransformWorld = transforms[i] * RendererState.TransformState.TransformWorld;

			// Go through all attachments this node has
			for(unsigned int n=0;n<vi->VisualInfo->NodeAttachments[i].size();n++)
			{
				SetWorldViewTransform(world * transforms[i], view);

				// Update animated textures
				node->TexAniState.UpdateTexList();

				if(std::string(node->NodeVisual->GetFileExtension(0)) == ".MMS")
				{
					zCMorphMesh* mm = (zCMorphMesh *)node->NodeVisual;
					mm->GetTexAniState()->UpdateTexList();
				}

				g->SetupVS_ExMeshDrawCall();
				g->SetupVS_ExConstantBuffer();
				g->SetupVS_ExPerInstanceConstantBuffer();

				// Go through all materials registered here
				for(std::map<zCMaterial *, std::vector<MeshInfo*>>::iterator itm = vi->VisualInfo->NodeAttachments[i][n]->Meshes.begin(); 
					itm != vi->VisualInfo->NodeAttachments[i][n]->Meshes.end();
					itm++)
				{
					if((*itm).first && (*itm).first->GetAniTexture())
					{
						if((*itm).first->GetAniTexture()->CacheIn(0.6f) == zRES_CACHED_IN)
							(*itm).first->GetAniTexture()->Bind(0);
					}

					// Go through all meshes using that material
					for(unsigned int m=0;m<(*itm).second.size();m++)
					{
						DrawMeshInfo((*itm).first, (*itm).second[m]);
					}
				}
			}
			//((ReferenceD3D11GraphicsEngine *)Engine::GraphicsEngine)->DrawTriangle();
		}
	}

	RendererState.RendererInfo.FrameDrawnVobs++;
}

/** Called when a particle system got removed */
void GothicAPI::OnParticleFXDeleted(zCParticleFX* pfx)
{
	// Remove this from our list
	for(std::list<zCVob *>::iterator it = ParticleEffectVobs.begin();it != ParticleEffectVobs.end(); it++)
	{
		if((*it)->GetVisual() == (zCVisual *)pfx)
			it = ParticleEffectVobs.erase(it);
	}
}

/** Draws a zCParticleFX */
void GothicAPI::DrawParticleFX(zCVob* source, zCParticleFX* fx)
{
	// Set the current vob
	//fx->SetVisualUsedBy(source);


	if(tempParticleNames.find(source) == tempParticleNames.end() || tempParticleNames[source].size() == 0)
	{
		LogInfo() << "Failed to find ParticleEffect at position " << float3(source->GetPositionWorld()).toString();

		return;
	}

	

	// Get our view-matrix
	D3DXMATRIX view;
	GetViewMatrix(&view);	
	
	D3DXMATRIX world;
	source->GetWorldMatrix(&world);
	SetWorldViewTransform(world, view);

	D3DXMATRIX scale;
	float effectBrightness = 1.0f;

	//DrawTriangle();

	// Update effects time
	fx->UpdateTime();


	fx->UpdateParticleFX();

	zTParticle* pfx = fx->GetFirstParticle();
	if(pfx)
	{
		// Get texture
		zCTexture* texture = NULL;
		if(fx->GetEmitter())
		{
			texture = fx->GetEmitter()->GetVisTexture();
			if(texture)
			{
				// Bind it
				if(texture->CacheIn(0.6f) == zRES_CACHED_IN)
					texture->Bind(0);
			}
		}

		// Set render states for this type
		ParticleRenderInfo inf;

		switch(fx->GetEmitter()->GetVisAlphaFunc())
		{
		case zRND_ALPHA_FUNC_ADD:
			inf.BlendState.SetAdditiveBlending();
			effectBrightness = 5.0f;
			break;

		case zRND_ALPHA_FUNC_BLEND:
		default:
			inf.BlendState.SetDefault();
			inf.BlendState.BlendEnabled = true;
			break;

		
		}

		FrameParticleInfo[texture] = inf;

		// Check for kill
		zTParticle*	kill = NULL;
		zTParticle*	p = NULL;

		/*for (;;) 
		{
			kill = pfx;
			if (kill && (kill->LifeSpan < *fx->GetPrivateTotalTime())) 
			{			
				//if (kill->PolyStrip)	
				//	kill->polyStrip->Release(); // TODO: MEMLEAK RIGHT HERE!

				pfx = kill->Next;
				kill->Next = *(zTParticle **)GothicMemoryLocations::GlobalObjects::s_globFreePart;
				*(zTParticle **)GothicMemoryLocations::GlobalObjects::s_globFreePart = kill;
				continue;
			}
			break;
		}*/

		int i=0;
		for (p = pfx ; p ; p=p->Next) 
		{
			//if(i == fx->GetNumParticlesThisFrame() || i == 2048)
			//	break;

			for ( ;; ) 
			{
				kill = p->Next;
				if (kill && (kill->LifeSpan < *fx->GetPrivateTotalTime() )) 
				{
					//if (kill->PolyStrip)	
					//	kill->polyStrip->Release(); // TODO: MEMLEAK RIGHT HERE!

					p->Next			= kill->Next;
					kill->Next = *(zTParticle **)GothicMemoryLocations::GlobalObjects::s_globFreePart;
					*(zTParticle **)GothicMemoryLocations::GlobalObjects::s_globFreePart = kill;
					continue;
				}
				break;
			}

			// Construct world matrix
			D3DXMatrixIdentity(&world);

			D3DXMatrixTranslation(&world, p->PositionWS.x, p->PositionWS.y, p->PositionWS.z);
			D3DXMatrixTranspose(&world, &world);

			//RendererState.TransformState.TransformWorld = world * view;
			//DrawTriangle();

			D3DXMATRIX mat = view * world;
			
			// Undo all rotations from view
			for(int x=0;x<3;x++)
			{
				for(int y=0;y<3;y++) 
				{
					if ( x==y )
						mat(x,y) = 1.0;
					else
						mat(x,y) = 0.0;
				}
			}

			//mat *= scale;

			//D3DXMatrixTranspose(&mat, &mat);

			ParticleInstanceInfo ii;
			ii.worldview = mat;
			ii.scale = D3DXVECTOR2(p->Size.x, p->Size.y);

			float4 color;
			color.x = p->Color.x / 255.0f;
			color.y = p->Color.y / 255.0f;
			color.z = p->Color.z / 255.0f;
			color.w = p->Alpha / 255.0f;
			ii.color = color.ToDWORD();
			FrameParticles[texture].push_back(ii);

			if(FrameParticles[texture].size() > 1024)
				break;

			//fx->UpdateParticle(p);

			i++;
		}

	}
	// Create new particles?
	//fx->CreateParticlesUpdateDependencies();

	// Do something I dont exactly know what it does :)
	//fx->GetStaticPFXList()->TouchPfx(fx);
}

/** Debugging */
void GothicAPI::DrawTriangle()
{
	BaseVertexBuffer* vxb;
	Engine::GraphicsEngine->CreateVertexBuffer(&vxb);
	vxb->Init(NULL, 6 * sizeof(ExVertexStruct), BaseVertexBuffer::EBindFlags::B_VERTEXBUFFER, BaseVertexBuffer::EUsageFlags::U_DYNAMIC, BaseVertexBuffer::CA_WRITE);
	
	ExVertexStruct vx[6];
	ZeroMemory(vx, sizeof(vx));

	float scale = 50.0f;
	vx[0].Position = float3(0.0f, 0.5f * scale, 0.0f);
	vx[1].Position = float3(0.45f * scale, -0.5f * scale, 0.0f);
	vx[2].Position = float3(-0.45f * scale, -0.5f * scale, 0.0f);

	vx[0].Color = float4(1,0,0,1).ToDWORD();
	vx[1].Color = float4(0,1,0,1).ToDWORD();
	vx[2].Color = float4(0,0,1,1).ToDWORD();

	vx[3].Position = float3(0.0f, 0.5f * scale, 0.0f);
	vx[5].Position = float3(0.45f * scale, -0.5f * scale, 0.0f);
	vx[4].Position = float3(-0.45f * scale, -0.5f * scale, 0.0f);

	vx[3].Color = float4(1,0,0,1).ToDWORD();
	vx[5].Color = float4(0,1,0,1).ToDWORD();
	vx[4].Color = float4(0,0,1,1).ToDWORD();

	vxb->UpdateBuffer(vx);

	Engine::GraphicsEngine->DrawVertexBuffer(vxb, 6);

	delete vxb;
}

/** Debugging */
void GothicAPI::DrawTestInstances()
{
	BaseVertexBuffer* vxb;
	Engine::GraphicsEngine->CreateVertexBuffer(&vxb);
	vxb->Init(NULL, 6 * sizeof(ExVertexStruct), BaseVertexBuffer::EBindFlags::B_VERTEXBUFFER, BaseVertexBuffer::EUsageFlags::U_DYNAMIC, BaseVertexBuffer::CA_WRITE);

	BaseVertexBuffer* ib;
	Engine::GraphicsEngine->CreateVertexBuffer(&ib);
	ib->Init(NULL, 6 * sizeof(VERTEX_INDEX), BaseVertexBuffer::EBindFlags::B_INDEXBUFFER, BaseVertexBuffer::EUsageFlags::U_DYNAMIC, BaseVertexBuffer::CA_WRITE);
	
	ExVertexStruct vx[6];
	ZeroMemory(vx, sizeof(vx));

	float scale = 1.0f;
	vx[0].Position = float3(-scale * 0.5f, -scale * 0.5f, 0.0f);
	vx[1].Position = float3(scale * 0.5f, -scale * 0.5f, 0.0f);
	vx[2].Position = float3(-scale * 0.5f, scale * 0.5f, 0.0f);

	vx[0].TexCoord = float2(0,0);
	vx[1].TexCoord = float2(1,0);
	vx[2].TexCoord = float2(0,1);

	vx[0].Color = 0xFFFFFFFF;
	vx[1].Color = 0xFFFFFFFF;
	vx[2].Color = 0xFFFFFFFF;

	vx[3].Position = float3(scale * 0.5f, -scale * 0.5f, 0.0f);
	vx[4].Position = float3(scale * 0.5f, scale * 0.5f, 0.0f);
	vx[5].Position = float3(-scale * 0.5f, scale * 0.5f, 0.0f);

	vx[3].TexCoord = float2(1,0);
	vx[4].TexCoord = float2(1,1);
	vx[5].TexCoord = float2(0,1);

	vx[3].Color = 0xFFFFFFFF;
	vx[4].Color = 0xFFFFFFFF;
	vx[5].Color = 0xFFFFFFFF;

	vxb->UpdateBuffer(vx);

	VERTEX_INDEX indices[] = {0,1,2,3,4,5};
	ib->UpdateBuffer(indices, sizeof(indices));

	GothicBlendStateInfo bsi = RendererState.BlendState;

	RendererState.BlendState.SetAdditiveBlending();
	RendererState.BlendStateDirty = true;

	RendererState.DepthState.DepthWriteEnabled = false;
	RendererState.DepthStateDirty = true;
	RendererState.DepthState.DepthBufferCompareFunc = GothicDepthBufferStateInfo::CF_COMPARISON_LESS_EQUAL;

	for(std::map<zCTexture*, std::vector<ParticleInstanceInfo>>::iterator it = FrameParticles.begin(); it != FrameParticles.end();it++)
	{
		if((*it).first)
		{
			// Bind it
			if((*it).first->CacheIn(0.6f) == zRES_CACHED_IN)
				(*it).first->Bind(0);
		}

		// Setup blend state
		RendererState.BlendState = FrameParticleInfo[(*it).first].BlendState;
		RendererState.BlendStateDirty = true;

		Engine::GraphicsEngine->DrawInstanced(vxb, ib, 6, &(*it).second[0], sizeof(ParticleInstanceInfo), (*it).second.size());
	}

	RendererState.BlendState = bsi;
	RendererState.BlendStateDirty = true;

	RendererState.DepthState.DepthWriteEnabled = true;
	RendererState.DepthStateDirty = true;

	delete vxb;
	delete ib;
}

/** Sets the world matrix */
void GothicAPI::SetWorldTransform(const D3DXMATRIX& world)
{
	RendererState.TransformState.TransformWorld = world;
}
/** Sets the world matrix */
void GothicAPI::SetViewTransform(const D3DXMATRIX& view)
{
	RendererState.TransformState.TransformView = view;
}

/** Sets the world matrix */
void GothicAPI::SetWorldTransform(const D3DXMATRIX& world, bool transpose)
{
	if(transpose)
		D3DXMatrixTranspose(&RendererState.TransformState.TransformWorld, &world);
	else
		RendererState.TransformState.TransformWorld = world;
}

/** Sets the world matrix */
void GothicAPI::SetViewTransform(const D3DXMATRIX& view, bool transpose)
{
	if(transpose)
		D3DXMatrixTranspose(&RendererState.TransformState.TransformView, &view);
	else
		RendererState.TransformState.TransformView = view;
}

/** Sets the world matrix */
void GothicAPI::SetWorldViewTransform(const D3DXMATRIX& world, const D3DXMATRIX& view)
{
	RendererState.TransformState.TransformWorld = world;
	RendererState.TransformState.TransformView = view;
}

/** Sets the world matrix */
void GothicAPI::ResetWorldTransform()
{
	D3DXMatrixIdentity(&RendererState.TransformState.TransformWorld);
	D3DXMatrixTranspose(&RendererState.TransformState.TransformWorld,&RendererState.TransformState.TransformWorld);
}

/** Sets the world matrix */
void GothicAPI::ResetViewTransform()
{
	D3DXMatrixIdentity(&RendererState.TransformState.TransformView);
	D3DXMatrixTranspose(&RendererState.TransformState.TransformView,&RendererState.TransformState.TransformView);
}

/** Returns the loaded sections */
std::map<int, std::map<int, WorldMeshSectionInfo>>& GothicAPI::GetWorldSections()
{
	return WorldSections;
}

static bool TraceWorldMeshBoxCmp(const std::pair<WorldMeshSectionInfo*, float>& a, const std::pair<WorldMeshSectionInfo*, float>& b)
{
	return a.second > b.second;
}

/** Traces the worldmesh and returns the hit-location */
bool GothicAPI::TraceWorldMesh(const D3DXVECTOR3& origin, const D3DXVECTOR3& dir, D3DXVECTOR3& hit, std::string* hitTextureName, D3DXVECTOR3* hitTriangle, MeshInfo** hitMesh, zCMaterial** hitMaterial)
{
	const int maxSections = 2;
	float closest = FLT_MAX;
	std::list<std::pair<WorldMeshSectionInfo*, float>> hitSections;

	// Trace bounding-boxes first
	for(std::map<int, std::map<int, WorldMeshSectionInfo>>::iterator itx = WorldSections.begin(); itx != WorldSections.end(); itx++)
	{
		for(std::map<int, WorldMeshSectionInfo>::iterator ity = (*itx).second.begin(); ity != (*itx).second.end(); ity++)
		{
			WorldMeshSectionInfo& section = (*ity).second;

			float t;
			if(Toolbox::PositionInsideBox(origin, section.BoundingBox.Min, section.BoundingBox.Max) || Toolbox::IntersectBox(section.BoundingBox.Min, section.BoundingBox.Max, origin, dir, t))
			{
				if(t < maxSections * WORLD_SECTION_SIZE)
					hitSections.push_back(std::make_pair(&section, t));		
			}
		}
	}
	// Distance-sort
	hitSections.sort(TraceWorldMeshBoxCmp);

	int numProcessed = 0;
	for(std::list<std::pair<WorldMeshSectionInfo*, float>>::iterator bit = hitSections.begin(); bit != hitSections.end(); bit++)
	{
		for(std::map<MeshKey, MeshInfo*>::iterator it = (*bit).first->WorldMeshes.begin(); it != (*bit).first->WorldMeshes.end();it++)
		{
			float u,v,t;

			for(unsigned int i=0;i<(*it).second->Indices.size();i+=3)
			{
				if(Toolbox::IntersectTri(*(*it).second->Vertices[(*it).second->Indices[i]].Position.toD3DXVECTOR3(), 
					*(*it).second->Vertices[(*it).second->Indices[i+1]].Position.toD3DXVECTOR3(), 
					*(*it).second->Vertices[(*it).second->Indices[i+2]].Position.toD3DXVECTOR3(),
					origin, dir, u,v,t))
				{
					if(t > 0 && t < closest)
					{
						closest = t;

						if(hitTriangle)
						{
							hitTriangle[0] = *(*it).second->Vertices[(*it).second->Indices[i]].Position.toD3DXVECTOR3();
							hitTriangle[1] = *(*it).second->Vertices[(*it).second->Indices[i+1]].Position.toD3DXVECTOR3();
							hitTriangle[2] = *(*it).second->Vertices[(*it).second->Indices[i+2]].Position.toD3DXVECTOR3();	 
						}

						if(hitMesh)
						{
							*hitMesh = (*it).second;
						}

						if(hitMaterial)
						{
							*hitMaterial = (*it).first.Material;
						}

						if(hitTextureName && (*it).first.Material->GetTexture())
							*hitTextureName = (*it).first.Material->GetTexture()->GetNameWithoutExt();
					}
				}
			}

			numProcessed++;

			if(numProcessed == maxSections)
				break;
		}
	}
	if(closest == FLT_MAX)
		return false;

	hit = origin + dir * closest;

	return true;
}

/** Unprojects a pixel-position on the screen */
void GothicAPI::Unproject(const D3DXVECTOR3& p, D3DXVECTOR3* worldPos, D3DXVECTOR3* worldDir)
{
	D3DXVECTOR3 u;
	D3DXMATRIX proj = GetProjectionMatrix();
	D3DXMATRIX invView; GetInverseViewMatrix(&invView);
	D3DXMatrixTranspose(&invView, &invView);
	D3DXMatrixTranspose(&proj, &proj);

	// Convert to screenspace
	u.x = (((2 * p.x) / Engine::GraphicsEngine->GetResolution().x) - 1) / proj(0,0);
	u.y = -(((2 * p.y) / Engine::GraphicsEngine->GetResolution().y) - 1) / proj(1,1);
	u.z = 1;

	// Transform and output
	D3DXVec3TransformCoord(worldPos, &u, &invView);
	D3DXVec3Normalize(&u, &u);
	D3DXVec3TransformNormal(worldDir, &u, &invView);
}

/** Unprojects the current cursor */
D3DXVECTOR3 GothicAPI::UnprojectCursor()
{
	D3DXVECTOR3 mPos, mDir;
	POINT p; GetCursorPos(&p);
	ScreenToClient(OutputWindow, &p);
	Engine::GAPI->Unproject(D3DXVECTOR3((float)p.x, (float)p.y, 0), &mPos, &mDir);

	return mDir;
}

/** Returns the current cameraposition */
D3DXVECTOR3 GothicAPI::GetCameraPosition()
{
	if(CameraReplacementPtr)
		return CameraReplacementPtr->PositionReplacement;

	return oCGame::GetGame()->_zCSession_camVob->GetPositionWorld();
}

/** Returns the view matrix */
void GothicAPI::GetViewMatrix(D3DXMATRIX* view)
{
	if(CameraReplacementPtr)
	{
		*view = CameraReplacementPtr->ViewReplacement;
		return;
	}

	//D3DXMatrixTranspose(view, oCGame::GetGame()->_zCSession_camVob->GetWorldMatrixPtr());
	D3DXMatrixInverse(view, NULL, oCGame::GetGame()->_zCSession_camVob->GetWorldMatrixPtr());	
}


/** Returns the view matrix */
void GothicAPI::GetInverseViewMatrix(D3DXMATRIX* invView)
{
	if(CameraReplacementPtr)
	{
		D3DXMatrixInverse(invView, NULL, &CameraReplacementPtr->ViewReplacement);	
		return;
	}

	*invView = *(oCGame::GetGame()->_zCSession_camVob->GetWorldMatrixPtr());
	//D3DXMatrixTranspose(invView, invView);
}

/** Returns the projection-matrix */
D3DXMATRIX& GothicAPI::GetProjectionMatrix()
{
	if(CameraReplacementPtr)
	{
		return CameraReplacementPtr->ProjectionReplacement;
	}

	return RendererState.TransformState.TransformProj;
}

/** Returns the GSky-Object */
GSky* GothicAPI::GetSky()
{
	return SkyRenderer;
}

/** Returns the inventory */
GInventory* GothicAPI::GetInventory()
{
	return Inventory;
}

/** Draws the inventory */
void GothicAPI::DrawInventory(zCWorld* world, zCCamera& camera)
{
	Inventory->DrawInventory(world, camera);
}

LRESULT CALLBACK GothicAPI::GothicWndProc(
	HWND hWnd,
	UINT msg,
		WPARAM wParam,
		LPARAM lParam
		)
{
	return Engine::GAPI->OnWindowMessage(hWnd, msg, wParam, lParam);
}

/** Message-Callback for the main window */
LRESULT GothicAPI::OnWindowMessage(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	case WM_KEYDOWN:
		Engine::GraphicsEngine->OnKeyDown(wParam);
		switch(wParam)
		{
		case VK_F11:
			Engine::AntTweakBar->SetActive(!Engine::AntTweakBar->GetActive());
			break;

		case VK_NUMPAD1:
			SpawnVegetationBoxAt(GetCameraPosition());
			break;
		}
		break;

	}

	// This is only processed when the bar is activated, so just call this here
	Engine::AntTweakBar->OnWindowMessage(hWnd, msg, wParam, lParam);
	Engine::GraphicsEngine->OnWindowMessage(hWnd, msg, wParam, lParam);

	if(OriginalGothicWndProc)
		return (*(WNDPROC)(OriginalGothicWndProc))(hWnd, msg, wParam, lParam);
	else
		return 0;
}

/** Recursive helper function to draw the BSP-Tree */
void GothicAPI::DebugDrawTreeNode(zCBspBase* base, zTBBox3D boxCell, int clipFlags)
{
	while(base)
	{
		if (clipFlags>0) 
		{
			float yMaxWorld = Engine::GAPI->GetLoadedWorldInfo()->BspTree->GetRootNode()->BBox3D.Max.y;

			zTBBox3D nodeBox = base->BBox3D;
			float nodeYMax = std::min(yMaxWorld, Engine::GAPI->GetCameraPosition().y);
			nodeYMax = std::max(nodeYMax, base->BBox3D.Max.y);
			nodeBox.Max.y = nodeYMax;

			zTCam_ClipType nodeClip = zCCamera::GetCamera()->BBox3DInFrustum(nodeBox, clipFlags); 

			if (nodeClip==ZTCAM_CLIPTYPE_OUT) 
				return; // Nothig to see here. Discard this node and the subtree
		}



		if(base->IsLeaf())
		{
			// Check if this leaf is inside the frustum
			if(clipFlags > 0) 
			{
				if(zCCamera::GetCamera()->BBox3DInFrustum(base->BBox3D, clipFlags) == ZTCAM_CLIPTYPE_OUT) 
					return;
			}

			Engine::GraphicsEngine->GetLineRenderer()->AddAABBMinMax(base->BBox3D.Min, base->BBox3D.Max);
			return;
		}else
		{
			zCBspNode* node = (zCBspNode *)base;

			int	planeAxis = node->PlaneSignbits;

			boxCell.Min.y	= node->BBox3D.Min.y;
			boxCell.Max.y	= node->BBox3D.Min.y;

			zTBBox3D tmpbox = boxCell;
			if (D3DXVec3Dot(&node->Plane.Normal, &GetCameraPosition()) > node->Plane.Distance)
			{ 
				if(node->Front) 
				{
					((float *)&tmpbox.Min)[planeAxis] = node->Plane.Distance;
					DebugDrawTreeNode(node->Front, tmpbox, clipFlags);
				}

				((float *)&boxCell.Max)[planeAxis] = node->Plane.Distance;
				base = node->Back;

			} else 
			{
				if (node->Back ) 
				{
					((float *)&tmpbox.Max)[planeAxis] = node->Plane.Distance;
					DebugDrawTreeNode(node->Back, tmpbox, clipFlags);
				}

				((float *)&boxCell.Min)[planeAxis] = node->Plane.Distance;
				base = node->Front;
			}
		}
	}
}

/** Draws the AABB for the BSP-Tree using the line renderer*/
void GothicAPI::DebugDrawBSPTree()
{
	zCBspTree* tree = LoadedWorldInfo->BspTree;

	zCBspBase* root = tree->GetRootNode();

	// Recursively go through the tree and draw all nodes
	DebugDrawTreeNode(root, root->BBox3D);
}

/** Collects vobs using gothics BSP-Tree */
void GothicAPI::CollectVisibleVobs(std::list<VobInfo *>& vobs, std::list<VobLightInfo *>& lights)
{
	zCBspTree* tree = LoadedWorldInfo->BspTree;

	zCBspBase* root = tree->GetRootNode();
	if(zCCamera::GetCamera())
		zCCamera::GetCamera()->Activate();

	// Recursively go through the tree and draw all nodes
	CollectVisibleVobsHelper(root, root->BBox3D, 63, vobs, lights);
}

/** Collects visible sections from the current camera perspective */
void GothicAPI::CollectVisibleSections(std::list<WorldMeshSectionInfo*>& sections)
{
	D3DXVECTOR3 camPos = Engine::GAPI->GetCameraPosition();
	INT2 camSection = WorldConverter::GetSectionOfPos(camPos);

	// run through every section and check for range and frustum
	int sectionViewDist = Engine::GAPI->GetRendererState()->RendererSettings.SectionDrawRadius;
	for(std::map<int, std::map<int, WorldMeshSectionInfo>>::iterator itx = WorldSections.begin(); itx != WorldSections.end(); itx++)
	{
		for(std::map<int, WorldMeshSectionInfo>::iterator ity = (*itx).second.begin(); ity != (*itx).second.end(); ity++)
		{
			WorldMeshSectionInfo& section = (*ity).second;

			// Simple range-check
			if(abs((*itx).first - camSection.x) < sectionViewDist && 
				abs((*ity).first - camSection.y) < sectionViewDist)
			{
				int flags = 15; // Frustum check, no farplane
				if(zCCamera::GetCamera()->BBox3DInFrustum(section.BoundingBox, flags) == ZTCAM_CLIPTYPE_OUT)
					continue;

				sections.push_back(&section);
			}
		}
	}
}

static void CVVH_AddNotDrawnVobToList(std::list<VobInfo *>& target, std::list<VobInfo *>& source)
{
	for(std::list<VobInfo *>::iterator it = source.begin(); it != source.end(); it++)
	{
		if(!(*it)->VisibleInRenderPass)
		{
			target.push_back((*it));
			(*it)->VisibleInRenderPass = true;
		}
	}
}

static void CVVH_AddNotDrawnVobToList(std::list<VobLightInfo *>& target, std::list<VobLightInfo *>& source)
{
	for(std::list<VobLightInfo *>::iterator it = source.begin(); it != source.end(); it++)
	{
		if(!(*it)->VisibleInRenderPass)
		{
			target.push_back((*it));
			(*it)->VisibleInRenderPass = true;
		}
	}
}

/** Recursive helper function to draw collect the vobs */
void GothicAPI::CollectVisibleVobsHelper(zCBspBase* base, zTBBox3D boxCell, int clipFlags, std::list<VobInfo *>& vobs, std::list<VobLightInfo *>& lights)
{
	float vobIndoorDist = Engine::GAPI->GetRendererState()->RendererSettings.IndoorVobDrawRadius;
	float vobOutdoorDist = Engine::GAPI->GetRendererState()->RendererSettings.OutdoorVobDrawRadius;
	float vobOutdoorSmallDist = Engine::GAPI->GetRendererState()->RendererSettings.OutdoorSmallVobDrawRadius;
	float vobSmallSize = Engine::GAPI->GetRendererState()->RendererSettings.SmallVobSize;
	D3DXVECTOR3 camPos = Engine::GAPI->GetCameraPosition();

	while(base)
	{
		if (clipFlags>0) 
		{
			float yMaxWorld = Engine::GAPI->GetLoadedWorldInfo()->BspTree->GetRootNode()->BBox3D.Max.y;

			zTBBox3D nodeBox = base->BBox3D;
			float nodeYMax = std::min(yMaxWorld, Engine::GAPI->GetCameraPosition().y);
			nodeYMax = std::max(nodeYMax, base->BBox3D.Max.y);
			nodeBox.Max.y = nodeYMax;

			zTCam_ClipType nodeClip = zCCamera::GetCamera()->BBox3DInFrustum(nodeBox, clipFlags); 

			//float dist = D3DXVec3Length(&(base->BBox3D.Min - camPos));

			if (nodeClip==ZTCAM_CLIPTYPE_OUT) 
				return; // Nothig to see here. Discard this node and the subtree
		}



		if(base->IsLeaf())
		{
			// Check if this leaf is inside the frustum
			if(clipFlags > 0) 
			{
				if(zCCamera::GetCamera()->BBox3DInFrustum(base->BBox3D, clipFlags) == ZTCAM_CLIPTYPE_OUT) 
					return;
			}
			zCBspLeaf* leaf = (zCBspLeaf *)base;
			BspVobInfo& bvi = BspLeafVobLists[leaf];
			std::list<VobInfo *>& listA = bvi.IndoorVobs;
			std::list<VobInfo *>& listB = bvi.SmallVobs;
			std::list<VobInfo *>& listC = bvi.Vobs;
			std::list<VobLightInfo *>& listL = bvi.Lights;
			std::list<VobLightInfo *>& listL_Indoor = bvi.IndoorLights;

			// Concat the lists
			float dist = D3DXVec3Length(&(base->BBox3D.Min - camPos));

			if(Engine::GAPI->GetRendererState()->RendererSettings.DrawVOBs)
			{
				if(dist < vobIndoorDist)
					CVVH_AddNotDrawnVobToList(vobs, listA);

				if(dist < vobOutdoorSmallDist)
					CVVH_AddNotDrawnVobToList(vobs, listB);
			}

			if(dist < vobOutdoorDist)
			{
				if(Engine::GAPI->GetRendererState()->RendererSettings.DrawVOBs)
				{
					CVVH_AddNotDrawnVobToList(vobs, listC);
				}

				if(RendererState.RendererSettings.EnableDynamicLighting)
				{
					CVVH_AddNotDrawnVobToList(lights, listL);

					if(dist < vobIndoorDist)
						CVVH_AddNotDrawnVobToList(lights, listL_Indoor);

					// Add dynamic lights
					for(int i=bvi.NumStaticLights;i<leaf->LightVobList.NumInArray;i++)
					{
						zCVobLight* v = leaf->LightVobList.Array[i];
						VobLightInfo* vi;

						// Check if we already have this light
						if(VobLightMap.find(v) == VobLightMap.end())
						{
							// Add if not
							vi = new VobLightInfo;
							vi->Vob = leaf->LightVobList.Array[i];
							VobLightMap[leaf->LightVobList.Array[i]] = vi;
						}else
						{
							vi = VobLightMap[leaf->LightVobList.Array[i]];
						}

						if(!vi->VisibleInRenderPass)
						{
							vi->VisibleInRenderPass = true;

							// Render it
							lights.push_back(vi);
						}
					}
				}
			}
			//Engine::GraphicsEngine->GetLineRenderer()->AddAABBMinMax(base->BBox3D.Min, base->BBox3D.Max);
			return;
		}else
		{
			zCBspNode* node = (zCBspNode *)base;

			int	planeAxis = node->PlaneSignbits;

			boxCell.Min.y	= node->BBox3D.Min.y;
			boxCell.Max.y	= node->BBox3D.Min.y;

			zTBBox3D tmpbox = boxCell;
			if (D3DXVec3Dot(&node->Plane.Normal, &GetCameraPosition()) > node->Plane.Distance)
			{ 
				if(node->Front) 
				{
					((float *)&tmpbox.Min)[planeAxis] = node->Plane.Distance;
					CollectVisibleVobsHelper(node->Front, tmpbox, clipFlags, vobs, lights);
				}

				((float *)&boxCell.Max)[planeAxis] = node->Plane.Distance;
				base = node->Back;

			} else 
			{
				if (node->Back ) 
				{
					((float *)&tmpbox.Max)[planeAxis] = node->Plane.Distance;
					CollectVisibleVobsHelper(node->Back, tmpbox, clipFlags, vobs, lights);
				}

				((float *)&boxCell.Min)[planeAxis] = node->Plane.Distance;
				base = node->Front;
			}
		}
	}
}

/** Helper function for going through the bsp-tree */
void GothicAPI::BuildBspVobMapCacheHelper(zCBspBase* base)
{
	if(!base)
		return;


	if(base->IsLeaf())
	{
		zCBspLeaf* leaf = (zCBspLeaf *)base;

		// Put it into the cache
		BspVobInfo& bvi = BspLeafVobLists[base];

		for(int i=0;i<leaf->LeafVobList.NumInArray;i++)
		{
			// Get the vob info for this one
			if(VobMap.find(leaf->LeafVobList.Array[i]) != VobMap.end())
			{
				VobInfo* v = VobMap[leaf->LeafVobList.Array[i]];

				if(v)
				{				
					float vobSmallSize = Engine::GAPI->GetRendererState()->RendererSettings.SmallVobSize;

					if(v->Vob->GetGroundPoly() && v->Vob->GetGroundPoly()->GetLightmap())
					{
						bvi.IndoorVobs.push_back(v);	
						v->IsIndoorVob = true;
					}
					else if(v->VisualInfo->MeshSize < vobSmallSize)
						bvi.SmallVobs.push_back(v);
					else
						bvi.Vobs.push_back(v);
				}
			}

			for(int i=0;i<leaf->LightVobList.NumInArray;i++)
			{
				// Add the light to the map if not already done
				std::hash_map<zCVobLight*, VobLightInfo*>::iterator vit = VobLightMap.find(leaf->LightVobList.Array[i]);

				if(vit == VobLightMap.end())
				{
					VobLightInfo* vi = new VobLightInfo;
					vi->Vob = leaf->LightVobList.Array[i];
					VobLightMap[leaf->LightVobList.Array[i]] = vi;

					if(((zCVob *)vi->Vob)->GetGroundPoly() && ((zCVob *)vi->Vob)->GetGroundPoly()->GetLightmap())
					{
						vi->IsIndoorVob = true;
					}
				}


				VobLightInfo* vi = VobLightMap[leaf->LightVobList.Array[i]];
				if(!vi->IsIndoorVob)
					bvi.Lights.push_back(vi);
				else
					bvi.IndoorLights.push_back(vi);

				//zCVob* vob = (zCVob *) leaf->LightVobList.Array[i];
				//Engine::GraphicsEngine->GetLineRenderer()->AddAABB(vob->GetPositionWorld(), D3DXVECTOR3(50,50,50));
			}

			bvi.NumStaticLights = leaf->LightVobList.NumInArray;
		}
	}else
	{
		zCBspNode* node = (zCBspNode *)base;
		BuildBspVobMapCacheHelper(node->Front);
		BuildBspVobMapCacheHelper(node->Back);
	}
}

/** Builds our BspTreeVobMap */
void GothicAPI::BuildBspVobMapCache()
{
	BuildBspVobMapCacheHelper(LoadedWorldInfo->BspTree->GetRootNode());
}

/** Disables a problematic method which causes the game to conflict with other applications on startup */
void GothicAPI::DisableErrorMessageBroadcast()
{
	LogInfo() << "Disabling global message broadcast";

	// Unlock the memory region
	DWORD dwProtect;
	unsigned int size = GothicMemoryLocations::zERROR::BroadcastEnd - GothicMemoryLocations::zERROR::BroadcastStart;
	VirtualProtect((void *)GothicMemoryLocations::zERROR::BroadcastStart, size, PAGE_EXECUTE_READWRITE, &dwProtect);

	// NOP it
	REPLACE_RANGE(GothicMemoryLocations::zERROR::BroadcastStart, GothicMemoryLocations::zERROR::BroadcastEnd-1, INST_NOP);
}

/** Sets/Gets the far-plane */
void GothicAPI::SetFarPlane(float value)
{
	zCCamera::GetCamera()->SetFarPlane(value);
}

float GothicAPI::GetFarPlane()
{
	return zCCamera::GetCamera()->GetFarPlane();
}

/** Sets/Gets the far-plane */
void GothicAPI::SetNearPlane(float value)
{
	LogWarn() << "SetNearPlane not implemented yet!";
	
}

float GothicAPI::GetNearPlane()
{
	return zCCamera::GetCamera()->GetNearPlane();
}

/** Get material by texture name */
zCMaterial* GothicAPI::GetMaterialByTextureName(const std::string& name)
{
	for(std::set<zCMaterial *>::iterator it = LoadedMaterials.begin(); it != LoadedMaterials.end(); it++)
	{
		if((*it)->GetTexture())
		{
			std::string tn = (*it)->GetTexture()->GetNameWithoutExt();
			if(_strnicmp(name.c_str(), tn.c_str(), 255)==0)
				return (*it);
		}
	}

	return NULL;
}

void GothicAPI::GetMaterialListByTextureName(const std::string& name, std::list<zCMaterial*>& list)
{
	for(std::set<zCMaterial *>::iterator it = LoadedMaterials.begin(); it != LoadedMaterials.end(); it++)
	{
		if((*it)->GetTexture())
		{
			std::string tn = (*it)->GetTexture()->GetNameWithoutExt();
			if(_strnicmp(name.c_str(), tn.c_str(), 255)==0)
				list.push_back((*it));
		}
	}
}

/** Returns the time since the last frame */
float GothicAPI::GetDeltaTime()
{
	zCTimer* timer = zCTimer::GetTimer();

	return timer->frameTimeFloatSecs;
}

/** Sets the current texture test bind mode status */
void GothicAPI::SetTextureTestBindMode(bool enable, const std::string& currentTexture)
{
	TextureTestBindMode = enable;

	if(enable)
		BoundTestTexture = currentTexture;
}

/** If this returns true, the property holds the name of the currently bound texture. If that is the case, any MyDirectDrawSurfaces should not bind themselfes
to the pipeline, but rather check if there are additional textures to load */
bool GothicAPI::IsInTextureTestBindMode(std::string& currentBoundTexture)
{
	if(TextureTestBindMode)
		currentBoundTexture = BoundTestTexture;

	return TextureTestBindMode;
}

/** Lets Gothic draw its sky */
void GothicAPI::DrawSkyGothicOriginal()
{
	HookedFunctions::OriginalFunctions.original_zCWorldRender(oCGame::GetGame()->_zCSession_world, *zCCamera::GetCamera());
}

/** Returns the material info associated with the given material */
MaterialInfo* GothicAPI::GetMaterialInfoFrom(zCTexture* tex)
{
	std::hash_map<zCTexture*, MaterialInfo>::iterator f = MaterialInfos.find(tex);
	if(f == MaterialInfos.end() && tex)
	{
		// Make a new one and try to load it
		MaterialInfos[tex].LoadFromFile(tex->GetNameWithoutExt());
	}

	return &MaterialInfos[tex];
}

/** Adds a surface */
void GothicAPI::AddSurface(const std::string& name, MyDirectDrawSurface7* surface)
{
	SurfacesByName[name] = surface;
}

/** Removes a surface */
void GothicAPI::RemoveSurface(MyDirectDrawSurface7* surface)
{
	SurfacesByName.erase(surface->GetTextureName());
}

/** Returns the loaded skeletal mesh vobs */
std::list<SkeletalVobInfo *>& GothicAPI::GetSkeletalMeshVobs()
{
	return SkeletalMeshVobs;
}

/** Resets all vob-stats drawn this frame */
void GothicAPI::ResetVobFrameStats(std::list<VobInfo *>& vobs)
{
	for(std::list<VobInfo *>::iterator it = vobs.begin(); it != vobs.end();it++)
	{
		(*it)->VisibleInRenderPass = false;
	}
}

/** Sets the currently bound texture */
void GothicAPI::SetBoundTexture(int idx, zCTexture* tex)
{
	BoundTextures[idx] = tex;
}

zCTexture* GothicAPI::GetBoundTexture(int idx)
{
	return BoundTextures[idx];
}