#include "pch.h"
#include "SV_Panel.h"
#include "D2DView.h"

SV_Panel::SV_Panel(D2DView* view, D2DSubView* parent) : D2DSubView(view, parent)
{
	PanelColor = D2D1::ColorF(1,1,1,1);
	HasShadow = true;
	RenderMode = PR_Background;
	ShadowRange = 20.0f;
	Image = NULL;
}


SV_Panel::~SV_Panel(void)
{
	if(Image)Image->Release();
}

/** Draws this sub-view */
void SV_Panel::Draw(const D2D1_RECT_F& clientRectAbs, float deltaTime)
{
	//Set up the layer for this control
	MainView->GetRenderTarget()->SetTransform(D2D1::Matrix3x2F::Translation(clientRectAbs.left,clientRectAbs.top));



	if(HasShadow)
		MainView->DrawSmoothShadow(&ViewRect, ShadowRange, 0.8f, false, 7);

	if(RenderMode == PR_SolidColor)
	{
		MainView->GetBrush()->SetColor(PanelColor);
		MainView->GetRenderTarget()->FillRectangle(ViewRect, MainView->GetBrush());
	}else if(RenderMode == PR_Background)
	{
		float darkness = 1.0f - powf(0.7f, (float)Level);

		MainView->GetBackgroundBrush()->SetStartPoint(D2D1::Point2F(-clientRectAbs.left,-clientRectAbs.top));
		MainView->GetBackgroundBrush()->SetEndPoint(D2D1::Point2F(clientRectAbs.right-clientRectAbs.left,clientRectAbs.bottom-clientRectAbs.top));

		MainView->GetRenderTarget()->FillRectangle(D2D1::RectF(ViewRect.left,ViewRect.top,ViewRect.right,ViewRect.bottom),MainView->GetBackgroundBrush());

		MainView->GetBrush()->SetColor(D2D1::ColorF(0,0,0, darkness));
		MainView->GetRenderTarget()->FillRectangle(ViewRect, MainView->GetBrush());
	}else if(RenderMode == PR_Image)
	{

	}

	D2DSubView::Draw(clientRectAbs, deltaTime);
}

/** Sets the border color */
void SV_Panel::SetPanelColor(const D2D1_COLOR_F& color)
{
	PanelColor = color;
}

/** Set if this panel should have a shadow */
void SV_Panel::SetPanelShadow(bool shadow, float range)
{
	HasShadow = shadow;
	ShadowRange = range;
}

/** Set if this panel should render as a solid color */
void SV_Panel::SetRenderMode(EPanelRenderMode mode)
{
	RenderMode = mode;
}

/** Returns the panels solid color */
D2D1_COLOR_F SV_Panel::GetPanelColor()
{
	return PanelColor;
}


/** Sets the image of this panel from d3d11 */
HRESULT SV_Panel::SetD3D11TextureAsImage(D3D11Texture* texture)
{
	if(Image)Image->Release();
	Image = NULL;
}