#include "pch.h"
#include "SV_Slider.h"
#include "D2DSubView.h"
#include "D2DView.h"

const float SV_SLIDERCONTROL_SLIDER_SIZEX = 5;

SV_Slider::SV_Slider(D2DView* view, D2DSubView* parent) : D2DSubView(view, parent)
{
	BarPosition = 0.0f;
	Min = 0.0f;
	Max = 1.0f;
	Value = 0.0f;
	DraggingSlider = false;

	DataToUpdate = NULL;
}


SV_Slider::~SV_Slider(void)
{
}

/** Processes a window-message. Return false to stop the message from going to children */
bool SV_Slider::OnWindowMessage(HWND hWnd, unsigned int msg, WPARAM wParam, LPARAM lParam, const D2D1_RECT_F& clientRectAbs)
{
	switch(msg)
	{
	case WM_MOUSEMOVE:
		if(DraggingSlider)
		{
			// Calculate valueP from the x-axis
			POINT p;
			GetCursorPos(&p);
			ScreenToClient(hWnd, &p);

			float vP = (p.x - clientRectAbs.left) / (clientRectAbs.right - clientRectAbs.left);
			SetValueP(vP);
		}
		break;

	case WM_LBUTTONDOWN:
		{
			POINT p;
			GetCursorPos(&p);
			ScreenToClient(hWnd, &p);
			if(PointInsideRect(D2D1::Point2F((float)p.x, (float)p.y), clientRectAbs))
			{
				DraggingSlider = true;
				return false;
			}
		}
		break;

	case WM_LBUTTONUP:
		DraggingSlider = false;
		break;
	}

	return D2DSubView::OnWindowMessage(hWnd, msg, wParam, lParam, clientRectAbs);
}

/** Draws this sub-view */
void SV_Slider::Draw(const D2D1_RECT_F& clientRectAbs, float deltaTime)
{
	// Draw a shadow around the whole control
	MainView->DrawSmoothShadow(&ViewRect, 10, 0.5f, false, 7);

	// Draw the background box
	//float Level=(float)GetLevel()*SF_DEF_LEVEL_COLOR_INC+SF_DEF_LEVEL_COLOR_BASE;
	MainView->GetBrush()->SetColor(D2D1::ColorF(0.2f,0.2f,0.2f,1));
	MainView->GetRenderTarget()->FillRectangle(&ViewRect, MainView->GetBrush());
	MainView->DrawSmoothShadow(&ViewRect, -5/2, 0.5f, 0.0f);


	// Draw the outer lines
	MainView->GetRenderTarget()->DrawRectangle(&ViewRect, MainView->GetLinearReflectBrush());

	D2D1_RECT_F sc = ViewRect;
	D2DView::ShrinkRect(&sc, 1);
	MainView->GetBrush()->SetColor( SV_DEF_INNER_LINE_COLOR );
	MainView->GetRenderTarget()->DrawRectangle(&sc, MainView->GetBrush());
	
	sc = ViewRect;

	// Draw slider
	//SetLinearGradientToControl(D2DObjects->BrushCollection.LinearReflectBrush);
	MainView->GetBrush()->SetColor(D2D1::ColorF(0.2f,0.2f,0.2f,1));
	sc.right = (BarPosition*(sc.right-sc.left)) + sc.left + SV_SLIDERCONTROL_SLIDER_SIZEX;
	sc.left = sc.right - (SV_SLIDERCONTROL_SLIDER_SIZEX*2);
	sc.top+=2;
	sc.bottom-=2;
	MainView->GetRenderTarget()->FillRectangle(&sc, MainView->GetBrush());

	// Draw outer lines of the slider
	MainView->GetRenderTarget()->DrawRectangle(&sc, MainView->GetLinearReflectBrush());
	D2DView::ShrinkRect(&sc, 1);
	MainView->GetBrush()->SetColor( SV_DEF_INNER_LINE_COLOR );
	MainView->GetRenderTarget()->DrawRectangle(&sc, MainView->GetBrush());

	D2DSubView::Draw(clientRectAbs, deltaTime);
}

/** Sets the value of this slider (0..1) */
void SV_Slider::SetValueP(float value)
{
	value = std::min(1.0f, value);
	value = std::max(0.0f, value);

	Value = Toolbox::lerp(Min, Max, value);
	BarPosition = (GetSize().width - ((SV_SLIDERCONTROL_SLIDER_SIZEX+2)*2)) * value;

	if(DataToUpdate)
		*DataToUpdate = Value;
}

/** Sets the value of this slider (min..max) */
void SV_Slider::SetValue(float value)
{
	SetValueP((value - Min) / (Max - Min));
}

/** Sets the min-max values for this slider */
void SV_Slider::SetMinMax(float min, float max)
{
	Min = min;
	Max = max;
}

/** Sets the data location to update with this slider */
void SV_Slider::SetDataToUpdate(float* data)
{
	DataToUpdate = data;
}