//--------------------------------------------------------------------------------------
// Simple vertex shader
//--------------------------------------------------------------------------------------

cbuffer Matrices_PerFrame : register( b0 )
{
	matrix M_View;
	matrix M_Proj;
	matrix M_ViewProj;	
};


//--------------------------------------------------------------------------------------
// Input / Output structures
//--------------------------------------------------------------------------------------
struct VS_INPUT
{
	float3 vPosition	: POSITION;
	float3 vNormal		: NORMAL;
	float2 vTex1		: TEXCOORD0;
	float2 vTex2		: TEXCOORD1;
	float4 vDiffuse		: DIFFUSE;
	float4x4 InstanceWorldViewMatrix : INSTANCE_WORLD_MATRIX;
	float4 vInstanceColor : INSTANCE_COLOR;
	float2 vInstanceScale : INSTANCE_SCALE;
	
};

struct VS_OUTPUT
{
	float2 vTexcoord		: TEXCOORD0;
	float2 vTexcoord2		: TEXCOORD1;
	float4 vDiffuse			: TEXCOORD2;
	float3 vViewPosition 	: TEXCOORD3;
	float3 vNormalVS		: TEXCOORD4;
	float3 vWorldPosition	: TEXCOORD5;
	float4 vPosition		: SV_POSITION;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
//--------------------------------------------------------------------------------------
VS_OUTPUT VSMain( VS_INPUT Input )
{
	VS_OUTPUT Output;
	
	//Output.vPosition = float4(Input.vPosition, 1);
	Output.vPosition = mul( float4(Input.vPosition * float3(Input.vInstanceScale, 1),1), mul(Input.InstanceWorldViewMatrix, M_Proj) );
	Output.vTexcoord2 = Input.vTex1;
	Output.vTexcoord = Input.vTex1;
	Output.vDiffuse  = float4(Input.vInstanceColor.gba, Input.vInstanceColor.r);
	Output.vNormalVS = float3(0,0,0);//mul(Input.vNormal, (float3x3)mul(Input.InstanceWorldViewMatrix, M_View));
	Output.vViewPosition = float3(0,0,0);//mul(float4(Input.vPosition,1), mul(Input.InstanceWorldViewMatrix, M_View));
	Output.vWorldPosition = mul(float4(Input.vPosition,1), Input.InstanceWorldViewMatrix).rgb;
	
	return Output;
}

