#pragma once
#include "pch.h"

/** Misc. tools */

namespace Toolbox
{
	/** Returns true if the given position is inside the box */
	bool PositionInsideBox(const D3DXVECTOR3& p, const D3DXVECTOR3& min, const D3DXVECTOR3& max);

	/** Converts an errorcode into a string */
	std::string MakeErrorString(XRESULT code);

	/** Returns the number of bits inside a bitmask */
	WORD GetNumberOfBits( DWORD dwMask );

	/** Returns the size of a DDS-Image in bytes */
	unsigned int GetDDSStorageRequirements(unsigned int width, unsigned int height, bool dxt1);

	/** Returns the RowPitch-Size of a DDS-Image */
	unsigned int GetDDSRowPitchSize(unsigned int width, bool dxt1);

	/** Returns a random number between 0 and 1 */
	float frand();

	/** Linear interpolation */
	float lerp(float a, float b, float w);

	/** Converts a multi-byte-string to wide-char */
	std::wstring ToWideChar(const std::string& str);
	
	/** Does a ray vs aabb test */
	bool IntersectBox(const D3DXVECTOR3& min, const D3DXVECTOR3& max, const D3DXVECTOR3& origin, const D3DXVECTOR3& direction, float& t);

	/** Does a ray vs aabb test */
	bool IntersectTri(const D3DXVECTOR3& v0, const D3DXVECTOR3& v1, const D3DXVECTOR3& v2, const D3DXVECTOR3& origin, const D3DXVECTOR3& direction, float& u, float& v, float& t);

	/** Computes the normal of a triangle */
	D3DXVECTOR3 ComputeNormal(const D3DXVECTOR3& v0, const D3DXVECTOR3& v1, const D3DXVECTOR3& v2);

	/** Computes the distance of a point to an AABB */
	float ComputePointAABBDistance(const D3DXVECTOR3& p, const D3DXVECTOR3& min, const D3DXVECTOR3& max);
};