#include "pch.h"
#include "WorldConverter.h"
#include "Engine.h"
#include "BaseGraphicsEngine.h"
#include "BaseVertexBuffer.h"
#include "zCPolygon.h"
#include "zCMaterial.h"
#include "zCTexture.h"
#include "zCVisual.h"
#include "zCVob.h"
#include "zCProgMeshProto.h"
#include "zCMeshSoftSkin.h"
#include "zCModel.h"
#include "zCMorphMesh.h"
#include <set>
#include "ConstantBufferStructs.h"
#include "BaseConstantBuffer.h"
#include "zCMesh.h"
#include "zCLightmap.h"
#include "GMesh.h"

WorldConverter::WorldConverter(void)
{
}


WorldConverter::~WorldConverter(void)
{
}

/** Updates the vobs constantbuffer */
void VobInfo::UpdateVobConstantBuffer()
{
	VS_ExConstantBuffer_PerInstance cb;
	cb.World = *Vob->GetWorldMatrixPtr();
	VobConstantBuffer->UpdateBuffer(&cb);
}

SectionInstanceCache::~SectionInstanceCache()
{
	for(std::map<MeshVisualInfo *, BaseVertexBuffer*>::iterator it = InstanceCache.begin(); it != InstanceCache.end(); it++)
	{
		delete (*it).second;
	}
}

MeshInfo::~MeshInfo()
{
	Engine::GAPI->GetRendererState()->RendererInfo.VOBVerticesDataSize -= Indices.size() * sizeof(VERTEX_INDEX);
	Engine::GAPI->GetRendererState()->RendererInfo.VOBVerticesDataSize -= Vertices.size() * sizeof(ExVertexStruct);

	delete MeshVertexBuffer;
	delete MeshIndexBuffer;
}

SkeletalMeshInfo::~SkeletalMeshInfo()
{
	Engine::GAPI->GetRendererState()->RendererInfo.SkeletalVerticesDataSize -= Indices.size() * sizeof(VERTEX_INDEX);
	Engine::GAPI->GetRendererState()->RendererInfo.SkeletalVerticesDataSize -= Vertices.size() * sizeof(ExSkelVertexStruct);

	delete MeshVertexBuffer;
	delete MeshIndexBuffer;
}

/** Clears the cache for the given progmesh */
void SectionInstanceCache::ClearCacheForStatic(MeshVisualInfo* pm)
{
	if(InstanceCache.find(pm) != InstanceCache.end())
	{
		BaseVertexBuffer* vb = InstanceCache[pm];
		delete vb;
		InstanceCache[pm] = NULL;
		InstanceCacheData[pm].clear();
	}
}

/** Saves this sections mesh to a file */
void WorldMeshSectionInfo::SaveSectionMeshToFile(const std::string& name)
{
	FILE* f;
	fopen_s(&f, name.c_str(), "wb");

	if(!f)
		return;


}

/** Creates buffers for this mesh info */
XRESULT MeshInfo::Create(ExVertexStruct* vertices, unsigned int numVertices, VERTEX_INDEX* indices, unsigned int numIndices)
{
	Vertices.resize(numVertices * sizeof(ExVertexStruct));
	memcpy(&Vertices[0], vertices, numVertices * sizeof(ExVertexStruct));

	Indices.resize(numIndices * sizeof(VERTEX_INDEX));
	memcpy(&Indices[0], indices, numIndices * sizeof(VERTEX_INDEX));

	// Create the buffers
	Engine::GraphicsEngine->CreateVertexBuffer(&MeshVertexBuffer);
	Engine::GraphicsEngine->CreateVertexBuffer(&MeshIndexBuffer);

	// Init and fill it
	MeshVertexBuffer->Init(vertices, numVertices * sizeof(ExVertexStruct));
	MeshIndexBuffer->Init(indices, numIndices * sizeof(VERTEX_INDEX), BaseVertexBuffer::B_INDEXBUFFER);

	Engine::GAPI->GetRendererState()->RendererInfo.VOBVerticesDataSize += numVertices * sizeof(ExVertexStruct);
	Engine::GAPI->GetRendererState()->RendererInfo.VOBVerticesDataSize += numIndices * sizeof(VERTEX_INDEX);

	return XR_SUCCESS;
}
/** Converts a loaded custommesh to be the worldmesh */
XRESULT WorldConverter::LoadWorldMeshFromFile(const std::string& file, std::map<int, std::map<int, WorldMeshSectionInfo>>* outSections, WorldInfo* info)
{
	GMesh* mesh = new GMesh();

	// Load the meshfile
	XLE(mesh->LoadMesh(file));

	std::vector<MeshInfo *>& meshes = mesh->GetMeshes();
	std::vector<std::string>& textures = mesh->GetTextures();

	// run through meshes and pack them into sections
	for(unsigned int m = 0;m<meshes.size();m++)
	{
		zCMaterial* mat = Engine::GAPI->GetMaterialByTextureName(textures[m]);
		MeshKey key;
		key.Material = mat;
		//key.Lightmap = poly->GetLightmap();

		for(unsigned int i=0;i<meshes[m]->Indices.size();i+=3)
		{

			if(meshes[m]->Indices[i] > meshes[m]->Vertices.size() || 
				meshes[m]->Indices[i+1] > meshes[m]->Vertices.size() || 
				meshes[m]->Indices[i+2] > meshes[m]->Vertices.size())
				break; // Catch broken meshes
			
			ExVertexStruct* v[3] = {   &meshes[m]->Vertices[meshes[m]->Indices[i]],
										&meshes[m]->Vertices[meshes[m]->Indices[i+1]],
										&meshes[m]->Vertices[meshes[m]->Indices[i+2]]};

			// Use the section of the first point for the whole polygon
			INT2 sxy = GetSectionOfPos(*v[0]->Position.toD3DXVECTOR3());

			WorldMeshSectionInfo& section = (*outSections)[sxy.x][sxy.y];

			D3DXVECTOR3& bbmin = section.BoundingBox.Min;
			D3DXVECTOR3& bbmax = section.BoundingBox.Max;

			// Check bounding box
			bbmin.x = bbmin.x > v[0]->Position.x ? v[0]->Position.x : bbmin.x;
			bbmin.y = bbmin.y > v[0]->Position.y ? v[0]->Position.y : bbmin.y;
			bbmin.z = bbmin.z > v[0]->Position.z ? v[0]->Position.z : bbmin.z;

			bbmax.x = bbmax.x < v[0]->Position.x ? v[0]->Position.x : bbmax.x;
			bbmax.y = bbmax.y < v[0]->Position.y ? v[0]->Position.y : bbmax.y;
			bbmax.z = bbmax.z < v[0]->Position.z ? v[0]->Position.z : bbmax.z;

			if(section.WorldMeshes.find(key) == section.WorldMeshes.end())
				section.WorldMeshes[key] = new MeshInfo;

			for(int i=0;i<3;i++)
				section.WorldMeshes[key]->Vertices.push_back(*v[i]);
		}
	}

	// Dont need that anymore
	delete mesh;

	D3DXVECTOR2 avgSections = D3DXVECTOR2(0,0);
	int numSections = 0;

	// Create the vertexbuffers for every material
	for(std::map<int, std::map<int, WorldMeshSectionInfo>>::iterator itx = (*outSections).begin(); itx != (*outSections).end(); itx++)
	{
		for(std::map<int, WorldMeshSectionInfo>::iterator ity = (*itx).second.begin(); ity != (*itx).second.end(); ity++)
		{
			WorldMeshSectionInfo& section = (*ity).second;
			numSections++;
			avgSections += D3DXVECTOR2((float)(*itx).first, (float)(*ity).first);

			for(std::map<MeshKey, MeshInfo*>::iterator it = section.WorldMeshes.begin(); it != section.WorldMeshes.end();it++)
			{
				std::vector<ExVertexStruct> indexedVertices;
				std::vector<VERTEX_INDEX> indices;
				IndexVertices(&(*it).second->Vertices[0], (*it).second->Vertices.size(), indexedVertices, indices);

				(*it).second->Vertices = indexedVertices;
				(*it).second->Indices = indices;

				// Create the buffers
				Engine::GraphicsEngine->CreateVertexBuffer(&(*it).second->MeshVertexBuffer);
				Engine::GraphicsEngine->CreateVertexBuffer(&(*it).second->MeshIndexBuffer);

				// Optimize faces
				(*it).second->MeshVertexBuffer->OptimizeFaces(&(*it).second->Indices[0],
					(byte *)&(*it).second->Vertices[0], 
					(*it).second->Indices.size(), 
					(*it).second->Vertices.size(), 
					sizeof(ExVertexStruct));

				// Then optimize vertices
				(*it).second->MeshVertexBuffer->OptimizeVertices(&(*it).second->Indices[0],
					(byte *)&(*it).second->Vertices[0], 
					(*it).second->Indices.size(), 
					(*it).second->Vertices.size(), 
					sizeof(ExVertexStruct));

				// Init and fill them
				(*it).second->MeshVertexBuffer->Init(&(*it).second->Vertices[0], (*it).second->Vertices.size() * sizeof(ExVertexStruct), BaseVertexBuffer::B_VERTEXBUFFER, BaseVertexBuffer::U_IMMUTABLE);
				(*it).second->MeshIndexBuffer->Init(&(*it).second->Indices[0], (*it).second->Indices.size() * sizeof(VERTEX_INDEX), BaseVertexBuffer::B_INDEXBUFFER, BaseVertexBuffer::U_IMMUTABLE);


			}
		}
	}

	// Calculate the approx midpoint of the world
	avgSections /= (float)numSections;

	if(info)
	{
		WorldInfo i;
		i.MidPoint = avgSections * WORLD_SECTION_SIZE;
		i.LowestVertex = 0;
		i.HighestVertex = 0;

		memcpy(info, &i, sizeof(WorldInfo));
	}


	return XR_SUCCESS;
}

/** Converts the worldmesh into a more usable format */
HRESULT WorldConverter::ConvertWorldMesh(zCPolygon** polys, unsigned int numPolygons, std::map<int, std::map<int, WorldMeshSectionInfo>>* outSections, WorldInfo* info)
{
	// Go through every polygon and put it into it's section
	for(unsigned int i=0;i<numPolygons;i++)
	{
		zCPolygon* poly = polys[i];

		// Check if we even need this polygon
		if(poly->GetPolyFlags()->GhostOccluder || 
			poly->GetPolyFlags()->PortalPoly)
			continue;

		//if(polygons[i]->GetNumPolyVertices() != 3)
		//	continue;

		// Use the section of the first point for the whole polygon
		INT2 section = GetSectionOfPos(*poly->getVertices()[0]->Position.toD3DXVECTOR3());

		D3DXVECTOR3& bbmin = (*outSections)[section.x][section.y].BoundingBox.Min;
		D3DXVECTOR3& bbmax = (*outSections)[section.x][section.y].BoundingBox.Max;

		// Check bounding box
		bbmin.x = bbmin.x > poly->getVertices()[0]->Position.x ? poly->getVertices()[0]->Position.x : bbmin.x;
		bbmin.y = bbmin.y > poly->getVertices()[0]->Position.y ? poly->getVertices()[0]->Position.y : bbmin.y;
		bbmin.z = bbmin.z > poly->getVertices()[0]->Position.z ? poly->getVertices()[0]->Position.z : bbmin.z;

		bbmax.x = bbmax.x < poly->getVertices()[0]->Position.x ? poly->getVertices()[0]->Position.x : bbmax.x;
		bbmax.y = bbmax.y < poly->getVertices()[0]->Position.y ? poly->getVertices()[0]->Position.y : bbmax.y;
		bbmax.z = bbmax.z < poly->getVertices()[0]->Position.z ? poly->getVertices()[0]->Position.z : bbmax.z;

		DWORD sectionColor = float4((section.x % 2) + 0.5f, (section.x % 2) + 0.5f, 1, 1).ToDWORD();

		if(poly->GetNumPolyVertices() < 3)
		{
			LogWarn() << "Poly with less than 3 vertices!";
		}

		// Extract poly vertices
		std::vector<ExVertexStruct> polyVertices;
		for(int v=0;v<poly->GetNumPolyVertices();v++)
		{
			ExVertexStruct t;
			t.Position = poly->getVertices()[v]->Position;
			t.TexCoord = poly->getFeatures()[v]->texCoord;
			t.Normal = poly->getFeatures()[v]->normal;
			t.Color = poly->getFeatures()[v]->lightStatic;

			if(poly->GetLightmap())
			{
				t.TexCoord2 = poly->GetLightmap()->GetLightmapUV(*t.Position.toD3DXVECTOR3());
				t.Color = DEFAULT_LIGHTMAP_POLY_COLOR;
			}else
			{
				t.Color = 0xFFFFFFFF;
				t.TexCoord2.x = 1.0f;
				/*if((poly->GetMaterial() && poly->GetMaterial()->GetMatGroup() == zMAT_GROUP_WATER) ||
					(poly->GetMaterial() && (poly->GetMaterial()->GetAlphaFunc() != zMAT_ALPHA_FUNC_FUNC_MAT_DEFAULT ||
											 poly->GetMaterial()->GetAlphaFunc() != zMAT_ALPHA_FUNC_FUNC_NONE)))
					t.Color = 0xFFFFFFFF; // Get rid of ugly shadows on water*/
			}

			polyVertices.push_back(t);
		}		

		// Use the map to put the polygon to those using the same material

		zCMaterial* mat = poly->GetMaterial();
		MeshKey key;
		key.Material = mat;
		//key.Lightmap = poly->GetLightmap();

		if((*outSections)[section.x][section.y].WorldMeshes.count(key) == 0)
			(*outSections)[section.x][section.y].WorldMeshes[key] = new MeshInfo;

		std::vector<ExVertexStruct> finalVertices;
		TriangleFanToList(&polyVertices[0], polyVertices.size(), &finalVertices);

		if(finalVertices.size() == 3)
		{
			if(mat && mat->GetTexture())
			{
				/*if(strnicmp(mat->GetTexture()->GetNameWithoutExt().c_str(), "NW_Harbour_Stairs", strlen("NW_Harbour_Stairs")) == 0)
				{
					ExVertexStruct vx[3];
					memcpy(vx, &finalVertices[0], sizeof(ExVertexStruct) * 3);
					finalVertices.clear();

					TesselateTriangle(vx, finalVertices, 5);
				}*/
			}
		}

		//if(mat && mat->GetTexture())
		//	LogInfo() << "Got texture name: " << mat->GetTexture()->GetName();

		for(unsigned int v=0;v<finalVertices.size();v++)
			(*outSections)[section.x][section.y].WorldMeshes[key]->Vertices.push_back(finalVertices[v]);
	}
	
	D3DXVECTOR2 avgSections = D3DXVECTOR2(0,0);
	int numSections = 0;

	// Create the vertexbuffers for every material
	for(std::map<int, std::map<int, WorldMeshSectionInfo>>::iterator itx = (*outSections).begin(); itx != (*outSections).end(); itx++)
	{
		for(std::map<int, WorldMeshSectionInfo>::iterator ity = (*itx).second.begin(); ity != (*itx).second.end(); ity++)
		{
			WorldMeshSectionInfo& section = (*ity).second;
			numSections++;
			avgSections += D3DXVECTOR2((float)(*itx).first, (float)(*ity).first);

			for(std::map<MeshKey, MeshInfo*>::iterator it = section.WorldMeshes.begin(); it != section.WorldMeshes.end();it++)
			{
				std::vector<ExVertexStruct> indexedVertices;
				std::vector<VERTEX_INDEX> indices;
				IndexVertices(&(*it).second->Vertices[0], (*it).second->Vertices.size(), indexedVertices, indices);

				(*it).second->Vertices = indexedVertices;
				(*it).second->Indices = indices;

				// Create the buffers
				Engine::GraphicsEngine->CreateVertexBuffer(&(*it).second->MeshVertexBuffer);
				Engine::GraphicsEngine->CreateVertexBuffer(&(*it).second->MeshIndexBuffer);

				// Generate normals
				GenerateVertexNormals((*it).second->Vertices, (*it).second->Indices);

				// Optimize faces
				(*it).second->MeshVertexBuffer->OptimizeFaces(&(*it).second->Indices[0],
					(byte *)&(*it).second->Vertices[0], 
					(*it).second->Indices.size(), 
					(*it).second->Vertices.size(), 
					sizeof(ExVertexStruct));

				// Then optimize vertices
				(*it).second->MeshVertexBuffer->OptimizeVertices(&(*it).second->Indices[0],
					(byte *)&(*it).second->Vertices[0], 
					(*it).second->Indices.size(), 
					(*it).second->Vertices.size(), 
					sizeof(ExVertexStruct));

				// Init and fill them
				(*it).second->MeshVertexBuffer->Init(&(*it).second->Vertices[0], (*it).second->Vertices.size() * sizeof(ExVertexStruct), BaseVertexBuffer::B_VERTEXBUFFER, BaseVertexBuffer::U_IMMUTABLE);
				(*it).second->MeshIndexBuffer->Init(&(*it).second->Indices[0], (*it).second->Indices.size() * sizeof(VERTEX_INDEX), BaseVertexBuffer::B_INDEXBUFFER, BaseVertexBuffer::U_IMMUTABLE);


			}
		}
	}

	// Calculate the approx midpoint of the world
	avgSections /= (float)numSections;

	if(info)
	{
		WorldInfo i;
		i.MidPoint = avgSections * WORLD_SECTION_SIZE;
		i.LowestVertex = 0;
		i.HighestVertex = 0;

		memcpy(info, &i, sizeof(WorldInfo));
	}
	//SaveSectionsToObjUnindexed("Test.obj", (*outSections));

	return XR_SUCCESS;
}

/** Creates the FullSectionMesh for the given section */
void WorldConverter::GenerateFullSectionMesh(WorldMeshSectionInfo& section)
{
	std::vector<ExVertexStruct> vx;
	for(std::map<MeshKey, MeshInfo*>::iterator it = section.WorldMeshes.begin(); it != section.WorldMeshes.end();it++)
	{
		if(!(*it).first.Material ||
			(*it).first.Material->HasAlphaTest())
			continue;

		for(unsigned int i=0;i<(*it).second->Indices.size(); i+=3)
		{
			// Push all triangles
			vx.push_back((*it).second->Vertices[(*it).second->Indices[i]]);
			vx.push_back((*it).second->Vertices[(*it).second->Indices[i+1]]);
			vx.push_back((*it).second->Vertices[(*it).second->Indices[i+2]]);
		}
	}

	// Get VOBs
	for(std::list<VobInfo*>::iterator it = section.Vobs.begin(); it != section.Vobs.end(); it++)
	{
		if((*it)->IsIndoorVob)
			continue;

		D3DXMATRIX world;
		(*it)->Vob->GetWorldMatrix(&world);
		D3DXMatrixTranspose(&world, &world);

		// Insert the vob
		for(std::map<zCMaterial *, std::vector<MeshInfo*>>::iterator itm = (*it)->VisualInfo->Meshes.begin(); itm != (*it)->VisualInfo->Meshes.end();itm++)
		{
			if(!(*itm).first ||
				(*itm).first->HasAlphaTest())
				continue;

			for(unsigned int m=0;m<(*itm).second.size();m++)
			{
				for(unsigned int i=0;i<(*itm).second[m]->Indices.size(); i++)
				{
					ExVertexStruct v = (*itm).second[m]->Vertices[(*itm).second[m]->Indices[i]];

					// Transform everything into world space
					D3DXVec3TransformCoord(v.Position.toD3DXVECTOR3(), v.Position.toD3DXVECTOR3(), &world);
					vx.push_back(v);
				}
			}
		}
	}

	// Catch empty section
	if(vx.empty())
		return;

	// Index the mesh
	std::vector<ExVertexStruct> indexedVertices;
	std::vector<VERTEX_INDEX> indices;
	//IndexVertices(&vx[0], vx.size(), indexedVertices, indices);

	/*section.FullStaticMesh = new MeshInfo;
	section.FullStaticMesh->Indices = indices;
	section.FullStaticMesh->Vertices = indexedVertices;*/
	section.FullStaticMesh = new MeshInfo;
	section.FullStaticMesh->Vertices = vx;

	// Create the buffers
	Engine::GraphicsEngine->CreateVertexBuffer(&section.FullStaticMesh->MeshVertexBuffer);
	//Engine::GraphicsEngine->CreateVertexBuffer(&section.FullStaticMesh->MeshIndexBuffer);

	// Optimize faces
	/*section.FullStaticMesh->MeshVertexBuffer->OptimizeFaces(&section.FullStaticMesh->Indices[0],
		(byte *)&section.FullStaticMesh->Vertices[0], 
		section.FullStaticMesh->Indices.size(), 
		section.FullStaticMesh->Vertices.size(), 
		sizeof(ExVertexStruct));

	// Then optimize vertices
	section.FullStaticMesh->MeshVertexBuffer->OptimizeVertices(&section.FullStaticMesh->Indices[0],
		(byte *)&section.FullStaticMesh->Vertices[0], 
		section.FullStaticMesh->Indices.size(), 
		section.FullStaticMesh->Vertices.size(), 
		sizeof(ExVertexStruct));*/

	// Init and fill them
	section.FullStaticMesh->MeshVertexBuffer->Init(&section.FullStaticMesh->Vertices[0], section.FullStaticMesh->Vertices.size() * sizeof(ExVertexStruct), BaseVertexBuffer::B_VERTEXBUFFER, BaseVertexBuffer::U_IMMUTABLE);
	Engine::GAPI->GetRendererState()->RendererInfo.SkeletalVerticesDataSize += section.FullStaticMesh->Vertices.size() * sizeof(ExVertexStruct);
	
	//section.FullStaticMesh->MeshIndexBuffer->Init(&section.FullStaticMesh->Indices[0], section.FullStaticMesh->Indices.size() * sizeof(VERTEX_INDEX), BaseVertexBuffer::B_INDEXBUFFER, BaseVertexBuffer::U_IMMUTABLE);
}

/** Returns what section the given position is in */
INT2 WorldConverter::GetSectionOfPos(const D3DXVECTOR3& pos)
{
	// Find out where it belongs
	int px = (int)((pos.x / WORLD_SECTION_SIZE) + 0.5f);
	int py = (int)((pos.z / WORLD_SECTION_SIZE) + 0.5f);

	// Fix the centerpiece
	/*if(pos.x < 0)
		px-=1;

	if(pos.z < 0)
		py-=1;*/

	return INT2(px, py);
}

/** Converts a triangle fan to a list */
void WorldConverter::TriangleFanToList(ExVertexStruct* input, unsigned int numInputVertices, std::vector<ExVertexStruct>* outVertices)
{
	for(UINT i=1;i<numInputVertices-1;i++)
	{
		outVertices->push_back(input[0]);
		outVertices->push_back(input[i+1]);
		outVertices->push_back(input[i]);
	}
}

/** Saves the given section-array to an obj file */
void WorldConverter::SaveSectionsToObjUnindexed(const char* file, const std::map<int, std::map<int, WorldMeshSectionInfo>>& sections)
{
	FILE* f = fopen(file, "w");

	if(!f)
	{
		LogError() << "Failed to open file " << file << " for writing!";
		return;
	}

	fputs("o World\n", f);

	for(std::map<int, std::map<int, WorldMeshSectionInfo>>::const_iterator itx = sections.begin(); itx != sections.end(); itx++)
	{
		for(std::map<int, WorldMeshSectionInfo>::const_iterator ity = (*itx).second.begin(); ity != (*itx).second.end(); ity++)
		{
			const WorldMeshSectionInfo& section = (*ity).second;

			for(std::map<MeshKey, MeshInfo*>::const_iterator it = section.WorldMeshes.begin(); it != section.WorldMeshes.end();it++)
			{
				for(unsigned int i=0;i<(*it).second->Vertices.size();i++)
				{
					std::string ln = "v " + std::to_string((*it).second->Vertices[i].Position.x) + " " + std::to_string((*it).second->Vertices[i].Position.y) + " " + std::to_string((*it).second->Vertices[i].Position.z) + "\n";
					fputs(ln.c_str(), f);

					//if(i % 3 == 0)
					//	fputs("f -3 -2 -1\n", f);
					//ln = "vn " + std::to_string((*it).second.Vertices[i].Normal.x) + " " + std::to_string((*it).second.Vertices[i].Normal.y) + " " + std::to_string((*it).second.Vertices[i].Normal.z) + "\n";
					//fputs(ln.c_str(), f);
				}
			}
		}
	}

	fclose(f);
}

/** Extracts a 3DS-Mesh from a zCVisual */
void WorldConverter::Extract3DSMeshFromVisual(zCProgMeshProto* visual, MeshVisualInfo* meshInfo)
{
	std::vector<ExVertexStruct> vertices;

	// Get the data out for all submeshes
	vertices.clear();
	visual->ConstructVertexBuffer(&vertices);

	// This visual can hold multiple submeshes, each with it's own indices
	for(int i=0;i<visual->GetNumSubmeshes();i++)
	{
		zCSubMesh* m = visual->GetSubmesh(i);

		// Get the data from the indices
		for(int n=0; n<m->WedgeList.NumInArray;n++)
		{
			int idx = m->WedgeList.Get(n).position;


			vertices[idx].TexCoord.x = m->WedgeList.Get(n).texUV.x; // This produces wrong results
			vertices[idx].TexCoord.y = m->WedgeList.Get(n).texUV.y;
			vertices[idx].Color = 0xFFFFFFFF;
			*vertices[idx].Normal.toD3DXVECTOR3() = (*m->WedgeList.Get(n).normal.toD3DXVECTOR3());
		}

		// Get indices
		std::vector<VERTEX_INDEX> indices;
		for(int n=0;n<visual->GetSubmesh(i)->TriList.NumInArray;n++)
		{
			/*for(int x=0;x<3;x++)
			{
				int idx = visual->GetSubmesh(i)->WedgeList.Get(visual->GetSubmesh(i)->TriList.Get(n).wedge[x]).position;
				vertices[idx].TexCoord.x = m->WedgeList.Get(visual->GetSubmesh(i)->TriList.Get(n).wedge[x]).texUV.x;
				vertices[idx].TexCoord.y = m->WedgeList.Get(visual->GetSubmesh(i)->TriList.Get(n).wedge[x]).texUV.y;

				//*vertices[idx].nrm.toD3DXVECTOR3() = (*m->WedgeList.Get(n).normal.toD3DXVECTOR3());
			}*/

			indices.push_back(visual->GetSubmesh(i)->WedgeList.Get(visual->GetSubmesh(i)->TriList.Get(n).wedge[0]).position);
			indices.push_back(visual->GetSubmesh(i)->WedgeList.Get(visual->GetSubmesh(i)->TriList.Get(n).wedge[1]).position);
			indices.push_back(visual->GetSubmesh(i)->WedgeList.Get(visual->GetSubmesh(i)->TriList.Get(n).wedge[2]).position);
		}
		
		zCMaterial* mat = visual->GetSubmesh(i)->Material;

		MeshInfo* mi = new MeshInfo;

		mi->Vertices = vertices;
		mi->Indices = indices;
		
		// Create the buffers
		Engine::GraphicsEngine->CreateVertexBuffer(&mi->MeshVertexBuffer);
		Engine::GraphicsEngine->CreateVertexBuffer(&mi->MeshIndexBuffer);
	
		// Init and fill it
		mi->MeshVertexBuffer->Init(&vertices[0], vertices.size() * sizeof(ExVertexStruct));
		mi->MeshIndexBuffer->Init(&indices[0], indices.size() * sizeof(VERTEX_INDEX), BaseVertexBuffer::B_INDEXBUFFER);

		meshInfo->Meshes[mat].push_back(mi);
	}

	meshInfo->Visual = visual;
}

/** Extracts a skeletal mesh from a zCMeshSoftSkin */
void WorldConverter::ExtractSkeletalMeshFromVob(zCModel* model, SkeletalMeshVisualInfo* skeletalMeshInfo)
{
	// This type has multiple skinned meshes inside
	for(int i=0;i<model->GetMeshSoftSkinList()->NumInArray;i++)
	{
		zCMeshSoftSkin* s = model->GetMeshSoftSkinList()->Array[i];
		std::vector<ExSkelVertexStruct> posList;

		// This stream is built as the following:
		// 4byte int - Num of nodes
		// sizeof(zTWeightEntry) - The entry
		char* stream = s->GetVertWeightStream();

		// Get bone weights for each vertex
		for(int i=0; i<s->GetPositionList()->NumInArray;i++)
		{
			// Get num of nodes
			int numNodes = *(int *)stream;
			stream += 4;

			ExSkelVertexStruct vx;
			//vx.Position = s->GetPositionList()->Array[i];
			vx.Color = 0xFFFFFFFF;
			vx.Normal = float3(0,0,0);
			ZeroMemory(vx.weights, sizeof(vx.weights));
			for(int x=0;x<4;x++)
			{
				vx.boneIndices[x] = 0;
			}

			for(int n=0;n<numNodes;n++)
			{
				// Get entry
				zTWeightEntry weightEntry = *(zTWeightEntry *)stream;
				stream += sizeof(zTWeightEntry);

				//if(s->GetNormalsList() && i < s->GetNormalsList()->NumInArray)
				//	(*vx.Normal.toD3DXVECTOR3()) += weightEntry.Weight * (*s->GetNormalsList()->Array[i].toD3DXVECTOR3());

				// Get index and weight
				if(n < 4)
				{
					vx.weights[n] = weightEntry.Weight;
					vx.boneIndices[n] = weightEntry.NodeIndex;
					vx.Position[n] = weightEntry.VertexPosition;
				}
			}

			posList.push_back(vx);
		}

		

		// The rest is the same as a zCProgMeshProto, but with a different vertex type
		for(int i=0;i<s->GetNumSubmeshes();i++)
		{
			std::vector<ExSkelVertexStruct> vertices;

			// Get the data out
			zCSubMesh* m = s->GetSubmesh(i);

			// Get vertices
			for(int t=0;t<m->TriList.NumInArray;t++)
			{
				for(int v=0;v<3;v++)
				{
					ExSkelVertexStruct vx;
					vx = posList[m->WedgeList.Array[m->TriList.Array[t].wedge[v]].position];
					vx.TexCoord	= m->WedgeList.Array[m->TriList.Array[t].wedge[v]].texUV;
					vx.Color = 0xFFFFFFFF;
					vx.Normal = m->WedgeList.Array[m->TriList.Array[t].wedge[v]].normal;

					vertices.push_back(vx);
				}
			}

			// Create the indexed mesh
			std::vector<ExSkelVertexStruct> indexedVertices;
			std::vector<VERTEX_INDEX> indices;
			WorldConverter::IndexVertices(&vertices[0], vertices.size(), indexedVertices, indices);

			/*for(int n=0; n<m->WedgeList.NumInArray;n++)
			{
				int idx = m->WedgeList.Get(n).position;


				vertices[idx].TexCoord.x = m->WedgeList.Get(n).texUV.x;
				vertices[idx].TexCoord.y = m->WedgeList.Get(n).texUV.y;
				vertices[idx].Color = 0xFFFFFFFF;
				*vertices[idx].Normal.toD3DXVECTOR3() = (*m->WedgeList.Get(n).normal.toD3DXVECTOR3());
			}


			std::vector<VERTEX_INDEX> indices;
			for(int n=0;n<s->GetSubmesh(i)->TriList.NumInArray;n++)
			{
				for(int x=0;x<3;x++)
				{
					int idx = s->GetSubmesh(i)->WedgeList.Get(s->GetSubmesh(i)->TriList.Get(n).wedge[x]).position;
					vertices[idx].TexCoord.x = m->WedgeList.Get(s->GetSubmesh(i)->TriList.Get(n).wedge[x]).texUV.x;
					vertices[idx].TexCoord.y = m->WedgeList.Get(s->GetSubmesh(i)->TriList.Get(n).wedge[x]).texUV.y;

					//*vertices[idx].nrm.toD3DXVECTOR3() = (*m->WedgeList.Get(n).normal.toD3DXVECTOR3());
				}

				indices.push_back(s->GetSubmesh(i)->WedgeList.Get(s->GetSubmesh(i)->TriList.Get(n).wedge[0]).position);
				indices.push_back(s->GetSubmesh(i)->WedgeList.Get(s->GetSubmesh(i)->TriList.Get(n).wedge[1]).position);
				indices.push_back(s->GetSubmesh(i)->WedgeList.Get(s->GetSubmesh(i)->TriList.Get(n).wedge[2]).position);
			}*/
		
			zCMaterial* mat = s->GetSubmesh(i)->Material;

			SkeletalMeshInfo* mi = new SkeletalMeshInfo;
			mi->Vertices = indexedVertices;
			mi->Indices = indices;

			// Create the buffers
			Engine::GraphicsEngine->CreateVertexBuffer(&mi->MeshVertexBuffer);
			Engine::GraphicsEngine->CreateVertexBuffer(&mi->MeshIndexBuffer);
	
			// Optimize faces
			mi->MeshVertexBuffer->OptimizeFaces(&mi->Indices[0],
				(byte *)&mi->Vertices[0], 
				mi->Indices.size(), 
				mi->Vertices.size(), 
				sizeof(ExSkelVertexStruct));

			// Then optimize vertices
			mi->MeshVertexBuffer->OptimizeVertices(&mi->Indices[0],
				(byte *)&mi->Vertices[0], 
				mi->Indices.size(), 
				mi->Vertices.size(), 
				sizeof(ExSkelVertexStruct));

			// Init and fill it
			mi->MeshVertexBuffer->Init(&indexedVertices[0], indexedVertices.size() * sizeof(ExSkelVertexStruct), BaseVertexBuffer::B_VERTEXBUFFER, BaseVertexBuffer::U_IMMUTABLE);
			mi->MeshIndexBuffer->Init(&indices[0], indices.size() * sizeof(VERTEX_INDEX), BaseVertexBuffer::B_INDEXBUFFER, BaseVertexBuffer::U_IMMUTABLE);

			Engine::GAPI->GetRendererState()->RendererInfo.SkeletalVerticesDataSize += indexedVertices.size() * sizeof(ExVertexStruct);
			Engine::GAPI->GetRendererState()->RendererInfo.SkeletalVerticesDataSize += indices.size() * sizeof(VERTEX_INDEX);

			skeletalMeshInfo->Meshes[mat].push_back(mi);
		}
	}

	// Extract all visuals we know at this time
	for(int i=0;i<model->GetNodeList()->NumInArray;i++)
	{
		ExtractNodeVisual(i, model->GetNodeList()->Array[i], skeletalMeshInfo);
	}
}

/** Extracts a node-visual */
void WorldConverter::ExtractNodeVisual(int index, zCModelNodeInst* node, SkeletalMeshVisualInfo* skeletalMeshInfo)
{
	// Extract node visuals
	if(node->NodeVisual)
	{
		const char* ext = node->NodeVisual->GetFileExtension(0);

		if(strcmp(ext, ".3DS") == 0)
		{
			zCProgMeshProto* pm = (zCProgMeshProto *)node->NodeVisual;

			MeshVisualInfo* mi = new MeshVisualInfo;

			Extract3DSMeshFromVisual2(pm, mi);

			skeletalMeshInfo->NodeAttachments[index].push_back(mi);
		}else if(strcmp(ext, ".MMS") == 0)
		{
			// These are zCMorphMeshes
			zCProgMeshProto* pm = ((zCMorphMesh *)node->NodeVisual)->GetMorphMesh();

			MeshVisualInfo* mi = new MeshVisualInfo;

			Extract3DSMeshFromVisual2(pm, mi);

			skeletalMeshInfo->NodeAttachments[index].push_back(mi);
		}
	}
}

/** Extracts a 3DS-Mesh from a zCVisual */
void WorldConverter::Extract3DSMeshFromVisual2(zCProgMeshProto* visual, MeshVisualInfo* meshInfo)
{
	D3DXVECTOR3 tri0, tri1, tri2;
	D3DXVECTOR2	uv0, uv1, uv2;
	D3DXVECTOR3 bbmin, bbmax;
	bbmin = D3DXVECTOR3(FLT_MAX, FLT_MAX, FLT_MAX);
	bbmax = D3DXVECTOR3(FLT_MIN, FLT_MIN, FLT_MIN);

	D3DXVECTOR3* posList = (D3DXVECTOR3 *)visual->GetPositionList()->Array;

	// Construct unindexed mesh
	for(int i=0; i < visual->GetNumSubmeshes(); i++)
	{
		std::vector<ExVertexStruct> vertices;

		// Get vertices
		for(int t=0;t<visual->GetSubmeshes()[i].TriList.NumInArray;t++)
		{				
			for(int v=0;v<3;v++)
			{
				ExVertexStruct vx;
				vx.Position = posList[visual->GetSubmeshes()[i].WedgeList.Array[visual->GetSubmeshes()[i].TriList.Array[t].wedge[v]].position];
				vx.TexCoord	= visual->GetSubmeshes()[i].WedgeList.Array[visual->GetSubmeshes()[i].TriList.Array[t].wedge[v]].texUV;
				vx.Color = 0xFFFFFFFF;
				vx.Normal = visual->GetSubmeshes()[i].WedgeList.Array[visual->GetSubmeshes()[i].TriList.Array[t].wedge[v]].normal;

				vertices.push_back(vx);

				// Check bounding box
				bbmin.x = bbmin.x > vx.Position.x ? vx.Position.x : bbmin.x;
				bbmin.y = bbmin.y > vx.Position.y ? vx.Position.y : bbmin.y;
				bbmin.z = bbmin.z > vx.Position.z ? vx.Position.z : bbmin.z;

				bbmax.x = bbmax.x < vx.Position.x ? vx.Position.x : bbmax.x;
				bbmax.y = bbmax.y < vx.Position.y ? vx.Position.y : bbmax.y;
				bbmax.z = bbmax.z < vx.Position.z ? vx.Position.z : bbmax.z;
			}	
		}

		// Create the buffers and sort the mesh into the structure
		MeshInfo* mi = new MeshInfo;

		// Create the indexed mesh
		WorldConverter::IndexVertices(&vertices[0], vertices.size(), mi->Vertices, mi->Indices);

		// Catch empty mesh
		if(mi->Vertices.empty())
			return;

		// Create the buffers
		Engine::GraphicsEngine->CreateVertexBuffer(&mi->MeshVertexBuffer);
		Engine::GraphicsEngine->CreateVertexBuffer(&mi->MeshIndexBuffer);
	
		// Optimize faces
		/*mi->MeshVertexBuffer->OptimizeFaces(&mi->Indices[0],
			(byte *)&mi->Vertices[0], 
			mi->Indices.size(), 
			mi->Vertices.size(), 
			sizeof(ExVertexStruct));

		// Then optimize vertices
		mi->MeshVertexBuffer->OptimizeVertices(&mi->Indices[0],
			(byte *)&mi->Vertices[0], 
			mi->Indices.size(), 
			mi->Vertices.size(), 
			sizeof(ExVertexStruct));*/

		// Init and fill it
		mi->MeshVertexBuffer->Init(&mi->Vertices[0], mi->Vertices.size() * sizeof(ExVertexStruct), BaseVertexBuffer::B_VERTEXBUFFER, BaseVertexBuffer::U_IMMUTABLE);
		mi->MeshIndexBuffer->Init(&mi->Indices[0], mi->Indices.size() * sizeof(VERTEX_INDEX), BaseVertexBuffer::B_INDEXBUFFER, BaseVertexBuffer::U_IMMUTABLE);

		Engine::GAPI->GetRendererState()->RendererInfo.VOBVerticesDataSize += mi->Vertices.size() * sizeof(ExVertexStruct);
		Engine::GAPI->GetRendererState()->RendererInfo.VOBVerticesDataSize += mi->Indices.size() * sizeof(VERTEX_INDEX);

		zCMaterial* mat = visual->GetSubmesh(i)->Material;
		meshInfo->Meshes[mat].push_back(mi);
	}

	meshInfo->BBox.Min = bbmin;
	meshInfo->BBox.Max = bbmax;
	meshInfo->MeshSize = D3DXVec3Length(&(bbmin - bbmax));

	meshInfo->Visual = visual;
}


const float eps = 0.001f;


struct CmpClass // class comparing vertices in the set
{
    bool operator() (const std::pair<ExVertexStruct, int>& p1, const std::pair<ExVertexStruct, int>& p2) const
    {
        if (fabs(p1.first.Position.x-p2.first.Position.x) > eps) return p1.first.Position.x < p2.first.Position.x;
        if (fabs(p1.first.Position.y-p2.first.Position.y) > eps) return p1.first.Position.y < p2.first.Position.y;
        if (fabs(p1.first.Position.z-p2.first.Position.z) > eps) return p1.first.Position.z < p2.first.Position.z;

		if (fabs(p1.first.TexCoord.x-p2.first.TexCoord.x) > eps) return p1.first.TexCoord.x < p2.first.TexCoord.x;
		if (fabs(p1.first.TexCoord.y-p2.first.TexCoord.y) > eps) return p1.first.TexCoord.y < p2.first.TexCoord.y;

        return false;
    }
};



/** Indexes the given vertex array */
void WorldConverter::IndexVertices(ExVertexStruct* input, unsigned int numInputVertices, std::vector<ExVertexStruct>& outVertices, std::vector<VERTEX_INDEX>& outIndices)
{
	std::set<std::pair<ExVertexStruct, int>, CmpClass> vertices;
	int index = 0;

    for (unsigned int i=0; i<numInputVertices; i++)
    {
        std::set<std::pair<ExVertexStruct, int>>::iterator it = vertices.find(std::make_pair(input[i], 0/*this value doesn't matter*/));
        if (it!=vertices.end()) outIndices.push_back(it->second);
        else
        {
			/*char* c = (char *)&input[i].color;
			c[0] = rand() % 255;
			c[1] = rand() % 255;
			c[2] = rand() % 255;*/

            vertices.insert(std::make_pair(input[i], index));
            outIndices.push_back(index++);
        }
    }

    // Notice that the vertices in the set are not sorted by the index
    // so you'll have to rearrange them like this:
    outVertices.resize(vertices.size());
    for (std::set<std::pair<ExVertexStruct, int>>::iterator it=vertices.begin(); it!=vertices.end(); it++)
        outVertices[it->second] = it->first;
}

struct CmpClassSkel // class comparing vertices in the set
{
    bool operator() (const std::pair<ExSkelVertexStruct, int>& p1, const std::pair<ExSkelVertexStruct, int>& p2) const
    {
		for(int i=0;i<4;i++)
		{
			if (fabs(p1.first.Position[i].x-p2.first.Position[i].x) > eps) return p1.first.Position[i].x < p2.first.Position[i].x;
			if (fabs(p1.first.Position[i].y-p2.first.Position[i].y) > eps) return p1.first.Position[i].y < p2.first.Position[i].y;
			if (fabs(p1.first.Position[i].z-p2.first.Position[i].z) > eps) return p1.first.Position[i].z < p2.first.Position[i].z;
		}

		if (fabs(p1.first.TexCoord.x-p2.first.TexCoord.x) > eps) return p1.first.TexCoord.x < p2.first.TexCoord.x;
		if (fabs(p1.first.TexCoord.y-p2.first.TexCoord.y) > eps) return p1.first.TexCoord.y < p2.first.TexCoord.y;

        return false;
    }
};

void WorldConverter::IndexVertices(ExSkelVertexStruct* input, unsigned int numInputVertices, std::vector<ExSkelVertexStruct>& outVertices, std::vector<VERTEX_INDEX>& outIndices)
{
	std::set<std::pair<ExSkelVertexStruct, int>, CmpClassSkel> vertices;
	int index = 0;

    for (unsigned int i=0; i<numInputVertices; i++)
    {
        std::set<std::pair<ExSkelVertexStruct, int>>::iterator it = vertices.find(std::make_pair(input[i], 0/*this value doesn't matter*/));
        if (it!=vertices.end()) outIndices.push_back(it->second);
        else
        {
			/*char* c = (char *)&input[i].color;
			c[0] = rand() % 255;
			c[1] = rand() % 255;
			c[2] = rand() % 255;*/

            vertices.insert(std::make_pair(input[i], index));
            outIndices.push_back(index++);
        }
    }

    // Notice that the vertices in the set are not sorted by the index
    // so you'll have to rearrange them like this:
    outVertices.resize(vertices.size());
    for (std::set<std::pair<ExSkelVertexStruct, int>>::iterator it=vertices.begin(); it!=vertices.end(); it++)
        outVertices[it->second] = it->first;
}

/** Computes vertex normals for a mesh with face normals */
void WorldConverter::GenerateVertexNormals(std::vector<ExVertexStruct>& vertices, std::vector<VERTEX_INDEX>& indices)
{
	std::vector<D3DXVECTOR3> normals(vertices.size(), D3DXVECTOR3(0,0,0));

	for(unsigned int i=0;i<indices.size(); i+=3)
	{
		D3DXVECTOR3 v[3] = {*vertices[indices[i]].Position.toD3DXVECTOR3(), *vertices[indices[i+1]].Position.toD3DXVECTOR3(), *vertices[indices[i+2]].Position.toD3DXVECTOR3()};
		D3DXVECTOR3 normal;
		D3DXVec3Cross(&normal, &(v[1] - v[0]), &(v[2] - v[0]));

		for (int j = 0; j < 3; ++j)
		{
			D3DXVECTOR3 a = v[(j+1) % 3] - v[j];
			D3DXVECTOR3 b = v[(j+2) % 3] - v[j];
			float weight = acos(D3DXVec3Dot(&a, &b) / (D3DXVec3Length(&a) * D3DXVec3Length(&b)));
			normals[indices[(i+j)]] += weight * normal;
		}
	}

	// Normalize everything and store it into the vertices
	for(unsigned int i=0;i<normals.size(); i++)
	{
		D3DXVec3Normalize(vertices[i].Normal.toD3DXVECTOR3(), &normals[i]);
	}
}

ExVertexStruct TessTriLerpVertex(ExVertexStruct& a, ExVertexStruct& b, float w)
{
	ExVertexStruct v;
	D3DXVec3Lerp(v.Position.toD3DXVECTOR3(), a.Position.toD3DXVECTOR3(), b.Position.toD3DXVECTOR3(), w);
	D3DXVec3Lerp(v.Normal.toD3DXVECTOR3(), a.Normal.toD3DXVECTOR3(), b.Normal.toD3DXVECTOR3(), w);
	D3DXVec2Lerp(v.TexCoord.toD3DXVECTOR2(), a.TexCoord.toD3DXVECTOR2(), b.TexCoord.toD3DXVECTOR2(), w);
	D3DXVec2Lerp(v.TexCoord2.toD3DXVECTOR2(), a.TexCoord2.toD3DXVECTOR2(), b.TexCoord2.toD3DXVECTOR2(), w);
	v.Color = a.Color;
	return v;
}

/** Outputs 4 new triangles for 1 input triangle */
static void TessSingleTri(ExVertexStruct* tri, std::vector<ExVertexStruct>& tesselated)
{
	ExVertexStruct half[3];
	half[0] = TessTriLerpVertex(tri[0], tri[1], 0.5f);
	half[1] = TessTriLerpVertex(tri[1], tri[2], 0.5f);
	half[2] = TessTriLerpVertex(tri[0], tri[2], 0.5f);

	tesselated.push_back(tri[0]);
	tesselated.push_back(half[0]);
	tesselated.push_back(half[2]);

	tesselated.push_back(half[0]);
	tesselated.push_back(tri[1]);
	tesselated.push_back(half[1]);

	tesselated.push_back(half[2]);
	tesselated.push_back(half[0]);
	tesselated.push_back(half[1]);

	tesselated.push_back(half[2]);
	tesselated.push_back(half[1]);
	tesselated.push_back(tri[2]);
}

/** Tesselates the given triangle and adds the values to the list */
void WorldConverter::TesselateTriangle(ExVertexStruct* tri, std::vector<ExVertexStruct>& tesselated, int amount)
{
	if(amount == 0)
	{
		tesselated.push_back(tri[0]);
		tesselated.push_back(tri[1]);
		tesselated.push_back(tri[2]);
		return;
	}

	std::vector<ExVertexStruct> tv;
	TessSingleTri(tri, tv);

	for(int i=0; i < 4 * 3; i+=3)
	{
		TesselateTriangle(&tv[i], tesselated, amount - 1);
	}
}

/** Marks the edges of the mesh */
void WorldConverter::MarkEdges(std::vector<ExVertexStruct>& vertices, std::vector<VERTEX_INDEX>& indices)
{

}