#pragma once
#include "pch.h"
#include "BaseVertexBuffer.h"
#include "BaseConstantBuffer.h"
#include "ConstantBufferStructs.h"
#include "zTypes.h"

/** Square size of a single world-section */
const float WORLD_SECTION_SIZE = 16000;

const float4 DEFAULT_LIGHTMAP_POLY_COLOR_F = float4(0.85f, 0.85f,0.85f,0.85f);
const DWORD DEFAULT_LIGHTMAP_POLY_COLOR = DEFAULT_LIGHTMAP_POLY_COLOR_F.ToDWORD();
const float3 DEFAULT_INDOOR_VOB_AMBIENT = float3(0.85f, 0.85f, 0.85f);

class zCMaterial;
class zCPolygon;
class BaseVertexBuffer;
class zCVob;
class zCLightmap;
struct zCModelNodeInst;

struct MeshKey
{
	zCMaterial* Material;
	//zCLightmap* Lightmap;
};

struct cmpMeshKey {
    bool operator()(const MeshKey& a, const MeshKey& b) const 
	{
        return (a.Material<b.Material);
    }
};

/*struct MeshKey
{
	zCMaterial* Material;
	zCLightmap* Lightmap;
};

struct cmpMeshKey {
    bool operator()(const MeshKey& a, const MeshKey& b) const 
	{
        return (a.Material<b.Material) || (a.Material == b.Material && a.Lightmap<b.Lightmap);
    }
};*/

/** Holds information about a mesh, ready to be loaded into the renderer */
struct MeshInfo
{
	MeshInfo()
	{
		MeshVertexBuffer = NULL;
		MeshIndexBuffer = NULL;

	}

	~MeshInfo();

	/** Creates buffers for this mesh info */
	XRESULT Create(ExVertexStruct* vertices, unsigned int numVertices, VERTEX_INDEX* indices, unsigned int numIndices);

	BaseVertexBuffer* MeshVertexBuffer;
	BaseVertexBuffer* MeshIndexBuffer;
	std::vector<ExVertexStruct> Vertices;
	std::vector<VERTEX_INDEX> Indices;	
};

/** Holds information about a skeletal mesh */
struct SkeletalMeshInfo
{
	SkeletalMeshInfo()
	{
		MeshVertexBuffer = NULL;
		MeshIndexBuffer = NULL;
	}

	~SkeletalMeshInfo();

	BaseVertexBuffer* MeshVertexBuffer;
	BaseVertexBuffer* MeshIndexBuffer;
	std::vector<ExSkelVertexStruct> Vertices;
	std::vector<VERTEX_INDEX> Indices;
};

/** Holds the converted mesh of a VOB */
class zCProgMeshProto;
struct MeshVisualInfo
{
	MeshVisualInfo()
	{
		Visual = NULL;
		UnloadedSomething = false;
	}

	~MeshVisualInfo()
	{
		for(std::map<zCMaterial *, std::vector<MeshInfo*>>::iterator it = Meshes.begin(); it != Meshes.end(); it++)
		{
			for(unsigned int i=0;i<(*it).second.size();i++)
				delete (*it).second[i];
		}
	}

	/** Starts a new frame for this mesh */
	void StartNewFrame()
	{
		Instances.clear();
	}

	std::map<zCMaterial *, std::vector<MeshInfo*>> Meshes;
	zCProgMeshProto* Visual;
	std::vector<D3DXMATRIX> Instances;

	/** "size" of the mesh. The distance between it's bbox min and bbox max */
	float MeshSize;

	/** Meshes bounding box */
	zTBBox3D BBox;

	/** This is true if we can't actually render something on this. TODO: Try to fix this! */
	bool UnloadedSomething;
};

/** Holds the converted mesh of a VOB */
struct SkeletalMeshVisualInfo
{
	SkeletalMeshVisualInfo()
	{

	}

	~SkeletalMeshVisualInfo()
	{
		for(std::map<zCMaterial *, std::vector<SkeletalMeshInfo*>>::iterator it = Meshes.begin(); it != Meshes.end(); it++)
		{
			for(unsigned int i=0;i<(*it).second.size();i++)
				delete (*it).second[i];
		}
	}

	/** Map of visuals attached to nodes */
	std::map<int, std::vector<MeshVisualInfo *>> NodeAttachments;

	/** Submeshes of this visual */
	std::map<zCMaterial *, std::vector<SkeletalMeshInfo*>> Meshes;
};

struct WorldMeshSectionInfo;
struct VobInfo
{
	VobInfo()
	{
		//Vob = NULL;
		VobConstantBuffer = NULL;
		IsIndoorVob = false;
		VisibleInRenderPass = false;
		VobSection = NULL;
	}

	~VobInfo()
	{
		//delete VisualInfo;
		delete VobConstantBuffer;
	}

	/** Updates the vobs constantbuffer */
	void UpdateVobConstantBuffer();

	/** Visual for this vob */
	MeshVisualInfo* VisualInfo;

	/** Constantbuffer which holds this vobs world matrix */
	BaseConstantBuffer* VobConstantBuffer;

	/** Position the vob was at while being rendered last time */
	D3DXVECTOR3 LastRenderPosition;

	/** Vob the data came from */
	zCVob* Vob;

	/** True if this is an indoor-vob */
	bool IsIndoorVob;

	/** Flag to see if this vob was drawn in the current render pass. Used to collect the same vob only once. */
	bool VisibleInRenderPass;

	/** Section this vob is in */
	WorldMeshSectionInfo* VobSection;
};

class zCVobLight;
struct VobLightInfo
{
	VobLightInfo()
	{
		Vob = NULL;
		VisibleInRenderPass = false;
		IsIndoorVob = false;
	}

	/** Vob the data came from */
	zCVobLight* Vob;

	/** Flag to see if this vob was drawn in the current render pass. Used to collect the same vob only once. */
	bool VisibleInRenderPass;

	/** True if this is an indoor-vob */
	bool IsIndoorVob;
};


/** Holds the converted mesh of a VOB */
struct SkeletalVobInfo
{
	SkeletalVobInfo()
	{
		//Vob = NULL;
	}

	~SkeletalVobInfo()
	{
		//delete VisualInfo;
	}

	/** Visual for this vob */
	SkeletalMeshVisualInfo* VisualInfo;

	/** The Vob the data came from */
	zCVob* Vob;
};

struct SectionInstanceCache
{
	SectionInstanceCache()
	{

	}

	~SectionInstanceCache();

	/** Clears the cache for the given progmesh */
	void ClearCacheForStatic(MeshVisualInfo* pm);

	std::map<MeshVisualInfo *, std::vector<VS_ExConstantBuffer_PerInstance>> InstanceCacheData;
	std::map<MeshVisualInfo *, BaseVertexBuffer*> InstanceCache;

};

/** Describes a world-section for the renderer */
struct WorldMeshSectionInfo
{
	WorldMeshSectionInfo()
	{
		BoundingBox.Min = D3DXVECTOR3(FLT_MAX, FLT_MAX, FLT_MAX);
		BoundingBox.Max = D3DXVECTOR3(FLT_MIN, FLT_MIN, FLT_MIN);
		FullStaticMesh = NULL;
	}

	~WorldMeshSectionInfo()
	{
		for(std::map<MeshKey, MeshInfo*>::iterator it = WorldMeshes.begin(); it != WorldMeshes.end(); it++)
		{
			delete (*it).second;
		}
		WorldMeshes.clear();

		for(std::list<VobInfo *>::iterator it = Vobs.begin(); it != Vobs.end(); it++)
		{
			delete (*it);
		}
		Vobs.clear();

		delete FullStaticMesh;
	}

	/** Saves this sections mesh to a file */
	void SaveSectionMeshToFile(const std::string& name);

	std::map<MeshKey, MeshInfo*, cmpMeshKey> WorldMeshes;
	std::list<VobInfo*> Vobs;

	/** The whole section as one single mesh, without alpha-test materials */
	MeshInfo* FullStaticMesh;

	/** This sections bounding box */
	zTBBox3D BoundingBox;

	SectionInstanceCache InstanceCache;
};

class zCBspTree;
class zCWorld;
struct WorldInfo
{
	WorldInfo()
	{
		BspTree = NULL;
	}

	D3DXVECTOR2 MidPoint;
	float LowestVertex;
	float HighestVertex;
	zCBspTree* BspTree;
	zCWorld* MainWorld;
};

class zCProgMeshProto;
class zCModel;
class zCMesh;
class WorldConverter
{
public:
	WorldConverter(void);
	virtual ~WorldConverter(void);

	/** Converts the worldmesh into a more usable format */
	static HRESULT ConvertWorldMesh(zCPolygon** polys, unsigned int numPolygons, std::map<int, std::map<int, WorldMeshSectionInfo>>* outSections, WorldInfo* info);

	/** Converts a loaded custommesh to be the worldmesh */
	static XRESULT LoadWorldMeshFromFile(const std::string& file, std::map<int, std::map<int, WorldMeshSectionInfo>>* outSections, WorldInfo* info);

	/** Returns what section the given position is in */
	static INT2 GetSectionOfPos(const D3DXVECTOR3& pos);

	/** Converts a world polygon triangle fan to a vertex list */
	static void TriangleFanToList(ExVertexStruct* input, unsigned int numInputVertices, std::vector<ExVertexStruct>* outVertices);

	/** Saves the given section-array to an obj file */
	static void SaveSectionsToObjUnindexed(const char* file, const std::map<int, std::map<int, WorldMeshSectionInfo>>& sections);

	/** Saves the given prog mesh to an obj-file */
	//static void SaveProgMeshToOBj(

	/** Extracts a 3DS-Mesh from a zCVisual */
	static void Extract3DSMeshFromVisual(zCProgMeshProto* visual, MeshVisualInfo* meshInfo);

	/** Extracts a 3DS-Mesh from a zCVisual */
	static void Extract3DSMeshFromVisual2(zCProgMeshProto* visual, MeshVisualInfo* meshInfo);

	/** Extracts a skeletal mesh from a zCModel */
	static void ExtractSkeletalMeshFromVob(zCModel* model, SkeletalMeshVisualInfo* skeletalMeshInfo);
	
	/** Extracts a node-visual */
	static void ExtractNodeVisual(int index, zCModelNodeInst* node, SkeletalMeshVisualInfo* skeletalMeshInfo);

	/** Indexes the given vertex array */
	static void IndexVertices(ExVertexStruct* input, unsigned int numInputVertices, std::vector<ExVertexStruct>& outVertices, std::vector<VERTEX_INDEX>& outIndices);
	static void IndexVertices(ExSkelVertexStruct* input, unsigned int numInputVertices, std::vector<ExSkelVertexStruct>& outVertices, std::vector<VERTEX_INDEX>& outIndices);

	/** Marks the edges of the mesh */
	static void MarkEdges(std::vector<ExVertexStruct>& vertices, std::vector<VERTEX_INDEX>& indices);

	/** Computes vertex normals for a mesh with face normals */
	static void GenerateVertexNormals(std::vector<ExVertexStruct>& vertices, std::vector<VERTEX_INDEX>& indices);

	/** Creates the FullSectionMesh for the given section */
	static void GenerateFullSectionMesh(WorldMeshSectionInfo& section);

	/** Tesselates the given triangle and adds the values to the list */
	static void TesselateTriangle(ExVertexStruct* tri, std::vector<ExVertexStruct>& tesselated, int amount);
};

