#pragma once
#include "pch.h"
#include "HookedFunctions.h"
#include "zCPolygon.h"
#include "Engine.h"
#include "GothicAPI.h"
#include "zSTRING.h"
#include "zCArray.h"
#include "zTypes.h"
#include "zCWorld.h"
#include "zCCamera.h"
#include "zCModelTexAniState.h"

class zCVisual;
class zCMeshSoftSkin;

struct zCModelNode;
struct zCModelNodeInst
{
	zCModelNodeInst* ParentNode;
	zCModelNode* ProtoNode;
	zCVisual* NodeVisual;
	D3DXMATRIX Trafo;			
	D3DXMATRIX TrafoObjToCam;	
	zTBBox3D BBox3D;

	zCModelTexAniState TexAniState;
};



struct zCModelNode
{
	zCModelNode* ParentNode;
	zSTRING	NodeName;
	zCVisual* Visual;
	D3DXMATRIX Transform;

	D3DXVECTOR3 NodeRotAxis;
	float NodeRotAngle;
	D3DXVECTOR3	Translation;
	D3DXMATRIX TransformObjToWorld;
	D3DXMATRIX* NodeTransformList;
	zCModelNodeInst* LastInstNode;
};

struct zTMdl_NodeVobAttachment
{
	zCVob* Vob;
	zCModelNodeInst* NodeInst;
};

struct zTMeshLibEntry
{				
	zCModelTexAniState	TexAniState;
	int MeshLibPtr;
};

class zCModelPrototype
{
public:
	zCArray<zCModelNode *>* GetNodeList()
	{
		return (zCArray<zCModelNode *> *)THISPTR_OFFSET(GothicMemoryLocations::zCModelPrototype::Offset_NodeList);
	}
};



class zCModel
{
public:
	/** Creates an array of matrices for the bone transforms */
	void __fastcall RenderNodeList( zTRenderContext& renderContext, zCArray<D3DXMATRIX*>& boneTransforms, zCRenderLightContainer& lightContainer, int lightingMode = 0)
	{
		XCALL(GothicMemoryLocations::zCModel::RenderNodeList);
	}

	zCArray<zCModelNodeInst *>* GetNodeList()
	{
		return (zCArray<zCModelNodeInst *> *)THISPTR_OFFSET(GothicMemoryLocations::zCModel::Offset_NodeList);
	}

	zCArray<zCMeshSoftSkin *>* GetMeshSoftSkinList()
	{
		return (zCArray<zCMeshSoftSkin *> *)THISPTR_OFFSET(GothicMemoryLocations::zCModel::Offset_MeshSoftSkinList);
	}

	zCArray<zCModelPrototype *>* GetModelProtoList()
	{
		return (zCArray<zCModelPrototype *> *)THISPTR_OFFSET(GothicMemoryLocations::zCModel::Offset_ModelProtoList);
	}

	zCArray<zTMeshLibEntry *>* GetMeshLibList()
	{
		return (zCArray<zTMeshLibEntry *> *)THISPTR_OFFSET(GothicMemoryLocations::zCModel::Offset_MeshLibList);
	}

	zCArray<zTMdl_NodeVobAttachment>* GetAttachedVobList()
	{
		return (zCArray<zTMdl_NodeVobAttachment> *)THISPTR_OFFSET(GothicMemoryLocations::zCModel::Offset_AttachedVobList); 
	}

	/* Updates the world matrices of the attached VOBs */
	void UpdateAttachedVobs()
	{
		XCALL(GothicMemoryLocations::zCModel::UpdateAttachedVobs); 
	}

	/** Fills a vector of (viewspace) bone-transformation matrices for this frame */
	void GetBoneTransforms(std::vector<D3DXMATRIX>* transforms, zCVob* vob)
	{
		if(!GetNodeList())
			return;

		std::vector<D3DXMATRIX*> tptr;
		tptr.resize(GetNodeList()->NumInArray);
		for (int i=0; i<GetNodeList()->NumInArray; i++) 
		{
			zCModelNodeInst* node = GetNodeList()->Array[i];
			zCModelNodeInst* parent = node->ParentNode;
			tptr[i] = &node->TrafoObjToCam;

			// Calculate transform for this node
			if(parent)	
			{
				node->TrafoObjToCam = parent->TrafoObjToCam * node->Trafo;
			}else
			{
				node->TrafoObjToCam = node->Trafo;
			}
		
		}
		// Put them into our vector
		for(unsigned int i=0;i<tptr.size();i++)
		{
			transforms->push_back((*tptr[i]));
		}
	}
};